//
//  IAPLocalStorage.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/10.
//

#import "IAPLocalStorage.h"
#import <UIKit/UIKit.h>

@interface IAPLocalStorage ()
@property (nonatomic, strong) dispatch_queue_t queue;
@property (nonatomic, strong) NSCache *memoryCache;
@property (nonatomic, strong) NSUserDefaults *diskCache;

@property (nonatomic, strong) NSMutableArray<NSString *> *preloadDataKeys;
@end

@implementation IAPLocalStorage

+ (instancetype)shared {
    static IAPLocalStorage *instance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        instance = [self new];
        [instance config];
    });
    return instance;
}

- (void)config {
    self.queue = dispatch_queue_create("IAPLocalStorage.queue", DISPATCH_QUEUE_SERIAL);
    self.memoryCache = [[NSCache alloc] init];
    self.memoryCache.totalCostLimit = 10485760;
    self.diskCache =  [[NSUserDefaults alloc] initWithSuiteName:@"IAPLocalStorage"];
    self.preloadDataKeys = [(id)[self codingObjectForKey:@"IAPLocalStorage.preLoadKeys"] ?: @[@"IAPLocalStorage.last.selected.wallet"] mutableCopy];
}

- (void)preload {
    dispatch_async(self.queue, ^{
        NSArray<NSString *> *preloadDataKeys = self.preloadDataKeys.copy;
        [preloadDataKeys enumerateObjectsUsingBlock:^(NSString * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            [self dataForKey:obj];
        }];
    });
}

- (void)addPreloadKeys:(NSArray<NSString *> *)keys {
    if (!keys.count) {
        return;
    }
    
    dispatch_async(self.queue, ^{
        [keys enumerateObjectsUsingBlock:^(NSString * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            if (self.preloadDataKeys.count == 4) {
                [self.preloadDataKeys removeLastObject];
                [self.preloadDataKeys insertObject:obj atIndex:2];
            } else {
                [self.preloadDataKeys addObject:obj];
            }
        }];
        [self setCodingObject:self.preloadDataKeys forKey:@"IAPLocalStorage.preLoadKeys"];
    });
}

- (BOOL)boolForKey:(NSString *)key {
    if (!key.length) {
        return NO;
    }
    
    return [[self diskCache] boolForKey:key];
}

- (void)setBool:(BOOL)value forKey:(NSString *)key {
    if (!key.length) {
        return;
    }
    [[self diskCache] setBool:value forKey:key];
}


- (nullable NSString *)stringForKey:(NSString *)key {
    if (!key.length) {
        return nil;
    }
    
    NSString *value = [[self diskCache] objectForKey:key];
    if (![value isKindOfClass:NSString.class]) {
        return nil;
    }
    
    return value;
}

- (void)setString:(nullable NSString *)value forKey:(NSString *)key {
    if (!key.length) {
        return;
    }
    
    if (value && ![value isKindOfClass:NSString.class]) {
        return;
    }
    
    [[self diskCache] setObject:value forKey:key];
}


- (nullable id<NSCoding>)codingObjectForKey:(NSString *)key {
    if (!key.length) {
        return nil;
    }
    
    id object = [self memoryCodingObjectForKey:key];
    if (object) {
        return object;
    }
    
    object = [self diskCodingObjectForKey:key];
    if (object) {
        [[self memoryCache] setObject:object forKey:key];
    }
    return object;
}

- (void)setCodingObject:(nullable id<NSCoding>)object forKey:(NSString *)key {
    if (!key.length) {
        return;
    }
    
    if (!object) {
        [[self memoryCache] removeObjectForKey:key];
        [[self diskCache] setObject:nil forKey:key];
        return;
    }
    
    if (![self isConfirmedNSCoding:object]) {
        return;
    }
    
    NSData *data = nil;
    @try {
        data = [NSKeyedArchiver archivedDataWithRootObject:object];
    } @catch (NSException *exception) {
        NSLog(@"");
    }
    
    if (data) {
        [[self memoryCache] setObject:object forKey:key];
        [[self diskCache] setObject:data forKey:key];
    }
}

- (nullable NSData *)memoryDataForKey:(NSString *)key {
    if (!key.length) {
        return nil;
    }
    
    NSData *data = [[self memoryCache] objectForKey:key];
    if ([data isKindOfClass:NSData.class]) {
        return data;
    }
    
    return nil;
}


- (nullable NSData *)dataForKey:(NSString *)key {
    if (!key.length) {
        return nil;
    }
    
    NSData *data = [self memoryDataForKey:key];
    if (data) {
        return data;
    }
    
    data = [[self diskCache] dataForKey:key];
    if (data) {
        [[self memoryCache] setObject:data forKey:key];
    }
    return data;
}

- (void)setData:(nullable NSData *)data forKey:(NSString *)key {
    if (!key.length) {
        return;
    }
    
    if (!data) {
        [[self memoryCache] removeObjectForKey:key];
        [[self diskCache] setObject:nil forKey:key];
        return;
    }
    
    if (![data isKindOfClass:NSData.class]) {
        return;
    }
    
    [[self memoryCache] setObject:data forKey:key];
    [[self diskCache] setObject:data forKey:key];
}

- (void)asyncGetDataForKey:(NSString *)key completion:(void(^)(NSData * _Nullable data, BOOL isSyncData))completion {
    if (!completion) {
        return;
    }
    
    NSData *data = [[IAPLocalStorage shared] memoryDataForKey:key];
    if (data) {
        completion(data, YES);
        return;
    } 
    
    dispatch_async([self queue], ^{
        NSData *data = [self dataForKey:key];
        completion(data, NO);
    });
}

- (void)asynSetData:(nullable NSData *)data forKey:(NSString *)key completion:(nullable void(^)(void))completion {
    dispatch_async([self queue], ^{
        [self setData:data forKey:key];
        if (completion) {
            completion();
        }
    });
}

#pragma mark private

- (BOOL)isConfirmedNSCoding:(id)object {
    return ([(NSObject *)object respondsToSelector:@selector(initWithCoder:)]
            && [(NSObject *)object respondsToSelector:@selector(encodeWithCoder:)]);
}

- (id<NSCoding>)memoryCodingObjectForKey:(NSString *)key {
    return [[self memoryCache] objectForKey:key];
}

- (id<NSCoding>)diskCodingObjectForKey:(NSString *)key {
    NSData *data = [[self diskCache] dataForKey:key];
    if (!data) {
        return nil;
    }
    
    id object = nil;
    @try {
        object = [NSKeyedUnarchiver unarchiveObjectWithData:data];
    } @catch (NSException *exception) {
        NSLog(@"");
    }
    
    return object;
}

@end
