//
//  IAPInquirePaymentOptionSponsor.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/16.
//

#import "IAPInquirePaymentOptionSponsor.h"
#import "IAPRequestUtils.h"
#import "NSString+IAPUtils.h"
#import "IAPJSONUtils.h"
#import "IAPLocalStorage.h"
#import "IAPPaymentOption.h"
#import "IAPInquirePaymentOptionParams.h"
#import "AlipayPlusClientAPI.h"
#import "IAPPaymentOption+Private.h"
#import "NSError+IAPUtils.h"
#import "AlipayPlusClientAPI+Private.h"
#import <AlipayPlusClient/IAPConstant.h>

NSString * const IAPInquireProductCashierPayment = @"CASHIER_PAYMENT";

@implementation IAPInquirePaymentOptionSponsor
+ (void)inquirePaymentOptionWithParams:(IAPInquirePaymentOptionParams *)params
                     completionHandler:(void(^)(IAPPaymentOption * _Nullable paymentOption, NSError * _Nullable error))completionHandler {
    if (!completionHandler) {
        return;
    }
    
    if (![AlipayPlusClient shared].configuration.acquirerId.length || ![AlipayPlusClient shared].configuration.merchantId.length) {
        NSError *resultError = [NSError iapu_paramsIllegalErrorWithExtraMessage:@"acquirerId or merchantId in configuration is nil"];
        [self executeInquirePaymentOptionCallback:completionHandler paymentOption:nil error:resultError];

        [[AlipayPlusClient shared] logWithName:@"inquirePaymentOptionError" parameter:@{@"message":resultError.localizedDescription ?: @"", @"code": @(resultError.code).stringValue}];
        return;
    }
    
    if (!params.paymentCurrency.length) {
        NSError *resultError = [NSError iapu_paramsIllegalErrorWithExtraMessage:@"paymentCurrency is nil"];
        [self executeInquirePaymentOptionCallback:completionHandler paymentOption:nil error:resultError];
        [[AlipayPlusClient shared] logWithName:@"inquirePaymentOptionError" parameter:@{@"message":resultError.localizedDescription ?: @"", @"code": @(resultError.code).stringValue}];
        return;
    }
    
    
    NSString *acquirerId = [AlipayPlusClient shared].configuration.acquirerId;
    NSString *merchantId = [AlipayPlusClient shared].configuration.merchantId;
    NSString *currency = params.paymentCurrency;
    NSString *userRegion = params.userRegion.length ? params.userRegion : @"defaultregion";
    NSString *logoPattern = params.logoPattern.length ? params.logoPattern : @"defaultlogo";
    NSMutableURLRequest *request = [self cdnRequestWithAcquirerId:acquirerId merchantId:merchantId currency:currency userRegion:userRegion logoPattern:logoPattern];
    
    if (!request.URL) {
        NSError *resultError = [NSError iapu_paramsIllegalErrorWithExtraMessage:@"acquirerId or merchantId is invalid"];
        [self executeInquirePaymentOptionCallback:completionHandler paymentOption:nil error:resultError];
        [[AlipayPlusClient shared] logWithName:@"inquirePaymentOptionError" parameter:@{@"message":resultError.localizedDescription ?: @"", @"code": @(resultError.code).stringValue}];
        return;
    }
    
    request.cachePolicy = NSURLRequestReloadIgnoringLocalCacheData;
    [[IAPRequestUtils shared] asyncSendRequest:request timeoutInterval:5 completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
        IAPPaymentOption *resultOption = nil;
        NSString *key = [NSString stringWithFormat:@"IAPPaymentOptionCache_%@_%@_%@_%@_%@_%@", IAPInquireProductCashierPayment, acquirerId, merchantId, params.paymentCurrency, userRegion, logoPattern];
        if (error) {
            resultOption = (IAPPaymentOption *)[IAPLocalStorage.shared codingObjectForKey:key];
            if (resultOption) {
                [resultOption markFromCache];
                [self executeInquirePaymentOptionCallback:completionHandler paymentOption:resultOption error:nil];
                [[AlipayPlusClient shared] logWithName:@"callback paymentOption use cache" parameter:nil];
            } else {
                NSError *resultError = [NSError iapu_invalidNetworkErrorWithExtraMessage:[NSString stringWithFormat:@"originalErrorCode:%ld", error.code]];
                [[AlipayPlusClient shared] logWithName:@"inquirePaymentOptionError" parameter:@{@"message":resultError.localizedDescription ?: @"", @"code": @(resultError.code).stringValue}];
                
                [self executeInquirePaymentOptionCallback:completionHandler paymentOption:nil error:resultError];
            }
        } else {
            NSDictionary *paymentOptionJSON = [IAPJSONUtils JSONDictionaryFromData:data];
            
            // mock cdn
            if ([paymentOptionJSON[@"jsonUrl"] isKindOfClass:NSDictionary.class]) {
                paymentOptionJSON = paymentOptionJSON[@"jsonUrl"];
            }
            
            if (!paymentOptionJSON.count) {
                paymentOptionJSON = @{@"disableReason": @"NO_PAYMENT_OPTIONS", @"paymentMethodType": @"CONNECT_WALLET"};
            }
            resultOption = [[IAPPaymentOption alloc] initWithJSONDictionary:paymentOptionJSON currency:params.paymentCurrency];
            [IAPLocalStorage.shared setCodingObject:resultOption forKey:key];
            [self executeInquirePaymentOptionCallback:completionHandler paymentOption:resultOption error:nil];
        }
    }];
}

#pragma mark- private

+ (void)executeInquirePaymentOptionCallback:(void(^)(IAPPaymentOption *paymentOption, NSError *error))callback paymentOption:(IAPPaymentOption *)option error:(NSError *)error {
    dispatch_async(dispatch_get_main_queue(), ^{
        callback(option, error);
    });
}

+ (NSMutableURLRequest *)cdnRequestWithAcquirerId:(NSString *)acquirerId
                                       merchantId:(NSString *)merchantId
                                         currency:(NSString *)currency
                                       userRegion:(NSString *)userRegion
                                      logoPattern:(NSString *)logoPattern
{
    NSMutableURLRequest *request = [NSMutableURLRequest new];
    if (![[AlipayPlusClient shared].configuration.envType isEqualToString:kIAPEnvSANDBOX]) {
        request = [self prodCdnRequestWithAcquirerId:acquirerId merchantId:merchantId currency:currency userRegion:userRegion logoPattern:logoPattern];
    } else {
        request = [self sandboxCdnRequestWithAcquirerId:acquirerId merchantId:merchantId currency:currency userRegion:userRegion logoPattern:logoPattern];
    }
    return request;
}

+ (NSMutableURLRequest *)prodCdnRequestWithAcquirerId:(NSString *)acquirerId
                                           merchantId:(NSString *)merchantId
                                             currency:(NSString *)currency
                                           userRegion:(NSString *)userRegion
                                          logoPattern:(NSString *)logoPattern
{
    NSMutableURLRequest *request = [NSMutableURLRequest new];
    request.URL = [NSURL URLWithString:[NSString stringWithFormat:@"https://resource.alipayplus.com/prod/%@/%@/%@/%@/%@/%@/alipayplus.json?version=%@&type=IOS", acquirerId, merchantId.iapu_SHA256, IAPInquireProductCashierPayment, currency, userRegion, logoPattern, IAPAlipayPlusClientVersionString]];
    [[AlipayPlusClient shared] logWithName:@"paymentOption env: PROD" parameter:nil];
    return request;
}

+ (NSMutableURLRequest *)sandboxCdnRequestWithAcquirerId:(NSString *)acquirerId
                                              merchantId:(NSString *)merchantId
                                                currency:(NSString *)currency
                                              userRegion:(NSString *)userRegion
                                             logoPattern:(NSString *)logoPattern
{
    NSMutableURLRequest *request = [NSMutableURLRequest new];
    NSString *URLString = nil;
    NSString *sandboxInnerEnv = [[NSUserDefaults standardUserDefaults] objectForKey:@"AlipayPlusClient_sandbox_inner_env"];
    
    BOOL isSANDBOX_DEV = NO;
    if ([sandboxInnerEnv isEqualToString:@"SANDBOX_SIT"]) {
        URLString = @"http:imgs-sggz00a-1.test.alipay.net/imgw.htm";
    } else if ([sandboxInnerEnv isEqualToString:@"SANDBOX_PROD"]) {
        URLString = @"https://imgs-sea.alipay.com/mgw.htm";
    } else {
        sandboxInnerEnv = @"SANDBOX_DEV";
        URLString = @"http://imgs-ac-saas.dl.alipaydev.com/mgw.htm";
        isSANDBOX_DEV = YES;
    }
    
    [[AlipayPlusClient shared] logWithName:[NSString stringWithFormat:@"paymentOption env: SANDBOX-%@", sandboxInnerEnv] parameter:nil];
    
    request.URL = [NSURL URLWithString:URLString];
    request.HTTPMethod = @"POST";
    
    NSMutableDictionary *headers = @{
        @"appId" : @"SIMULATION_ACQUIRER_AC",
        @"workspaceId" : @"default",
        @"operation-type" : @"ac.simulation.wallet.post",
        @"inner_operation_type" : @"ac.testacquirer.mock.cdn",
        @"Content-Type" : @"application/json",
        @"load-test": @"1"
    }.mutableCopy;
    if (isSANDBOX_DEV) {
        NSString *sandboxDevSofa = [[NSUserDefaults standardUserDefaults] objectForKey:@"AlipayPlusClient_sandbox_inner_dev_sofa-group-name"] ?: @"GROUP_20220317224203";
        [headers setObject:sandboxDevSofa forKey:@"sofa-group-name"];
    }
    [request setAllHTTPHeaderFields:headers];
    
    NSArray *dataArray = @[@{
        @"operationType": @"ac.testacquirer.mock.cdn",
        @"pspId": @"xxx",
        @"requestData": @{
            @"mid":merchantId ?: @"000",
            @"acquirerId": acquirerId?:@"000",
            @"product": IAPInquireProductCashierPayment,
            @"currency":currency?:@"",
            @"region":userRegion?:@"",
            @"logoPattern":logoPattern
        },
        @"envInfo": @{
            @"extendInfo": @{
                @"instanceId": @"xxx"
            }
        }
    }];
    request.HTTPBody = [NSJSONSerialization dataWithJSONObject:dataArray options:0 error:NULL];
    return request;
}

@end
