//
//  IAPPaymentData.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/19.
//

#import "IAPPaymentData.h"
#import "NSDictionary+IAPUtils.h"
#import "IAPLogo+Private.h"
#import "IAPJSONUtils.h"
#import "IAPMultiLanguageUtils.h"
#import "IAPLocalStorage.h"
#import "IAPRequestUtils.h"
#import "IAPStyleUtils.h"
#import "AlipayPlusClientAPI.h"

@interface IAPPaymentWalletRedirectionInfo ()
@property (nonatomic, strong) NSString *codeValue;
@end
@implementation IAPPaymentWalletRedirectionInfo

- (BOOL)canOpenUrl {
    return self.schemeUrl.length || self.applinkUrl.length ||  self.normalUrl.length ||  self.orderInfo.length;
}

// for testCNWallet while integration testing
- (BOOL)isTestCNWallet {
    return [self.schemeUrl containsString:@"pspName=ALIPAY_CN"] ||
           [self.applinkUrl containsString:@"pspName=ALIPAY_CN"] ||
           [self.normalUrl containsString:@"pspName=ALIPAY_CN"];
}

- (void)openUrl {
    BOOL success = NO;
    if (self.schemeUrl.length) {
        success = [[UIApplication sharedApplication] openURL:[NSURL URLWithString:self.schemeUrl]];
    }
    
    if (success) {
        return;
    }
    
    if (self.applinkUrl) {
        [[UIApplication sharedApplication] openURL:[NSURL URLWithString:self.applinkUrl]];
    } else {
        [[UIApplication sharedApplication] openURL:[NSURL URLWithString:self.normalUrl]];
    }
}

- (nullable instancetype)initWithJSONDictionary:(NSDictionary *)json codeValue:(NSString *)codeValue {
    if (![json isKindOfClass:NSDictionary.class] || !codeValue.length) {
        return nil;
    }
    
    if (self = [super init]) {
        _codeValue = codeValue;
        
        _schemeUrl = [json iapu_stringForKey:@"schemeUrl"];
        _applinkUrl = [json iapu_stringForKey:@"applinkUrl"];
        _normalUrl = [json iapu_stringForKey:@"normalUrl"];
        _orderInfo = [json iapu_stringForKey:@"orderInfo"];
        
        [self representCodeValue];
        
        if ([[AlipayPlusClient shared].configuration.envType isEqualToString:kIAPEnvSANDBOX]) {
            [self appendScheme];
        }
    }
    return self;
}

- (void)representCodeValue {
    _schemeUrl = [_schemeUrl stringByReplacingOccurrencesOfString:@"${codeValue}" withString:_codeValue];
    _applinkUrl = [_applinkUrl stringByReplacingOccurrencesOfString:@"${codeValue}" withString:_codeValue];
    _normalUrl = [_normalUrl stringByReplacingOccurrencesOfString:@"${codeValue}" withString:_codeValue];
}

- (void)appendScheme {
    NSString *fromScheme = [AlipayPlusClient shared].configuration.fromScheme;
    if (fromScheme) {
        _schemeUrl = [self appendSchemeQueryCompoment:_schemeUrl scheme:fromScheme];
        _applinkUrl = [self appendSchemeQueryCompoment:_applinkUrl scheme:fromScheme];
        _normalUrl = [self appendSchemeQueryCompoment:_normalUrl scheme:fromScheme];
    }
}

#pragma mark - private
- (NSString *)appendSchemeQueryCompoment:(NSString *)url scheme:(NSString *)scheme {
    NSString *appendString = [@"&fromScheme=" stringByAppendingString:scheme];
    if ([url containsString:@"pspName=ALIPAY_CN"]) {
        return [url stringByAppendingString:appendString];
    } else {
        return url;
    }
}

@end


@interface IAPPaymentWalletData ()
@property (nonatomic, strong) NSString *originalDisableReasonKey;
@property (nonatomic, strong) NSArray<NSString *> *originalServerPromoNames;
@property (nonatomic, strong) NSDictionary<NSString *, NSString *> *reasonTemplate;
@end
@implementation IAPPaymentWalletData

- (nullable instancetype)initWithJSONDictionary:(NSDictionary *)json reasonTemplate:(NSDictionary<NSString *, NSString *> *)reasonTemplate codeValue:(NSString *)codeValue {
    if (![json isKindOfClass:NSDictionary.class]) {
        return nil;
    }
    
    if (self = [super init]) {
        _reasonTemplate = reasonTemplate;
        
        _walletName = [json iapu_stringForKey:@"walletName"];
        _walletLogo = [[IAPLogo alloc] initWithJSONDictionary:[json iapu_dictionaryForKey:@"walletLogo"]];
        _walletBrandName = [json iapu_stringForKey:@"walletBrandName"];
        _enabled = [json iapu_boolForKey:@"enabled"];
        self.originalDisableReasonKey = [json iapu_stringForKey:@"disableReason"];
        _searchKeywords = [json iapu_arrayForKey:@"searchKeywords"];
        _redirectionInfo = [[IAPPaymentWalletRedirectionInfo alloc] initWithJSONDictionary:[json iapu_dictionaryForKey:@"redirectionInfo"] codeValue:codeValue];
        self.originalServerPromoNames = [json iapu_arrayForKey:@"promoNames"];
        
    }
    return self;
}

- (void)bizAsyncFetchLogoWithCallback:(void(^)(UIImage * _Nullable logo, BOOL isPlaceHolder))callback {
    if (!callback) {
        return;
    }
    
    __block UIImage *image = nil;
    [IAPLocalStorage.shared asyncGetDataForKey:self.walletLogo.logoUrl completion:^(NSData * _Nullable imageData, BOOL isSyncData) {
        if (imageData) {
            image = [[UIImage alloc] initWithData:imageData];
        }
        
        if (image) {
            if (isSyncData) {
                callback(image, NO);
            } else {
                dispatch_async(dispatch_get_main_queue(), ^{
                    callback(image, NO);
                });
            }
            return;
        }
        
        if (self.walletLogo.logoName.length) {
            image = [IAPStyleUtils bundleImageNamed:self.walletLogo.logoName];
            if (image) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    callback(image, YES);
                });
            }
        }
        
        [[IAPRequestUtils shared] asyncDownloadCDNFileWithURLString:self.walletLogo.logoUrl timeoutInterval:5 completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
            if (!error && data) {
                image = [[UIImage alloc] initWithData:data];
                if (image) {
                    callback(image, NO);
                    [IAPLocalStorage.shared asynSetData:data forKey:self.walletLogo.logoUrl completion:nil];
                } else {
                    callback(nil, NO);
                }
            } else {
                callback(nil, NO);
            }
        }];
    }];
    
}

#pragma mark - private

- (void)setOriginalDisableReasonKey:(NSString *)originalDisableReasonKey {
    _originalDisableReasonKey = originalDisableReasonKey;
    NSString *disableReason = nil;
    if (originalDisableReasonKey.length) {
        disableReason = _reasonTemplate[originalDisableReasonKey];
    }
    _disableReason = disableReason;
}

- (void)setOriginalServerPromoNames:(NSArray<NSString *> *)originalServerPromoNames {
    _originalServerPromoNames = originalServerPromoNames;
    _promoNames = [IAPMultiLanguageUtils matchedLanguageTextsWithJSONStringArray:originalServerPromoNames];
}

@end


@interface IAPPaymentData ()
@property (nonatomic, strong) NSDictionary<NSString *, NSString *> *originalReasonTemplate;
@property (nonatomic, strong) NSArray<NSDictionary *> *originalWallets;
@end
@implementation IAPPaymentData

- (nullable instancetype)initWithJSONDictionary:(NSDictionary *)json {
    if (![json isKindOfClass:NSDictionary.class]) {
        return nil;
    }
    
    if (self = [super init]) {
        _displayPaymentAmount = [json iapu_stringForKey:@"displayPaymentAmount"];
        _displayPaymentCurrency = [json iapu_stringForKey:@"displayPaymentCurrency"];
        _codeValue = [json iapu_stringForKey:@"codeValue"];
        self.originalReasonTemplate = [json iapu_dictionaryForKey:@"reasonTemplate"];
        self.originalWallets = [json iapu_arrayForKey:@"wallets"];
        
    }
    return self;
}

#pragma mark - private

- (void)setOriginalReasonTemplate:(NSDictionary<NSString *,NSString *> *)originalReasonTemplate {
    _originalReasonTemplate = originalReasonTemplate;
    NSMutableDictionary *reasonTemplate = [NSMutableDictionary dictionaryWithCapacity:originalReasonTemplate.count];
    [originalReasonTemplate enumerateKeysAndObjectsUsingBlock:^(NSString * _Nonnull key, NSString * _Nonnull obj, BOOL * _Nonnull stop) {
        reasonTemplate[key] = [IAPMultiLanguageUtils matchedLanguageTextWithJSONString:obj];
    }];
    _reasonTemplate = reasonTemplate.copy;
}

- (void)setOriginalWallets:(NSArray<NSDictionary *> *)originalWallets {
    _originalWallets = originalWallets;
    NSMutableArray<IAPPaymentWalletData *> *wallets = [NSMutableArray arrayWithCapacity:originalWallets.count];
    [originalWallets enumerateObjectsUsingBlock:^(NSDictionary * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        IAPPaymentWalletData *walletData = [[IAPPaymentWalletData alloc] initWithJSONDictionary:obj reasonTemplate:self.reasonTemplate codeValue:self.codeValue];
        if (walletData) {
            if ((walletData.enabled && walletData.redirectionInfo.canOpenUrl) || !walletData.enabled) {
                [wallets addObject:walletData];
            }
        }
    }];
    
    _wallets = wallets.copy;
}

- (BOOL)hasAvailableWallets {
    __block  BOOL has = NO;
    [self.wallets enumerateObjectsUsingBlock:^(IAPPaymentWalletData * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        has = (has || obj.enabled);
        if (has) {
            *stop = YES;
        }
    }];
    return has;
}
@end
