//
//  IAPPaymentSheetView.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/20.
//

#import "IAPPaymentSheetView.h"
#import "IAPPaymentSheetCheckoutTopView.h"
#import "IAPPaymentSheetContentView.h"
#import "UIView+IAPUtils.h"
#import "UIColor+IAPUtils.h"
#import "IAPStyleUtils+PaymentSheet.h"
#import "IAPMultiLanguageUtils+PaymentSheet.h"
#import "IAPInnerDefines.h"
#import "IAPJSONUtils.h"
#import "NSDictionary+IAPUtils.h"
#import "IAPPaymentData.h"
#import "IAPPaymentSheetEvent+Private.h"
#import "AlipayPlusClientAPI+Private.h"
#import "IAPLocalStorage.h"
#import "IAPCommonWalletAction.h"
#import "IAPAlipayCNWalletAction.h"


@interface IAPPaymentSheetView ()
@property (nonatomic, strong) UIView *curtainView;
@property (nonatomic, strong) IAPPaymentSheetContentView *contentView;
@property (nonatomic, strong) IAPPaymentData *paymentData;
@property (nonatomic, copy) void(^callback)(IAPPaymentSheetEvent *event);
@property (nonatomic, strong) UILabel *toastLabel;

@property (nonatomic, assign) UIInterfaceOrientation interfaceOrientation;
@end

@implementation IAPPaymentSheetView
+ (void)showWithPaymentData:(IAPPaymentData *)paymentData
                   callback:(void (^)(IAPPaymentSheetEvent * _Nonnull))callback {
    if (!callback) {
        return;
    }
        
    if (!paymentData.codeValue.length ||
        !paymentData.displayPaymentAmount.length ||
        !paymentData.displayPaymentCurrency.length) {
        IAPPaymentSheetEvent *event = [IAPPaymentSheetEvent eventWithName:IAPPaymentSheetEventThrowException
                                                                  message:@"PARAM_ILLEGAL"];
        callback(event);
        [[AlipayPlusClient shared] logWithName:@"paymentSheetCallback"
                                     parameter:@{@"message":event.message ?: @""}];
        return;
    }
    
    IAPPaymentSheetView *sheetView = [[self alloc] initWithFrame:[UIScreen mainScreen].bounds
                                                     paymentData:paymentData];
    sheetView.callback = callback;
    
    [sheetView showAnimated:YES];
}


# pragma mark private

- (instancetype)initWithFrame:(CGRect)frame paymentData:(IAPPaymentData *)paymentData {
    if (self = [super initWithFrame:frame]) {
        _paymentData = paymentData;
        _interfaceOrientation = [UIApplication sharedApplication].statusBarOrientation;
        [self initSubviews];
    }
    return self;
}

- (void)showAnimated:(BOOL)animated {
    self.curtainView.alpha = 0;
    self.contentView.iapu_top = self.iapu_height;
    
    [[UIApplication sharedApplication].keyWindow addSubview:self];
    [UIView animateWithDuration:(animated ? 0.3 : 0) animations:^{
        self.curtainView.alpha = 1;
        self.contentView.iapu_bottom = self.iapu_height;
    } completion:^(BOOL finished) {
        IAPPaymentSheetEvent *event = [IAPPaymentSheetEvent eventWithName:IAPPaymentSheetEventDidShow message:@""];
        [self executeCallback:event];
        
        [self.contentView preload];
        
        NSMutableArray<IAPPaymentWalletData *> *dataSourceWallets = [self.contentView.walletTableView.dataSourceWallets mutableCopy];
        NSMutableArray *preloadKeys = [NSMutableArray arrayWithCapacity:3];
        [dataSourceWallets enumerateObjectsUsingBlock:^(IAPPaymentWalletData * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            if (obj.walletLogo.logoUrl) {
                [preloadKeys addObject:obj.walletLogo.logoUrl];
            }
        }];
        [[IAPLocalStorage shared] addPreloadKeys:preloadKeys];
        
    }];
}

- (void)close:(BOOL)userCancel {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [UIView animateWithDuration:0.3 animations:^{
        self.curtainView.alpha = 0;
        self.contentView.iapu_top = self.iapu_height;
    } completion:^(BOOL finished) {
        IAPPaymentSheetEvent *event = [IAPPaymentSheetEvent eventWithName:userCancel ? IAPPaymentSheetEventUserDidCancel : IAPPaymentSheetEventDidSelectWalletAndPay message:@""];
        [self executeCallback:event];
        
        [self removeFromSuperview];
    }];
}

- (void)initSubviews {
    self.curtainView = [[UIView alloc] initWithFrame:self.bounds];
    [self addSubview:self.curtainView];
    self.curtainView.backgroundColor = [IAPStyleUtils sheetCurtainColor];
    @iapu_weakify(self);
    [self.curtainView iapu_touchUpInside:^{
        @iapu_strongify(self);
        [self.contentView.searchView endEditing:YES];
    }];
    
    
    self.contentView = [[IAPPaymentSheetContentView alloc] initWithFrame:CGRectMake(0, 0, self.iapu_width, 427) paymentData:self.paymentData interfaceOrientation:self.interfaceOrientation];
    [self addSubview:self.contentView];
    self.contentView.checkoutTopView.closeAction = ^{
        @iapu_strongify(self);
        [self close:YES];
    };
    [self.contentView.operationView setGoToPayAction:^{
        @iapu_strongify(self);
        [self clickGoToPay];
    }];
    [self.contentView.operationView setReturnToMerchantAction:^{
        @iapu_strongify(self);
        [self close:YES];
    }];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(orientationChanged:) name:UIApplicationWillChangeStatusBarOrientationNotification object:nil];
}


- (void)clickGoToPay {
    if (self.contentView.walletTableView.selectedWallet) {
        [[IAPLocalStorage shared] setData:[self.contentView.walletTableView.selectedWalletName dataUsingEncoding:NSUTF8StringEncoding] forKey:@"IAPLocalStorage.last.selected.wallet"];
        [[AlipayPlusClient shared] payWithWallet:self.contentView.walletTableView.selectedWallet callback:^(IAPPaymentSheetEvent * _Nonnull sheetEvent) {
            [self executeCallback:sheetEvent];
        }];
        [self close:NO];
    } else {
        [self showToast];
    }
}


- (UILabel *)toastLabel {
    if (!_toastLabel) {
        _toastLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, self.contentView.iapu_top + 117 + 56, 0, 28)];
        _toastLabel.backgroundColor = [[UIColor blackColor] colorWithAlphaComponent:0.7];
        _toastLabel.font = [IAPStyleUtils fontOfSize:14];
        _toastLabel.textColor = [UIColor whiteColor];
        _toastLabel.textAlignment = NSTextAlignmentCenter;
        _toastLabel.layer.cornerRadius = 9;
        _toastLabel.clipsToBounds = YES;
    }
    return _toastLabel;
}

- (void)showToast {
    [self addSubview:self.toastLabel];
    NSString *toastText = [IAPMultiLanguageUtils presetPaymentSheetText:IAPPaymentSheetPresetText_Select_your_payment_method];
    self.toastLabel.text = toastText;
    self.toastLabel.iapu_width = [[NSAttributedString alloc] initWithString:toastText ?: @"" attributes:@{NSFontAttributeName: self.toastLabel.font}].size.width + 16;
    self.toastLabel.iapu_centerX = self.iapu_width / 2;
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [self.toastLabel removeFromSuperview];
    });
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    NSLog(@"");
}

- (void)executeCallback:(IAPPaymentSheetEvent *)event {
    self.callback(event);
    [[AlipayPlusClient shared] logWithName:@"paymentSheetCallback" parameter:@{@"message":event.message ?: @""}];
}

- (void)orientationChanged:(NSNotification *)noti {
    UIInterfaceOrientation interfaceOrientation = [noti.userInfo[ UIApplicationStatusBarOrientationUserInfoKey] integerValue];
    if (interfaceOrientation != self.interfaceOrientation) {
        self.interfaceOrientation = interfaceOrientation;
    } else {
        return;
    }
    
    [self.curtainView removeFromSuperview];
    self.curtainView = nil;
    [self.contentView removeFromSuperview];
    self.contentView = nil;
    self.toastLabel = nil;
    [self removeFromSuperview];
    
    CGFloat width = [UIScreen mainScreen].bounds.size.width;
    CGFloat height = [UIScreen mainScreen].bounds.size.height;
    
    if (interfaceOrientation == UIInterfaceOrientationPortrait || interfaceOrientation == UIInterfaceOrientationPortraitUpsideDown) {
        self.frame = CGRectMake(0, 0, MIN(width, height), MAX(width, height));
    } else if (interfaceOrientation == UIInterfaceOrientationLandscapeLeft || interfaceOrientation == UIInterfaceOrientationLandscapeRight) {
        self.frame = CGRectMake(0, 0, MAX(width, height), MIN(width, height));
    }
    
    [self initSubviews];
    [self showAnimated:NO];
}

@end
