//
//  AlipayPlusClientAPI.m
//  AlipayPlusClient
//
//  Created by assuner on 2022/3/12.
//

#import "AlipayPlusClientAPI.h"
#import "IAPAlipayCNWalletAction.h"
#import "IAPCommonWalletAction.h"
#import "IAPInquirePaymentOptionSponsor.h"
#import "IAPJSONUtils.h"
#import "IAPPaymentData.h"
#import "IAPPaymentSheetEvent+Private.h"
#import "IAPPaymentSheetView.h"

@interface AlipayPlusClient ()
@property (nonatomic, assign) BOOL hasShowPaymentSheet;
@property (nonatomic, copy) void (^callbackBlock)(IAPPaymentSheetEvent *_Nonnull sheetEvent);
@end

@implementation AlipayPlusClient
+ (instancetype)shared {
    static AlipayPlusClient *client = nil;
    static dispatch_once_t onceToken;

    dispatch_once(&onceToken, ^{
        client = [AlipayPlusClient new];
    });
    
    return client;
}

- (void)inquirePaymentOptionWithParams:(IAPInquirePaymentOptionParams *)params
                     completionHandler:(void(^)(IAPPaymentOption * _Nullable paymentOption, NSError * _Nullable error))completionHandler {
    [IAPInquirePaymentOptionSponsor inquirePaymentOptionWithParams:params completionHandler:completionHandler];
}

- (void)showPaymentSheet:(NSString *)paymentData
                callback:(void(^)(IAPPaymentSheetEvent * _Nonnull sheetEvent))callback {
    if (self.hasShowPaymentSheet || !callback) {
        return;
    }

    self.callbackBlock = callback;

    NSDictionary *json = [IAPJSONUtils JSONDictionaryFromString:paymentData];

    if (!json) {
        callback([IAPPaymentSheetEvent eventWithName:IAPPaymentSheetEventThrowException message:@"PARAM_ILLEGAL"]);
        [[AlipayPlusClient shared] logWithName:@"paymentSheetCallback" parameter:@{ @"message": @"PARAM_ILLEGAL" }];
        return;
    }

    IAPPaymentData *paymentDataFromJSON = [[IAPPaymentData alloc] initWithJSONDictionary:json];

    if (paymentDataFromJSON.wallets.count == 1 && paymentDataFromJSON.wallets.firstObject.enabled) {
        callback([IAPPaymentSheetEvent eventWithName:IAPPaymentSheetEventDidSelectWalletAndPay message:@""]);
        [self payWithWallet:paymentDataFromJSON.wallets.firstObject callback:callback];
    } else {
        [self payWithPaymentData:paymentDataFromJSON callback:^(IAPPaymentSheetEvent * _Nonnull sheetEvent) {
            callback(sheetEvent);
        }];
    }
}

- (BOOL)canProcessOrderWithPaymentResult:(NSURL *)resultUrl {
    if ([resultUrl.host isEqualToString:@"safepay"]) {
        return YES;
    } else {
        return NO;
    }
}

- (void)processOrderWithPaymentResult:(NSURL *)resultUrl {
    [[IAPAlipayCNWalletAction shared] processOrderWithPaymentResult:resultUrl];
}

#pragma mark - private

- (IAPConfiguration *)configuration {
    if (!_configuration) {
        _configuration = [IAPConfiguration new];
    }

    return _configuration;
}

- (void)logWithName:(NSString *)name parameter:(nullable NSDictionary *)parameter {
    if (name.length && [self.logService respondsToSelector:@selector(logWithName:parameter:)]) {
        [self.logService logWithName:name parameter:parameter];
    }
}

- (void)payWithWallet:(IAPPaymentWalletData *)walletData callback:(void (^)(IAPPaymentSheetEvent *_Nonnull sheetEvent))callback {
    // for testCNWallet while integration testing
    if (walletData.redirectionInfo.isTestCNWallet &&
        [[AlipayPlusClient shared].configuration.envType isEqualToString:kIAPEnvSANDBOX]) {
        [[NSUserDefaults standardUserDefaults] setObject:@"pay" forKey:@"ali_type"];
        // for testCNWallet call processOrderWithPaymentResult function while integration testing
        [[IAPAlipayCNWalletAction shared] handlePayWithWalletData:walletData callback:callback];
        return;
    }
    
    if (walletData.redirectionInfo.orderInfo) {
        [[IAPAlipayCNWalletAction shared] handlePayWithWalletData:walletData callback:callback];
    } else {
        [[IAPCommonWalletAction new] handlePayWithWalletData:walletData callback:callback];
    }
}

- (void)payWithPaymentData:(IAPPaymentData *)paymentData callback:(void (^)(IAPPaymentSheetEvent *_Nonnull sheetEvent))callback {
    self.hasShowPaymentSheet = YES;
    [IAPPaymentSheetView showWithPaymentData:paymentData callback:^(IAPPaymentSheetEvent *_Nonnull sheetEvent) {
        callback(sheetEvent);
        if ([sheetEvent.name isEqualToString:IAPPaymentSheetEventThrowException] ||
            [sheetEvent.name isEqualToString:IAPPaymentSheetEventUserDidCancel] ||
            [sheetEvent.name isEqualToString:IAPPaymentSheetEventDidSelectWalletAndPay]) {
            self.hasShowPaymentSheet = NO;
        }
    }];
}

@end
