package com.iap.basic.alipay.async;

import android.os.AsyncTask;

import androidx.annotation.Nullable;

/**
 * Async task with a {@link IAPAsyncCallback}, you can implement something time consuming task by override {@link
 * #run()} method.
 * And this task will automatically callback with {@link IAPAsyncCallback} interface when post execute.
 * {@link #onPostExecute(Object)} will automatically call:
 * <ol>
 * <li>{@link IAPAsyncCallback#onSuccess(Object)} if {@link #run()} is successful and return a expected result</li><p/>
 * or:<p/>
 * <li>{@link IAPAsyncCallback#onFailure(Exception)} if {@link #run()} throw a exception or return a
 * {@link Exception} object.</li><p/>
 * </ol>
 */
abstract public class IAPAsyncTask extends AsyncTask<Void, Void, Object> {
    /**
     * LifecycleCallback object for this async task
     */
    protected IAPAsyncCallback callback;

    /**
     * @param callback LifecycleCallback object for this async task,
     */
    public IAPAsyncTask(IAPAsyncCallback callback) {
        this.callback = callback;
    }

    public static <T> void asyncTask(final Runner<T> runner) {
        new IAPAsyncTask(runner) {
            @Override
            protected Object run() throws Exception {
                return runner.execute();
            }
        }.executeOnExecutor(AsyncTaskExecutor.getInstance().getExecutor());
    }

    public static void asyncTask(final Runnable runnable) {
        new IAPAsyncTask(null) {
            @Override
            @Nullable
            protected Object run() throws Exception {
                runnable.run();
                return null;
            }
        }.executeOnExecutor(AsyncTaskExecutor.getInstance().getExecutor());
    }


    @Override
    protected Object doInBackground(Void... params) {
        try {
            return run();
        } catch (Exception e) {
            return e;
        }
    }

    /**
     * Do something time consuming task, this task will be done in {@link #doInBackground(Void...)} life cycle.
     *
     * @return a expected result
     * @throws Exception if error happened
     */
    @Nullable
    abstract protected Object run() throws Exception;

    @Override
    @SuppressWarnings("unchecked")
    protected void onPostExecute(Object o) {
        if (callback == null) {
            return;
        }

        if (o instanceof Exception) {
            callback.onFailure((Exception)o);
        } else {
            callback.onSuccess(o);
        }
    }

    public interface Runner<T> extends IAPAsyncCallback<T> {
        @Nullable
        T execute() throws Exception;
    }
}
