package com.iap.basic.alipay.async;

import java.util.ArrayList;
import java.util.concurrent.Executor;

public class StandardPipeline implements PipeLine {

    static final String TAG = "StandardPipeline";
    /**
     * 串行执行的缓存队列
     */
    protected ArrayList<NamedRunnable> mTasks = new ArrayList<NamedRunnable>();
    protected volatile boolean mIsStart = false;
    /**
     * 当前正在执行的AsyncTask任务
     */
    volatile NamedRunnable mActive;
    private Executor mExecutor;
    /**
     * 串行执行的回调接口
     */
    final IScheduleNext next = new IScheduleNext() {
        @Override
        public void scheduleNext() {
            if (StandardPipeline.this.mIsStart) {
                StandardPipeline.this.executeNext();
            }
        }
    };

    public StandardPipeline() {
        this(null);
    }

    public StandardPipeline(Executor executor) {
        mExecutor = executor;
    }

    @Override
    public void setExecutor(Executor executor) {
        mExecutor = executor;
    }

    @Override
    public void addTask(Runnable task, String threadName) {
        addTask(task, threadName, 0);
    }

    @Override
    public void addTask(Runnable task, String threadName, int weight) {
        NamedRunnable _task = NamedRunnable.TASK_POOL.obtain(task, threadName, weight);
        addTask(_task);
    }

    public void addTask(final NamedRunnable task) {
        // if shutdown is invoked, mTasks will be null, which lead that addTask throws RuntimeException
        if (null == mTasks) {
            //  create new list to restart task
            mTasks = new ArrayList<>();
        }
        task.setScheduleNext(next);
        synchronized (mTasks) {
            int index = 0;
            if (!mTasks.isEmpty()) {
                for (index = mTasks.size() - 1; index >= 0; index--) {
                    if (task.mWeight <= mTasks.get(index).mWeight) {
                        index += 1;
                        break;
                    }
                }
                index = index < 0 ? 0 : index;
            }
            mTasks.add(index, task);
        }
        if (mIsStart) {
            doStart();
        }
    }

    @Override
    public void start() {
        if (null == mExecutor) {
            return;
        }
        mIsStart = true;
        doStart();
    }

    /**
     *
     */
    protected void doStart() {
        if (mActive == null) {
            executeNext();
        }
    }

    /**
     * 执行下一个AsyncTask
     */
    private void executeNext() {
        synchronized (mTasks) {
            if (!mTasks.isEmpty()) {
                mActive = mTasks.remove(0);
            } else {
                mActive = null;
            }
        }
        if (mActive != null) {
            if (null != mExecutor) {
                mExecutor.execute(mActive);
            }
        }
    }

    /**
     * 关闭执行器
     */
    @Override
    public void stop() {
        mIsStart = false;
    }

    /**
     * 串行执行的回调接口
     * @author xinjian.lxj
     * @since 2014.07.23
     */
    interface IScheduleNext {
        /**
         * 串行执行的回调函数
         */
        void scheduleNext();
    }
}
