package com.iap.basic.alipay.http.core;

import android.content.Context;
import android.os.Build;
import android.text.TextUtils;
import android.util.Log;
import android.webkit.CookieManager;
import android.webkit.CookieSyncManager;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.iap.basic.alipay.http.model.HttpMethod;
import com.iap.basic.alipay.http.model.HttpRequest;
import com.iap.basic.alipay.http.model.HttpResponse;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;

public class HttpClient {

    private static final String TAG = "HttpClient";
    private static final String DEFAULT_CONTENT_TYPE = "application/x-www-form-urlencoded";
    private static final int DEFAULT_TIMEOUT_MS = 5000;
    private static final String HEADER_CONTENT_TYPE = "Content-Type";
    private static final String PROTOCOL_HTTPS = "https";
    private boolean mEnableCache;
    private List<HttpInterceptor> interceptors = new ArrayList<>();


    public HttpClient(boolean enableCache) {
        this.mEnableCache = enableCache;
    }

    public Call newCall(@NonNull HttpRequest request) {
        return RealCall.newRealCall(this, request, interceptors);
    }

    public HttpClient addInterceptor(HttpInterceptor interceptor) {
        interceptors.add(interceptor);
        return this;
    }

    public HttpClient removeInterceptor(HttpInterceptor interceptor) {
        interceptors.remove(interceptor);
        return this;
    }

    HttpResponse executed(@NonNull RealCall realCall) throws Exception {
        HttpRequest request = realCall.request;
        if (TextUtils.isEmpty(request.url)) {
            throw new HttpException("request.url is empty!");
        } else {
            URL url = new URL(request.url);
            HttpURLConnection connection = this.openConnection(url, request);
            addHeaders(connection, request.headers);
            this.setConnectionRequestType(connection, request.method, request.data);

            int responseCode;
            try {
                responseCode = connection.getResponseCode();
            } catch (IOException exception) {
                responseCode = connection.getResponseCode();
            }

            if (responseCode == -1) {
                throw new HttpException("Could not retrieve response code from HttpUrlConnection.");
            } else {
                String responseMessage = connection.getResponseMessage();
                HttpResponse response;
                if (this.hasResponseBody(responseCode)) {
                    response = new HttpResponse(responseCode, responseMessage, this.getContent(connection), connection.getHeaderFields());
                } else {
                    response = new HttpResponse(responseCode, responseMessage, null, connection.getHeaderFields());
                }
                return response;
            }
        }
    }

    private boolean hasResponseBody(int responseCode) {
        return responseCode >= 200 && responseCode != 204 && responseCode != 304;
    }

    @Nullable
    private byte[] getContent(@NonNull HttpURLConnection connection) {
        InputStream inputStream;
        try {
            inputStream = connection.getInputStream();
        } catch (IOException var4) {
            inputStream = connection.getErrorStream();
        }
        return HttpUtils.readBytes(inputStream);
    }

    private void setConnectionRequestType(@NonNull HttpURLConnection connection, @Nullable HttpMethod method, @Nullable String data) throws IOException {
        if (method == null) {
            method = HttpMethod.GET;
        }

        connection.setRequestMethod(method.method);
        if ((method == HttpMethod.POST || method == HttpMethod.PUT) && !TextUtils.isEmpty(data)) {
            this.addBody(connection, data.getBytes("UTF-8"));
        }
    }

    private void addBody(@NonNull HttpURLConnection connection, @NonNull byte[] body) throws IOException {
        connection.setFixedLengthStreamingMode(body.length);
        connection.setDoOutput(true);
        if (TextUtils.isEmpty(connection.getRequestProperty(HEADER_CONTENT_TYPE))) {
            connection.addRequestProperty(HEADER_CONTENT_TYPE, DEFAULT_CONTENT_TYPE);
        }
        connection.connect();
        DataOutputStream out = new DataOutputStream(connection.getOutputStream());
        out.write(body);
        out.close();
    }


    private HttpURLConnection openConnection(@NonNull URL url, @NonNull HttpRequest request)
            throws IOException, NoSuchAlgorithmException, KeyManagementException {
        HttpURLConnection connection = this.createConnection(url);
        if (request.timeoutMilliseconds <= 0) {
            request.timeoutMilliseconds = DEFAULT_TIMEOUT_MS;
        }

        connection.setConnectTimeout(request.timeoutMilliseconds);
        connection.setReadTimeout(request.timeoutMilliseconds);
        connection.setUseCaches(this.mEnableCache);
        connection.setDefaultUseCaches(this.mEnableCache);
        connection.setDoInput(true);
        connection.setInstanceFollowRedirects(!request.notFollowRedirects);
        if (PROTOCOL_HTTPS.equals(url.getProtocol())) {
            SSLContext sslContext = SSLContext.getInstance("TLS");
            sslContext.init(null, null, null);
            HttpsURLConnection httpsURLConnection = (HttpsURLConnection) connection;
            httpsURLConnection.setSSLSocketFactory(sslContext.getSocketFactory());
        }

        return connection;
    }

    private HttpURLConnection createConnection(URL url) throws IOException {
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setInstanceFollowRedirects(HttpURLConnection.getFollowRedirects());
        return connection;
    }

    private static void addHeaders(@NonNull HttpURLConnection connection, @Nullable Map<String, ?> headers) {
        if (headers != null) {
            for (Map.Entry<String, ?> stringEntry : headers.entrySet()) {
                String key = stringEntry.getKey();
                if (TextUtils.isEmpty(connection.getRequestProperty(key))) {
                    connection.addRequestProperty(key, String.valueOf(stringEntry.getValue()));
                }
            }
        }
    }

}
