package com.iap.basic.alipay.image;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.util.Log;
import android.util.LruCache;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;


import androidx.annotation.DrawableRes;
import androidx.annotation.Nullable;

import com.iap.basic.alipay.async.IAPAsyncTask;
import com.iap.basic.alipay.utils.DigestManager;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.lang.ref.WeakReference;
import java.net.HttpURLConnection;
import java.net.URL;

/**
 * A default implementation of {@link ImageService}.
 *
 * @author jiabing.yjb
 */
public class DefaultImageService implements ImageService {

    private static final String TAG = "DefaultImageService";

    private final static String FILE_DIRECTORY = "alipayplusclient";

    private final LruCache<String, Bitmap> mMemoryCache = new LruCache<String, Bitmap>(
            (int) (Runtime.getRuntime().maxMemory() / 1024 / 8)) {
        @Override
        protected int sizeOf(String key, Bitmap bitmap) {
            return bitmap.getByteCount() / 1024;
        }
    };


    private DefaultImageService() {
    }

    private static class LazyHolder {
        static final DefaultImageService INSTANCE = new DefaultImageService();
    }

    public static DefaultImageService getInstance() {
        return LazyHolder.INSTANCE;
    }

    @Override
    public void loadImage(final Context context, final String url, ImageView imageView, final CallBack callBack) {
        loadImage(context, url, "",  0, imageView, null, callBack);
    }

    @Override
    public void loadImage(final Context context,
                          final String url,
                          final String placeHolder,
                          final int localId,
                          ImageView imageView,
                          final TextView nameView,
                          final CallBack callBack) {
        if (imageView == null) {
            return;
        }
        imageView.setImageResource(android.R.color.transparent);
        if (TextUtils.isEmpty(url)) {
            return;
        }
        Bitmap bitmap = mMemoryCache.get(url);
        if (bitmap != null) {
            imageView.setImageBitmap(bitmap);
            if (callBack != null) {
                callBack.onSuccess(url);
            }
        } else {
            getBitmapFromLocal(context, url, placeHolder, localId, imageView, nameView, callBack);
        }
    }

    public void loadFromServer(final Context context,
                               final String url,
                               final int localId,
                               final String placeHolder,
                               final WeakReference<ImageView> imageViewWeakReference,
                               final WeakReference<TextView> nameViewWeakReference,
                               final CallBack callBack) {

        ImageView imageView = (imageViewWeakReference == null || imageViewWeakReference.get() == null) ? null : imageViewWeakReference.get();
        TextView nameView = (nameViewWeakReference == null || nameViewWeakReference.get() == null) ? null : nameViewWeakReference.get();
        if (imageView == null || nameView == null) {
            return;
        }
        imageView.setTag(url);
        IAPAsyncTask.asyncTask(new IAPAsyncTask.Runner<Bitmap>() {
            @Override
            public Bitmap execute() throws Exception {
                URL imageUrl = new URL(url);
                HttpURLConnection connection = (HttpURLConnection) imageUrl.openConnection();
                connection.setDoInput(true);
                connection.setConnectTimeout(5000);
                connection.connect();
                return BitmapFactory.decodeStream(connection.getInputStream());
            }

            @Override
            public void onSuccess(@Nullable Bitmap bitmap) {
                if (bitmap == null) {
                    if (callBack != null) {
                        DefaultImageService.this.onFailure(imageViewWeakReference, nameViewWeakReference, url, localId, placeHolder, callBack);
                    }
                    return;
                }
                if (mMemoryCache.get(url) == null) {
                    mMemoryCache.put(url, bitmap);
                }
                if (!TextUtils.isEmpty(url)) {
                    saveBitmapToLocal(context, url, bitmap);
                }
                // set image
                ImageView iv = imageViewWeakReference.get();
                if (iv == null) {
                    if (callBack != null) {
                        DefaultImageService.this.onFailure(imageViewWeakReference, nameViewWeakReference, url, localId, placeHolder, callBack);
                    }
                    return;
                }
                Object tag = iv.getTag();
                if (!(tag instanceof String) || !tag.equals(url)) {
                    if (callBack != null) {
                        DefaultImageService.this.onFailure(imageViewWeakReference, nameViewWeakReference, url, localId, placeHolder, callBack);
                    }
                    return;
                }
                iv.setImageBitmap(bitmap);
                if (callBack != null) {
                    callBack.onSuccess(url);
                }
            }

            @Override
            public void onFailure(Exception e) {
                DefaultImageService.this.onFailure(imageViewWeakReference, nameViewWeakReference, url, localId, placeHolder, callBack);
            }
        });

    }

    private void onFailure(WeakReference<ImageView> imageViewWeakReference,
                           WeakReference<TextView> nameViewweakReference,
                           String url,
                           int localId, String placeHolder,
                           CallBack callBack) {
        ImageView imageView = imageViewWeakReference.get();
        TextView nameView = nameViewweakReference.get();
        if (imageView != null && nameView != null) {
            Object tag = imageView.getTag();
            if (!(tag instanceof String) || !tag.equals(url)) {
                if (callBack != null) {
                    callBack.onFail(url);
                }
                return;
            }
            if (localId != 0) {
                imageView.setVisibility(View.VISIBLE);
                imageView.setImageResource(localId);
                nameView.setVisibility(View.GONE);
            } else {
                imageView.setVisibility(View.GONE);
                nameView.setText(placeHolder);
                nameView.setVisibility(View.VISIBLE);
            }
        }
    }

    public static void saveBitmapToLocal(final Context context, final String url, final Bitmap bitmap) {
        IAPAsyncTask.asyncTask(new Runnable() {
            @Override
            public void run() {
                try {
                    if (bitmap == null) {
                        return;
                    }
                    File parent = context.getFilesDir();
                    File directory = new File(parent, FILE_DIRECTORY);
                    if (!directory.exists()) {
                        directory.mkdir();
                    }
                    String key = DigestManager.getMD5(url);
                    // 创建文件流，指向该路径，文件名叫做fileName
                    File  file = new File(directory.getAbsolutePath(), key);
                    // file其实是图片，它的父级File是文件夹，判断一下文件夹是否存在，如果不存在，创建文件夹
                    File fileParent = file.getParentFile();
                    if (!fileParent.exists()) {
                        // 文件夹不存在
                        fileParent.mkdirs();// 创建文件夹
                    }
                    // 将图片保存到本地
                    bitmap.compress(Bitmap.CompressFormat.PNG, 100,
                            new FileOutputStream(file));
                } catch (Exception e) {
                    Log.e(TAG, "saveBitmapToLocal occurs exception");
                }
            }
        });
    }

    public  void getBitmapFromLocal(final Context context,
                                          final String url,
                                          final String placeHolder,
                                          @DrawableRes final int localId,
                                          final ImageView imageView,
                                          final TextView namView,
                                          final CallBack callBack) {
        imageView.setTag(url);
        final WeakReference<ImageView> imageViewWeakReference = new WeakReference<>(imageView);
        final WeakReference<TextView> nameViewWeakReference = new WeakReference<>(namView);
        IAPAsyncTask.asyncTask(new Runnable() {
            @Override
            public void run() {
                Bitmap bitmap = null;
                try {
                    File parent = context.getFilesDir();
                    File directory = new File(parent, FILE_DIRECTORY);
                    File file = new File(directory, DigestManager.getMD5(url));
                    if (file.exists()) {
                        bitmap = BitmapFactory.decodeStream(new FileInputStream(
                                file));
                    } else {
                        loadFromServer(context, url, localId, placeHolder, imageViewWeakReference, nameViewWeakReference, callBack);
                    }
                } catch (Exception e) {
                    Log.e(TAG, "getBitmapFromLocal occurs exception");
                }
                ImageView iv = imageViewWeakReference.get();
                if (iv == null) {
                    return;
                }
                Object tag = iv.getTag();
                if (!(tag instanceof String) || !tag.equals(url)) {
                    return;
                }
                final Bitmap finalBitmap = bitmap;
                new Handler(Looper.getMainLooper()).post(new Runnable() {
                    @Override
                    public void run() {
                        if (finalBitmap != null) {
                            imageView.setImageBitmap(finalBitmap);
                            if (callBack != null) {
                                callBack.onSuccess(url);
                            }
                        } else if (localId != 0) {
                            imageView.setImageResource(localId);
                            if (callBack != null) {
                                callBack.onSuccess(url);
                            }
                        }
                    }
                });
            }
        });

    }

}
