package com.iap.cashier.core;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;

import com.iap.alipayplusclient.R;
import com.iap.basic.alipay.LogService;
import com.iap.basic.alipay.async.IAPAsyncTask;
import com.iap.basic.alipay.config.ConfigurationManager;
import com.iap.basic.alipay.http.core.HttpClient;
import com.iap.basic.alipay.http.model.HttpRequest;
import com.iap.basic.alipay.http.model.HttpResponse;
import com.iap.basic.alipay.utils.NetworkUtils;
import com.iap.basic.alipay.utils.SharedPreferenceManager;
import com.iap.cashier.callback.IAPInquirePaymentCallback;
import com.iap.cashier.callback.IAPPaymentSheetEventCallback;
import com.iap.cashier.data.Constants;
import com.iap.cashier.data.inner.RequestCompose;
import com.iap.cashier.data.model.IAPInquirePaymentOptionParams;
import com.iap.cashier.data.model.IAPPaymentOption;
import com.iap.cashier.data.model.IAPPaymentSheetEvent;
import com.iap.cashier.utils.JSONConfig;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;


/**
 * Author kanguihu
 * Data   2022/3/7
 * Desc
 */
public class AlipayCashierClient {

    private static final String TAG = AlipayCashierClient.class.getSimpleName();

    public static final String CONTENT_TYPE = "Content-Type";
    public static final String CONTENT_TYPE_JSON = "application/json";

    public static LogService logService;


    private static volatile AlipayCashierClient mInstance;
    private HttpClient httpClient;

    public static AlipayCashierClient getInstance() {
        if (mInstance == null) {
            synchronized (AlipayCashierClient.class) {
                if (mInstance == null) {
                    mInstance = new AlipayCashierClient();
                }
            }
        }
        return mInstance;
    }

    private AlipayCashierClient () {
        httpClient = new HttpClient(false);
    }

    public void inquirerPaymentOption(final Context context, final IAPInquirePaymentOptionParams params, final IAPInquirePaymentCallback<IAPPaymentOption> callback) {
        if (callback == null) {
            return;
        }
        final String envType = ConfigurationManager.getInstance().getConfiguration().envType;
        String acquirerId = ConfigurationManager.getInstance().getConfiguration().acquirerId;
        String merchantId = ConfigurationManager.getInstance().getConfiguration().merchantId;
        final String paymentCurrency = params.paymentCurrency;
        if (params == null
                || TextUtils.isEmpty(acquirerId)
                || TextUtils.isEmpty(merchantId)
                || TextUtils.isEmpty(paymentCurrency)) {
            //参数异常
            onFailure(callback, Constants.ERROR_PARAM_ILLEGAL, Constants.MESSAGE_PARAM_ILLEGAL);
            return;
        }
        final String region = TextUtils.isEmpty(params.userRegion) ? "defaultregion" : params.userRegion;
        final String logoPattern = TextUtils.isEmpty(params.logoPattern) ? "defaultlogo" : params.logoPattern;
        if (!NetworkUtils.isNetworkAvailable(context)) {
            netWorkExceptionCallback(context, paymentCurrency, region, logoPattern, callback);
            return;
        }
        HttpRequest httpRequest = null;
        if (TextUtils.equals(Constants.ENV_SANDBOX, envType)) {
            try {
                httpRequest = RequestCompose.getSandboxReqeust(context, merchantId, acquirerId, paymentCurrency, region, logoPattern);
            } catch (JSONException e) {
                onFailure(callback, Constants.ERROR_PARAM_ILLEGAL, Constants.MESSAGE_PARAM_ILLEGAL);
            }
        } else {
            httpRequest = RequestCompose.getCDNRequest(merchantId, acquirerId, paymentCurrency, region, logoPattern);
        }
        if (httpRequest != null) {
            httpRequest.addHeader(CONTENT_TYPE, CONTENT_TYPE_JSON);
        }
        final HttpRequest finalHttpRequest = httpRequest;
        IAPAsyncTask.asyncTask(new Runnable() {
            @Override
            public void run() {
                execute(context, paymentCurrency, region, logoPattern, finalHttpRequest, callback);
            }
        });
    }

    private void execute(Context context,
                         String currency,
                         String region,
                         String logoPattern,
                         HttpRequest finalHttpRequest, IAPInquirePaymentCallback<IAPPaymentOption> callback) {
        try {
            HttpResponse execute = httpClient.newCall(finalHttpRequest).execute();
            if (isSuccess(execute)) {
                String data = new String(execute.data);
                JSONObject object = new JSONObject(data);
                IAPPaymentOption iapPaymentOption = JSONConfig.convert2IAPPaymentOption(context, object);
                if (iapPaymentOption == null) {
                    //系统异常
                    onFailure(callback, Constants.ERROR_SYSTEM_ERROR, Constants.MESSAGE_SYSTEM_ERROR);
                }  else {
                    onSuccess(context, currency, region, logoPattern, callback, iapPaymentOption);
                }

            } else {
                IAPPaymentOption iapPaymentOption = new IAPPaymentOption();
                iapPaymentOption.paymentMethodType = "CONNECT_WALLET";
                iapPaymentOption.enabled = false;
                iapPaymentOption.disableReason = context.getResources().getString(R.string.cashier_payment_method_not_found);
                onSuccess(context,currency, region, logoPattern, callback, iapPaymentOption);
            }
        } catch (Throwable throwable) {
            //系统异常
            if (throwable instanceof IOException) {
                netWorkExceptionCallback(context, currency, region, logoPattern, callback);
            } else {
                onFailure(callback, Constants.ERROR_SYSTEM_ERROR, Constants.MESSAGE_SYSTEM_ERROR);
            }
        }
    }

    private void netWorkExceptionCallback(Context context,
                                          String currency,
                                          String region,
                                          String logoPattern,
                                          IAPInquirePaymentCallback<IAPPaymentOption> callback) {
        String key = getStoragePaymentOptionKey(currency, region, logoPattern);
        String optionStr = SharedPreferenceManager.getInstance(context).fetch(key);
        if (TextUtils.isEmpty(optionStr)) {
            onFailure(callback, Constants.ERROR_INVALIDE_NETWORK, Constants.MESSAGE_INVALID_NETWORK);
            return;
        }
        try {
            IAPPaymentOption iapPaymentOption = JSONConfig.getIAPPaymentOption(optionStr);
            if (iapPaymentOption != null) {
                iapPaymentOption.promoNames = null;
                iapPaymentOption.disableReason = null;
            }
            onSuccess(context, currency, region, logoPattern, callback, iapPaymentOption);
        } catch (Exception e) {
            onFailure(callback, Constants.ERROR_INVALIDE_NETWORK, Constants.MESSAGE_INVALID_NETWORK);
        }
    }

    private void onSuccess(final Context context,
                           String currency,
                           String region,
                           String logoPattern,
                           final IAPInquirePaymentCallback<IAPPaymentOption> callback,
                           final IAPPaymentOption iapPaymentOption) {
        String key = getStoragePaymentOptionKey(currency, region, logoPattern);
        SharedPreferenceManager.getInstance(context).save(key, JSONConfig.getIAPPaymentOptionsStr(iapPaymentOption));
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                callback.onSuccess(iapPaymentOption);
            }
        });
    }

    private String getStoragePaymentOptionKey(String currency, String region, String logoPattern) {
        String acquirerId = ConfigurationManager.getInstance().getConfiguration().acquirerId;
        String merchantId = ConfigurationManager.getInstance().getConfiguration().merchantId;
        return Constants.KEY_PAYMENT_OPTION + acquirerId + "_" + merchantId + "_CASHIER_PAYMENT_" + "_" + currency + "_" + region + "_" + logoPattern;

    }

    private void onFailure(final IAPInquirePaymentCallback callback, final String errorCode, final String errorMessage) {
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                callback.onFailure(errorCode, errorMessage);
            }
        });
    }


    private boolean isSuccess(HttpResponse response) {
        return response != null && response.statusCode == 200 && response.data != null;
    }

    public void showPaymentSheet(Context context, String paymentData, IAPPaymentSheetEventCallback<IAPPaymentSheetEvent> callback) {
        if (CashierUtils.validateSingleWallet(context, paymentData, callback)) {
            return;
        }
        PaymentSheetActivity.startActivity(context, paymentData, callback);

    }


}
