/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.callbacks;

import com.alipay.ap.request.CancelTokenRequest;
import com.alipay.ap.response.APSResponse;
import com.alipay.ap.response.CancelTokenResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.enums.ResultCode;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.APSRequest;

/**
 * subclass of {@link HttpSendCallback}
 * implements the callback to handle the response from A+
 *
 * @author wqy01239909
 * @version $Id: DefaultCancelTokenCallback.java, v 0.1 2022‐07-11 16:08 wqy01239909 Exp $
 */
public abstract class DefaultCancelTokenCallback extends
                                                 HttpSendCallback<CancelTokenRequest, CancelTokenResponse> {
    /** logger */
    private static final Logger logger = LoggerFactory.getLogger(DefaultCancelTokenCallback.class);

    /**
     * on response.result.resultStatus = S
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface and process responses
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception
     */
    @Override
    public void onSstatus(APSClient apsClient, CancelTokenRequest request,
                          CancelTokenResponse response) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG][onSstatus][CancelTokenRequest={}],[CancelTokenResponse={}],",
            request, response);
        this.onCancelTokenSuccess(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = F
     * 
     * @param apsClient  one abstract class that defines methods to call A+ interface and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception
     */
    @Override
    public void onFstatus(APSClient apsClient, CancelTokenRequest request,
                          CancelTokenResponse response) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG][onFstatus][CancelTokenRequest={}],[CancelTokenResponse={}],",
            request, response);
        // In addition, when the following two result codes are returned, the revoke of the access token can be regarded as a success conditionally
        ResultCode resultCode = ResultCode.getByCode(response.getResult().getResultCode());
        if (ResultCode.INVALID_TOKEN == resultCode
            || ResultCode.EXPIRED_ACCESS_TOKEN == resultCode) {
            this.onCancelTokenSuccess(apsClient, request, response);
        } else {
            this.onCancelTokenFail(apsClient, request, response);
        }
    }

    /**
     * on response.result.resultStatus = U
     * 
     * @param apsClient  one abstract class that defines methods to call A+ interface and process responses
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception
     */
    @Override
    public void onUstatus(APSClient apsClient, CancelTokenRequest request,
                          CancelTokenResponse response) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG][onUstatus][CancelTokenRequest={}],[CancelTokenResponse={}],",
            request, response);
        this.onCancelTokenUnknown(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = S 
     * implement this method by ACQ for customization logic
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    public abstract void onCancelTokenSuccess(APSClient apsClient, CancelTokenRequest request,
                                              CancelTokenResponse response) throws Exception;

    /**
     * on response.result.resultStatus = F
     * implement this method by ACQ for customization logic
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    public abstract void onCancelTokenFail(APSClient apsClient, CancelTokenRequest request,
                                           CancelTokenResponse response) throws Exception;

    /**
     * on response.result.resultStatus = U 
     * implement this method by ACQ for customization logic
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     */
    public abstract void onCancelTokenUnknown(APSClient apsClient, CancelTokenRequest request,
                                              CancelTokenResponse response) throws Exception;
}