/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.callbacks;

import com.alipay.ap.response.APSResponse;
import com.alipay.ap.response.CashierPaymentResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.enums.ResultCode;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.APSRequest;
import com.alipay.ap.request.CashierPaymentPayRequest;

/**
 * subclass of {@link HttpSendCallback}
 * implements the callback to handle the response from A+
 * 
 * @author wangpeng
 * @version $Id: DefaultCashierPaymentCallback.java, v 0.1 2022‐06-30 19:04 wangpeng Exp $
 */
public abstract class DefaultCashierPaymentCallback extends

                                                    HttpSendCallback<CashierPaymentPayRequest, CashierPaymentResponse> {
    /** logger */
    private static final Logger logger = LoggerFactory
        .getLogger(DefaultCashierPaymentCallback.class);

    /**
     * on response.result.resultStatus = F 
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception If exception happens
     */
    @Override
    public void onFstatus(APSClient apsClient, CashierPaymentPayRequest request,
                          CashierPaymentResponse response) throws Exception {
        logger.debug(
            "[ALIPAYPLUS_LOG][onFstatus][CashierPaymentPayRequest={}],[CashierPaymentResponse={}],",
            request, response);
        // calling payment failure implementation by ACQ
        this.onPaymentFail(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = U 
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception If exception happens
     */
    @Override
    public void onUstatus(APSClient apsClient, CashierPaymentPayRequest request,
                          CashierPaymentResponse response) throws Exception {
        logger.debug(
            "[ALIPAYPLUS_LOG][onUstatus][CashierPaymentPayRequest={}],[CashierPaymentResponse={}],",
            request, response);
        if (ResultCode.PAYMENT_IN_PROCESS == ResultCode
            .getByCode(response.getResult().getResultCode())) {

            // calling payment in process implementation by ACQ
            this.onPaymentInProcess(apsClient, request, response);
        } else {

            // calling payment unknown implementation by ACQ
            this.onPaymentUnknown(apsClient, request, response);
        }
    }

    /**
     * on response.result.resultStatus = S 
     *
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception If exception happens
     */
    @Override
    public void onSstatus(APSClient apsClient, CashierPaymentPayRequest request,
                          CashierPaymentResponse response) throws Exception {

        logger.debug(
            "[ALIPAYPLUS_LOG][onSstatus][CashierPaymentPayRequest={}],[CashierPaymentResponse={}],",
            request, response);
        //        //For CashierPayment, A+ will not response S status to ACQ. Throw AlipayPlusApiException
        //        throw new AlipayPlusApiException(
        //            String.format("Incorrect payment status for Cashier Payment [resultStatus={}]",
        //                response.getResult().getResultStatus().name()),
        //            CommonConstants.HTTP_SUCCESS_CODE);

        //If the user has completed the payment successfully and you invoke the pay API again with the same parameters, the resutStatus filed is returned as S.
        this.onPaymentSuccess(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = F
     * 
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception If exception happens                
     */
    public abstract void onPaymentFail(APSClient apsClient, CashierPaymentPayRequest request,
                                       CashierPaymentResponse response) throws Exception;

    /**
     * on response.result.resultStatus = U and result.resultCode == PAYMENT_IN_PROCESS
     * implement this method by ACQ for customization logic
     * 
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception If exception happens
     */
    public abstract void onPaymentInProcess(APSClient apsClient, CashierPaymentPayRequest request,
                                            CashierPaymentResponse response) throws Exception;

    /**
     * on response.result.resultStatus = U and result.resultCode != PAYMENT_IN_PROCESS
     * implement this method by ACQ for customization logic
     * 
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception If alipay api exception happens
     */
    public abstract void onPaymentUnknown(APSClient apsClient, CashierPaymentPayRequest request,
                                          CashierPaymentResponse response) throws Exception;

    /**
     * on response.result.resultStatus = S and result.resultCode == SUCCESS
     * implement this method by ACQ for customization logic
     * 
     * @param apsClient  one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request subclasses of {@link APSRequest}
     * @param response subclasses of {@link APSResponse}
     * @throws Exception If alipay api exception happens
     */
    public abstract void onPaymentSuccess(APSClient apsClient, CashierPaymentPayRequest request,
                                          CashierPaymentResponse response) throws Exception;
}