/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.callbacks;

import com.alipay.ap.request.InquiryPaymentRequest;
import com.alipay.ap.response.APSResponse;
import com.alipay.ap.response.InquiryPaymentResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.model.Result;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.APSRequest;

/**
 * subclass of {@link HttpSendCallback}
 * implements the callback to handle the response from A+
 *
 * @author wangpeng
 * @version $Id: DefaultInquiryPaymentCallback.java, v 0.1 2022‐06-28 18:37:18 wangpeng Exp $
 */
public abstract class DefaultInquiryPaymentCallback extends
                                                    HttpSendCallback<InquiryPaymentRequest, InquiryPaymentResponse> {
    /** logger */
    private static final Logger logger = LoggerFactory
        .getLogger(DefaultInquiryPaymentCallback.class);

    /**
     * on response.result.resultStatus = F
     *
     * @param apsClient one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request   subclasses of {@link APSRequest}
     * @param response  subclasses of {@link APSResponse}
     * @throws Exception If exception happens
     */
    @Override
    public void onFstatus(APSClient apsClient, InquiryPaymentRequest request,
                          InquiryPaymentResponse response) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG][onFstatus][InquiryPaymentRequest={}],[InquiryPaymentResponse={}],",
            request, response);
        // 1, call onPaymentInquiryFail
        this.onPaymentInquiryFail(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = U
     *
     * @param apsClient one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request   subclasses of {@link APSRequest}
     * @param response  subclasses of {@link APSResponse}
     * @throws Exception If exception happens
     */
    @Override
    public void onUstatus(APSClient apsClient, InquiryPaymentRequest request,
                          InquiryPaymentResponse response) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG][onUstatus][InquiryPaymentRequest={}],[InquiryPaymentResponse={}],",
            request, response);
        //1, call onPaymentInquiryFail
        // override this method to customise business codes
        this.onPaymentInquiryUnknown(apsClient, request, response);
    }

    /**
     * on response.result.resultStatus = S
     *
     * @param apsClient one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request   subclasses of {@link APSRequest}
     * @param response  subclasses of {@link APSResponse}
     * @throws Exception If exception happens
     */
    @Override
    public void onSstatus(APSClient apsClient, InquiryPaymentRequest request,
                          InquiryPaymentResponse response) throws Exception {
        logger.debug("[ALIPAYPLUS_LOG][onSstatus][InquiryPaymentRequest={}],[InquiryPaymentResponse={}],",
            request, response);
        Result paymentResult = response.getPaymentResult();
        switch (paymentResult.getResultStatus()) {
            case S:
                // payment status = s , call  onPaymentStatusSuccess
                // override this method to customise business codes
                onPaymentStatusSuccess(apsClient, request, response);
                break;

            case F:
                // payment status = F ,call  onPaymentStatusFail
                // override this method to customise business codes
                onPaymentStatusFail(apsClient, request, response);
                break;
            case U:
                // payment status = U ,call  onPaymentStatusSuccess
                // override this method to customise business codes
                onPaymentStatusProcessing(apsClient, request, response);
                break;
            default:
                break;

        }

    }

    /**
     * on response.result.resultStatus = F
     * implement this method by ACQ for customization logic
     *
     * @param apsClient one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request   subclasses of {@link APSRequest}
     * @param response  subclasses of {@link APSResponse}
     * @throws Exception If alipay api exception happens
     */
    public abstract void onPaymentInquiryFail(APSClient apsClient, InquiryPaymentRequest request,
                                              InquiryPaymentResponse response) throws Exception;

    /**
     * on inquriy payment unknown result
     * implement this method by ACQ for customization logic
     *
     * @param apsClient one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request   subclasses of {@link APSRequest}
     * @param response  subclasses of {@link APSResponse}
     * @throws Exception If alipay api exception happens
     */
    public abstract void onPaymentInquiryUnknown(APSClient apsClient, InquiryPaymentRequest request,
                                                 InquiryPaymentResponse response) throws Exception;

    /**
     * on response.result.resultStatus = S & response.paymentResult.resultStatus = S
     * implement this method by ACQ for customization logic
     *
     * @param apsClient one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request   subclasses of {@link APSRequest}
     * @param response  subclasses of {@link APSResponse}
     * @throws Exception If alipay api exception happens
     */
    public abstract void onPaymentStatusSuccess(APSClient apsClient, InquiryPaymentRequest request,
                                                InquiryPaymentResponse response) throws Exception;

    /**
     * on response.result.resultStatus = S & response.paymentResult.resultStatus = F
     * implement this method by ACQ for customization logic
     *
     * @param apsClient one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request   subclasses of {@link APSRequest}
     * @param response  subclasses of {@link APSResponse}
     * @throws Exception If alipay api exception happens
     */
    public abstract void onPaymentStatusFail(APSClient apsClient, InquiryPaymentRequest request,
                                             InquiryPaymentResponse response) throws Exception;

    /**
     * on response.result.resultStatus = S & response.paymentResult.resultStatus = U
     * implement this method by ACQ for customization logic
     *
     * @param apsClient one abstract class that defines methods to call A+ interface  and process responses {@link APSClient}
     * @param request   subclasses of {@link APSRequest}
     * @param response  subclasses of {@link APSResponse}
     * @throws Exception If alipay api exception happens
     */
    public abstract void onPaymentStatusProcessing(APSClient apsClient,
                                                   InquiryPaymentRequest request,
                                                   InquiryPaymentResponse response) throws Exception;

}
