/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.cfg;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.Properties;

import com.alipay.ap.constants.CommonConstants;
import com.alipay.ap.utils.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Global configuration for the APSSettings
 * ACQP could use the best practice configuration or could override the specific param.
 *
 * @author wangpeng
 * @version $Id: APSSettings.java, v 0.1 2022‐06-21 11:09 wangpeng Exp $
 */
public class APSSettings implements Serializable {

    /** logger */
    private static final Logger logger                             = LoggerFactory
        .getLogger(APSSettings.class);

    /** serialVersionUID */
    private static final long   serialVersionUID                   = -628578313361694675L;

    /** client id that represent merchant **/
    private String              clientId;

    /** key that is used to encode signature by acq **/
    private String              privateKey;

    /** key that is used to decode signature by acq **/
    private String              alipayPlusPublicKey;

    /** url of A+ interface **/
    private String              gatewayUrl;

    /**
     * open signature verify flag
     * default value is true
     */
    private boolean             isDevMode                          = false;

    /** Max milliseconds to wait for a connection from the pool **/
    private int                 apacheHttpConnectionRequestTimeout = 5000;
    /** Max milliseconds to wait for a new connection with Alipay gateway to be established **/
    private int                 apacheHttpConnectTimeout           = 5000;
    /** Max milliseconds to wait for response data **/
    private int                 apacheHttpSocketTimeout            = 5000;
    /** Max size of Pool **/
    private int                 apacheMaxPoolSize                  = 20;

    /** configs of scheduled thread pool **/
    private int                 corePoolSizeOfJobExecutor          = 10;
    private int                 maxSizeOFDelayQueue                = 50;
    private int                 initDelay                          = 600;
    private int                 delay                              = 600;

    /** interval seconds for scheduled retry task **/
    // For Cashier Payment, keep querying with an incremental frequency within the payment expiry time, which is 10 minutes by default.
    private int[]               inquiryIntervalForCashierpay       = new int[] { 20, 30, 40, 50, 60,
                                                                                 70, 80 };
    /** In the first 60 seconds, you can retry the request with an interval of 5 to 10 seconds.
    //After 60 seconds, you can still retry the request before 00:15 UTC+8 of T+1 day.*/
    private int[]               cancelInterval                     = new int[] { 5, 5, 5, 5, 10, 10,
                                                                                 10, 10 };
    /** It is recommended to keep trying for at least 1 hour.*/
    private int[]               refundInterval                     = new int[] { 2, 4, 8, 16, 32,
                                                                                 64, 128, 256,
                                                                                 512 };
    /** The ACQP can call the API in a fixed frequency, for example, once a day*/
    private int[]               inquiryRegistrationStatusInterval  = new int[] { 86400, 86400,
                                                                                 86400, 86400,
                                                                                 86400, 86400,
                                                                                 86400 };

    /**
     * Default construction method
     * 
     * clientId/privateKey/AlipayPlusPublicKey/gatewayURL under the /src/main/resource/config.properties files
     */
    public APSSettings() {

        InputStream inputStream = null;
        try {
            Properties prop = new Properties();
            inputStream = getClass().getClassLoader()
                .getResourceAsStream(CommonConstants.APS_CONFIG_FILE);

            if (inputStream == null) {
                logger.error(
                    "[ALIPAYPLUS_LOG][CONFIG_FILE_NOT_FOUND]property file not found in the classpath [src/main/resources/config.properties]");

                throw new FileNotFoundException(String.format(
                    "property file '%s' not found in the classpath [src/main/resources/config.properties]",
                    CommonConstants.APS_CONFIG_FILE));
            }
            prop.load(inputStream);
            String clientId = prop.getProperty("clientId");
            String privateKey = prop.getProperty("privateKey");
            String alipayPlusPublicKey = prop.getProperty("alipayPlusPublicKey");
            String gatewayUrl = prop.getProperty("gatewayUrl");
            String isDevMode = prop.getProperty("isDevMode");
            if (StringUtil.isBlank(clientId) || StringUtil.isBlank(privateKey)
                || StringUtil.isBlank(alipayPlusPublicKey) || StringUtil.isBlank(gatewayUrl)) {
                logger.error(
                    "[ALIPAYPLUS_LOG][CONFIG_PROPERTY_NOT_FOUND]config property not found in the classpath [src/main/resources/config.properties]");

                throw new IllegalArgumentException(String.format(
                    "config property not found in the classpath [src/main/resources/config.properties]"));
            }
            this.clientId = clientId;
            this.privateKey = privateKey;
            this.alipayPlusPublicKey = alipayPlusPublicKey;
            this.gatewayUrl = gatewayUrl;
            this.isDevMode = Boolean.valueOf(isDevMode);

        } catch (IOException e) {

            logger.error(
                "[ALIPAYPLUS_LOG][CONFIG_FILE_IO_EXCEPTION] src/main/resources/config.properties io exception , {} .",
                e);

            throw new IllegalStateException(e);

        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    logger.error(
                        "[ALIPAYPLUS_LOG][CONFIG_FILE_IO_EXCEPTION] src/main/resources/config.properties io exception , {} .",
                        e);
                    throw new IllegalStateException(e);
                }
            }
        }

    }

    /**
     * construction and inject by the ACQ
     * 
     * @param clientId   client id that represent ACQ for making the API call
     * @param privateKey key that is used to sign the request by ACQ
     * @param alipayPlusPublicKey key that is used to verify signature by ACQ
     * @param gatewayUrl url of A+ gateway address
     */
    public APSSettings(String clientId, String privateKey, String alipayPlusPublicKey,
                       String gatewayUrl) {
        this.clientId = clientId;
        this.privateKey = privateKey;
        this.alipayPlusPublicKey = alipayPlusPublicKey;
        this.gatewayUrl = gatewayUrl;
    }

    /**
     * Getter method for property clientId.
     *
     * @return property value of clientId
     */
    public String getClientId() {
        return clientId;
    }

    /**
     * Setter method for property clientId.
     *
     * @param clientId value to be assigned to property clientId
     */
    public void setClientId(String clientId) {
        this.clientId = clientId;
    }

    /**
     * Getter method for property privateKey.
     *
     * @return property value of privateKey
     */
    public String getPrivateKey() {
        return privateKey;
    }

    /**
     * Setter method for property privateKey.
     *
     * @param privateKey value to be assigned to property privateKey
     */
    public void setPrivateKey(String privateKey) {
        this.privateKey = privateKey;
    }

    /**
     * Getter method for property alipayPlusPublicKey.
     *
     * @return property value of alipayPlusPublicKey
     */
    public String getAlipayPlusPublicKey() {
        return alipayPlusPublicKey;
    }

    /**
     * Setter method for property alipayPlusPublicKey.
     *
     * @param alipayPlusPublicKey value to be assigned to property alipayPlusPublicKey
     */
    public void setAlipayPlusPublicKey(String alipayPlusPublicKey) {
        this.alipayPlusPublicKey = alipayPlusPublicKey;
    }

    /**
     * Getter method for property gatewayUrl.
     *
     * @return property value of gatewayUrl
     */
    public String getGatewayUrl() {
        return gatewayUrl;
    }

    /**
     * Setter method for property gatewayUrl.
     *
     * @param gatewayUrl value to be assigned to property gatewayUrl
     */
    public void setGatewayUrl(String gatewayUrl) {
        this.gatewayUrl = gatewayUrl;
    }

    /**
     * Getter method for property isDevMode.
     *
     * @return property value of isDevMode
     */
    public boolean isDevMode() {
        return isDevMode;
    }

    /**
     * Setter method for property devMode.
     *
     * @param devMode value to be assigned to property devMode
     */
    public void setDevMode(boolean devMode) {
        isDevMode = devMode;
    }

    /**
     * Getter method for property apacheHttpConnectionRequestTimeout.
     *
     * @return property value of apacheHttpConnectionRequestTimeout
     */
    public int getApacheHttpConnectionRequestTimeout() {
        return apacheHttpConnectionRequestTimeout;
    }

    /**
     * Setter method for property apacheHttpConnectionRequestTimeout.
     *
     * @param apacheHttpConnectionRequestTimeout value to be assigned to property apacheHttpConnectionRequestTimeout
     */
    public void setApacheHttpConnectionRequestTimeout(int apacheHttpConnectionRequestTimeout) {
        this.apacheHttpConnectionRequestTimeout = apacheHttpConnectionRequestTimeout;
    }

    /**
     * Getter method for property apacheHttpConnectTimeout.
     *
     * @return property value of apacheHttpConnectTimeout
     */
    public int getApacheHttpConnectTimeout() {
        return apacheHttpConnectTimeout;
    }

    /**
     * Setter method for property apacheHttpConnectTimeout.
     *
     * @param apacheHttpConnectTimeout value to be assigned to property apacheHttpConnectTimeout
     */
    public void setApacheHttpConnectTimeout(int apacheHttpConnectTimeout) {
        this.apacheHttpConnectTimeout = apacheHttpConnectTimeout;
    }

    /**
     * Getter method for property apacheHttpSocketTimeout.
     *
     * @return property value of apacheHttpSocketTimeout
     */
    public int getApacheHttpSocketTimeout() {
        return apacheHttpSocketTimeout;
    }

    /**
     * Setter method for property apacheHttpSocketTimeout.
     *
     * @param apacheHttpSocketTimeout value to be assigned to property apacheHttpSocketTimeout
     */
    public void setApacheHttpSocketTimeout(int apacheHttpSocketTimeout) {
        this.apacheHttpSocketTimeout = apacheHttpSocketTimeout;
    }

    /**
     * Getter method for property apacheMaxPoolSize.
     *
     * @return property value of apacheMaxPoolSize
     */
    public int getApacheMaxPoolSize() {
        return apacheMaxPoolSize;
    }

    /**
     * Setter method for property apacheMaxPoolSize.
     *
     * @param apacheMaxPoolSize value to be assigned to property apacheMaxPoolSize
     */
    public void setApacheMaxPoolSize(int apacheMaxPoolSize) {
        this.apacheMaxPoolSize = apacheMaxPoolSize;
    }

    /**
     * Getter method for property corePoolSizeOfJobExecutor.
     *
     * @return property value of corePoolSizeOfJobExecutor
     */
    public int getCorePoolSizeOfJobExecutor() {
        return corePoolSizeOfJobExecutor;
    }

    /**
     * Setter method for property corePoolSizeOfJobExecutor.
     *
     * @param corePoolSizeOfJobExecutor value to be assigned to property corePoolSizeOfJobExecutor
     */
    public void setCorePoolSizeOfJobExecutor(int corePoolSizeOfJobExecutor) {
        this.corePoolSizeOfJobExecutor = corePoolSizeOfJobExecutor;
    }

    /**
     * Getter method for property maxSizeOFDelayQueue.
     *
     * @return property value of maxSizeOFDelayQueue
     */
    public int getMaxSizeOFDelayQueue() {
        return maxSizeOFDelayQueue;
    }

    /**
     * Setter method for property maxSizeOFDelayQueue.
     *
     * @param maxSizeOFDelayQueue value to be assigned to property maxSizeOFDelayQueue
     */
    public void setMaxSizeOFDelayQueue(int maxSizeOFDelayQueue) {
        this.maxSizeOFDelayQueue = maxSizeOFDelayQueue;
    }

    /**
     * Getter method for property initDelay.
     *
     * @return property value of initDelay
     */
    public int getInitDelay() {
        return initDelay;
    }

    /**
     * Setter method for property initDelay.
     *
     * @param initDelay value to be assigned to property initDelay
     */
    public void setInitDelay(int initDelay) {
        this.initDelay = initDelay;
    }

    /**
     * Getter method for property delay.
     *
     * @return property value of delay
     */
    public int getDelay() {
        return delay;
    }

    /**
     * Setter method for property delay.
     *
     * @param delay value to be assigned to property delay
     */
    public void setDelay(int delay) {
        this.delay = delay;
    }

    /**
     * Getter method for property inquiryIntervalForCashierpay.
     *
     * @return property value of inquiryIntervalForCashierpay
     */
    public int[] getInquiryIntervalForCashierpay() {
        return inquiryIntervalForCashierpay;
    }

    /**
     * Setter method for property inquiryIntervalForCashierpay.
     *
     * @param inquiryIntervalForCashierpay value to be assigned to property inquiryIntervalForCashierpay
     */
    public void setInquiryIntervalForCashierpay(int[] inquiryIntervalForCashierpay) {
        this.inquiryIntervalForCashierpay = inquiryIntervalForCashierpay;
    }

    /**
     * Getter method for property cancelInterval.
     *
     * @return property value of cancelInterval
     */
    public int[] getCancelInterval() {
        return cancelInterval;
    }

    /**
     * Setter method for property cancelInterval.
     *
     * @param cancelInterval value to be assigned to property cancelInterval
     */
    public void setCancelInterval(int[] cancelInterval) {
        this.cancelInterval = cancelInterval;
    }

    /**
     * Getter method for property refundInterval.
     *
     * @return property value of refundInterval
     */
    public int[] getRefundInterval() {
        return refundInterval;
    }

    /**
     * Setter method for property refundInterval.
     *
     * @param refundInterval value to be assigned to property refundInterval
     */
    public void setRefundInterval(int[] refundInterval) {
        this.refundInterval = refundInterval;
    }

    /**
     * Getter method for property inquiryRegistrationStatusInterval.
     *
     * @return property value of inquiryRegistrationStatusInterval
     */
    public int[] getInquiryRegistrationStatusInterval() {
        return inquiryRegistrationStatusInterval;
    }

    /**
     * Setter method for property inquiryRegistrationStatusInterval.
     *
     * @param inquiryRegistrationStatusInterval value to be assigned to property inquiryRegistrationStatusInterval
     */
    public void setInquiryRegistrationStatusInterval(int[] inquiryRegistrationStatusInterval) {
        this.inquiryRegistrationStatusInterval = inquiryRegistrationStatusInterval;
    }
}