/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.job;

import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;

import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.request.InquiryPaymentRequest;
import com.alipay.ap.request.RefundRequest;
import com.alipay.ap.request.ScheduleJobRequest;
import com.alipay.ap.enums.BizType;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.CancelPaymentRequest;
import com.alipay.ap.utils.GsonUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * Job Executor framework 
 * 
 * @author mulin.xl
 * @version $Id: APSScheduleJobExecutor.java, v 0.1 2022‐07‐19 15:29 mulin.xl Exp $$
 */
public class APSScheduleJobExecutor {

    /** logger */
    private static final Logger                logger             = LoggerFactory
        .getLogger(APSScheduleJobExecutor.class);

    /**
     * jobExecutorStarted
     */
    private static AtomicBoolean               jobExecutorStarted = new AtomicBoolean(false);

    /**
     * instance
     */
    public final static APSScheduleJobExecutor instance           = new APSScheduleJobExecutor();

    /**
     * schedule job support that manages jobs  , ACQ need to implement it
     */
    private ScheduleJobSupport                 scheduleJobSupport;

    /**
     * an abstract class that defines an APS client , ACQ need to implement it  
     */
    private APSClient                          apsClient;

    /**
     * subclass of HttpSendCallback that handles Alipayplus response , ACQ need to implement it 
     */
    private HttpSendCallback scheduledInquiryCallback;
    private HttpSendCallback scheduledRefundCallback;
    private HttpSendCallback scheduledCancelCallback;

    /**
     * scheduled threadPool executor
     */
    private ScheduledThreadPoolExecutor        executor;

    private APSScheduleJobExecutor() {

    }

    /**
     * register and persist the job
     * 
     * @param request schedule job request
     */
    public boolean put(ScheduleJobRequest request) {
        return scheduleJobSupport.register(request);
    }

    /**
     * start job executor when application started
     * 
     * @return boolean
     */
    public boolean startJobExecutor() {

        if (jobExecutorStarted.get()) {

            logger.warn(
                "[ALIPAYPLUS_LOG][JOBEXECUTOR_ALREADY_STARTED] jobExecutor has already been started. ");

            return false;

        } else {

            if (this.scheduleJobSupport == null || apsClient == null) {

                logger.error(
                    "[ALIPAYPLUS_LOG][JOBEXECUTOR_NOT_READY] JobExecutor not ready to start, missing apsClient or scheduleJobSupport injection ");

                throw new IllegalStateException(
                    "JobExecutor not ready to start. scheduleJobSupport, apsClient ");
            }

            executor = new ScheduledThreadPoolExecutor(
                apsClient.getApsSettings().getCorePoolSizeOfJobExecutor());

            if (jobExecutorStarted.compareAndSet(false, true)) {

                executor.scheduleWithFixedDelay(new Runnable() {

                    @Override
                    public void run() {
                        try {
                            ScheduleJob[] scheduleJobs = scheduleJobSupport.listJobs();

                            for (ScheduleJob job : scheduleJobs) {

                                if (job.getNextExecuteTime() > System.nanoTime()) {
                                    logger.debug(
                                        "[ALIPAYPLUS_LOG] job execute time does not arrive , jobId = {}, bizId={}",
                                        job.getJobId(), job.getBizId());
                                    continue;
                                }
                                try {
                                    //Execute this job right now. in a non-blocking way by submitting it to a executor.
                                    Runnable runnable = job2Task(job);

                                    if (runnable != null) {

                                        logger.debug("[ALIPAYPLUS_LOG] size of delay queue , size = {}",
                                            executor.getQueue().size());

                                        if (executor.getQueue().size() >= apsClient.getApsSettings()
                                            .getMaxSizeOFDelayQueue()) {
                                            logger.warn(
                                                "[ALIPAYPLUS_LOG] reaches max size of delay queue , jump this job , continue to handle next, jobId = {}, bizId={}",
                                                job.getJobId(), job.getBizId());
                                            continue;
                                        }

                                        logger.debug(
                                            "[ALIPAYPLUS_LOG] job executes , jobId = {}, bizId={}",
                                            job.getJobId(), job.getBizId());
                                        executor.execute(runnable);
                                    }
                                } catch (Exception e) {
                                    logger.error(
                                        "[ALIPAYPLUS_LOG][JOBEXECUTOR_RUNNING_TASK_ERROR] Exception when running tasks , jobId = {} , bizId={} , {} .",
                                        job.getJobId(), job.getBizId(), e);
                                }
                            }
                        } catch (Exception e) {

                            logger.error(
                                "[ALIPAYPLUS_LOG][JOBEXECUTOR_FETCH_TASK_ERROR] Exception when fetch tasks , {} .",
                                e);

                        }
                    }
                }, apsClient.getApsSettings().getInitDelay(), apsClient.getApsSettings().getDelay(),
                    TimeUnit.MILLISECONDS);

                logger.info(
                    "[ALIPAYPLUS_LOG] jobExecutor started. scheduleWithFixedDelay=[{}] MILLISECONDS",
                    apsClient.getApsSettings().getDelay());

                return true;

            } else {

                logger.warn(
                    "[ALIPAYPLUS_LOG][JOBEXECUTOR_ALREADY_STARTED] jobExecutor has already been started. ");

                return false;
            }

        }

    }

    /**
     * 
     * @param scheduleJob
     * @return Runnable
     */
    private Runnable job2Task(ScheduleJob scheduleJob) throws AlipayPlusApiException {

        BizType bizType = BizType.valueOf(scheduleJob.getBizType());

        switch (bizType) {
            case CANCEL:
                if (scheduledCancelCallback == null) {
                    logger.error(
                        "[ALIPAYPLUS_LOG][JOBEXECUTOR_SCHEDULE_CALLBACK_MISSING] scheduledCancelCallback is not injected");
                    throw new IllegalStateException("scheduledCancelCallback is required.");
                }
                CancelPaymentRequest cancelRequest = GsonUtils
                    .toJavaObject(scheduleJob.getBizContext(), CancelPaymentRequest.class);
                return new ScheduleRunnable(scheduleJobSupport, apsClient, cancelRequest,
                    scheduleJob, scheduledCancelCallback,
                    apsClient.getApsSettings().getCancelInterval());
            case REFUND:
                if (scheduledRefundCallback == null) {
                    logger.error(
                        "[ALIPAYPLUS_LOG][JOBEXECUTOR_SCHEDULE_CALLBACK_MISSING] scheduledRefundCallback is not injected");
                    throw new IllegalStateException("scheduledRefundCallback is required.");
                }
                RefundRequest refundRequest = GsonUtils.toJavaObject(scheduleJob.getBizContext(),
                    RefundRequest.class);
                return new ScheduleRunnable(scheduleJobSupport, apsClient, refundRequest,
                    scheduleJob, scheduledRefundCallback,
                    apsClient.getApsSettings().getRefundInterval());
            case INQUIRY:
                if (scheduledInquiryCallback == null) {
                    logger.error(
                        "[ALIPAYPLUS_LOG][JOBEXECUTOR_SCHEDULE_CALLBACK_MISSING] scheduledInquiryCallback is not injected");
                    throw new IllegalStateException("scheduledInquiryCallback is required.");
                }
                InquiryPaymentRequest inquiryRequest = GsonUtils
                    .toJavaObject(scheduleJob.getBizContext(), InquiryPaymentRequest.class);
                return new ScheduleRunnable(scheduleJobSupport, apsClient, inquiryRequest,
                    scheduleJob, scheduledInquiryCallback,
                    apsClient.getApsSettings().getInquiryIntervalForCashierpay());
            default:
                logger.error("[ALIPAYPLUS_LOG][JOBEXECUTOR_JOB_TYPE_UNKNOWN] bizType of Job is unknown");
                throw new AlipayPlusApiException(
                    String.format("bizType of Job is unknown . [jobId=%s , bizType=%s] ",
                        scheduleJob.getJobId(), scheduleJob.getBizType()));
        }

    }

    /**
     * Getter method for property <tt>jobExecutorStarted</tt>.
     *
     * @return property value of jobExecutorStarted
     */
    public static AtomicBoolean getJobExecutorStarted() {
        return jobExecutorStarted;
    }

    /**
     * Setter method for property <tt>jobExecutorStarted</tt>.
     *
     * @param jobExecutorStarted value to be assigned to property jobExecutorStarted
     */
    public static void setJobExecutorStarted(AtomicBoolean jobExecutorStarted) {
        APSScheduleJobExecutor.jobExecutorStarted = jobExecutorStarted;
    }

    /**
     * Getter method for property <tt>instance</tt>.
     *
     * @return property value of instance
     */
    public static APSScheduleJobExecutor getInstance() {
        return instance;
    }

    /**
     * Setter method for property <tt>scheduleJobSupport</tt>.
     *
     * @param scheduleJobSupport value to be assigned to property scheduleJobSupport
     */
    public void setScheduleJobSupport(ScheduleJobSupport scheduleJobSupport) {
        this.scheduleJobSupport = scheduleJobSupport;
    }

    /**
     * Getter method for property <tt>apsClient</tt>.
     *
     * @return property value of apsClient
     */
    public APSClient getApsClient() {
        return apsClient;
    }

    /**
     * Setter method for property <tt>apsClient</tt>.
     *
     * @param apsClient value to be assigned to property apsClient
     */
    public void setApsClient(APSClient apsClient) {
        this.apsClient = apsClient;
    }

    /**
     * Getter method for property <tt>scheduledInquiryCallback</tt>.
     *
     * @return property value of scheduledInquiryCallback
     */
    public HttpSendCallback getScheduledInquiryCallback() {
        return scheduledInquiryCallback;
    }

    /**
     * Setter method for property <tt>scheduledInquiryCallback</tt>.
     *
     * @param scheduledInquiryCallback value to be assigned to property scheduledInquiryCallback
     */
    public void setScheduledInquiryCallback(HttpSendCallback scheduledInquiryCallback) {
        this.scheduledInquiryCallback = scheduledInquiryCallback;
    }

    /**
     * Getter method for property <tt>scheduledRefundCallback</tt>.
     *
     * @return property value of scheduledRefundCallback
     */
    public HttpSendCallback getScheduledRefundCallback() {
        return scheduledRefundCallback;
    }

    /**
     * Setter method for property <tt>scheduledRefundCallback</tt>.
     *
     * @param scheduledRefundCallback value to be assigned to property scheduledRefundCallback
     */
    public void setScheduledRefundCallback(HttpSendCallback scheduledRefundCallback) {
        this.scheduledRefundCallback = scheduledRefundCallback;
    }

    /**
     * Getter method for property <tt>scheduledCancelCallback</tt>.
     *
     * @return property value of scheduledCancelCallback
     */
    public HttpSendCallback getScheduledCancelCallback() {
        return scheduledCancelCallback;
    }

    /**
     * Setter method for property <tt>scheduledCancelCallback</tt>.
     *
     * @param scheduledCancelCallback value to be assigned to property scheduledCancelCallback
     */
    public void setScheduledCancelCallback(HttpSendCallback scheduledCancelCallback) {
        this.scheduledCancelCallback = scheduledCancelCallback;
    }

}