/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.net;

import java.io.Closeable;
import java.io.IOException;
import java.util.Map;

import org.apache.commons.io.IOUtils;
import org.apache.http.HttpStatus;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.cfg.APSSettings;
import com.alipay.ap.constants.CommonConstants;
import com.alipay.ap.enums.ResultCode;
import com.alipay.ap.enums.ResultStatusType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.request.APSRequest;
import com.alipay.ap.response.APSResponse;
import com.alipay.ap.utils.HeaderUtil;
import com.alipay.ap.utils.SerializeUtil;
import com.alipay.ap.utils.SignatureTool;

/**
 * default implement of APSClient
 * 
 * @author mulin.xl
 * @version $Id: DefaultApacheHttpPostAPSClient.java, v 0.1 2022‐06‐23 13:26 mulin.xl Exp $$
 */
public class DefaultApacheHttpPostAPSClient extends APSClient implements Closeable {

    private static final Logger logger = LoggerFactory
        .getLogger(DefaultApacheHttpPostAPSClient.class);

    /** apache http client **/
    private CloseableHttpClient httpClient;

    /**
     * construction method 
     * 
     * @param cfg ap settings {@link APSSettings}
     */
    public DefaultApacheHttpPostAPSClient(APSSettings cfg) {
        super(cfg);

        PoolingHttpClientConnectionManager connManager = new PoolingHttpClientConnectionManager();
        connManager.setMaxTotal(cfg.getApacheMaxPoolSize());

        httpClient = HttpClients.custom().setConnectionManager(connManager)
            .setDefaultRequestConfig(RequestConfig.copy(RequestConfig.DEFAULT)
                .setSocketTimeout(cfg.getApacheHttpSocketTimeout())
                .setConnectTimeout(cfg.getApacheHttpConnectTimeout())
                .setConnectionRequestTimeout(cfg.getApacheHttpConnectionRequestTimeout()).build())
            .build();
    }

    /**
     * @see APSClient#execute(R, HttpSendCallback <R, P>)
     * @param request ap request {@link APSRequest}
     * @param callback {@link HttpSendCallback}
     * @param <R> request
     * @param <P> callback
     * @throws IOException
     * @throws AlipayPlusApiException
     * @throws Exception
     */
    @Override
    public <R extends APSRequest, P extends APSResponse> void execute(R request,
                                                                      HttpSendCallback<R, P> callback) throws IOException,
                                                                                                       AlipayPlusApiException,
                                                                                                       Exception {
        logger.debug("[ALIPAYPLUS_LOG][APSRequest] {}", request);

        HttpPost httpPost = null;
        CloseableHttpResponse response = null;

        try {
            //Step1: Construct the apache HttpPost client based on the configuration
            httpPost = new HttpPost(
                this.getApsSettings().getGatewayUrl() + request.getRequestURI());

            //Step2: Compose the HTTP request header
            this.composeHttpRequestHeader(request, httpPost);

            //Step3: Compose the HTTP request body
            this.composeHttpRequestBody(request, httpPost);

            //Step4: Execute the post request
            logger.info("[ALIPAYPLUS_LOG][HTTP_REQUEST][url = {} , headers = {} , body = {} ]",
                httpPost.getURI(), httpPost.getAllHeaders(), request.toJSONContent());
            response = httpClient.execute(httpPost);

            //Step5: Judge the http status code
            switch (response.getStatusLine().getStatusCode()) {

                case HttpStatus.SC_OK:

                    //200 status Deserialize the response
                    String rspBody = new String(
                        IOUtils.toByteArray(response.getEntity().getContent()),
                        CommonConstants.DEFAULT_CHARSET);

                    logger.info(
                        "[ALIPAYPLUS_LOG][HTTP_RESPONSE][httpStatus = 200 ,  header = {} , body = {}]",
                        response.getAllHeaders(), rspBody);

                    P apsResponse = SerializeUtil.deserializeResponse(request, rspBody,
                        HeaderUtil.toHeaderMap(response.getAllHeaders()));

                    logger.debug("[ALIPAYPLUS_LOG][APSResponse] {}", apsResponse);

                    onHttpStatus200(request, apsResponse, rspBody, callback);

                    break;

                default:
                    logger.info("[ALIPAYPLUS_LOG][HTTP_STATUS_NOT_200_ERROR][response = {}]",
                        response);
                    //Not 200 status throw AlipayPlusAPIException
                    throw new AlipayPlusApiException(
                        String.valueOf(response.getStatusLine().getStatusCode()),
                        "Not 200 HTTP status.", response.getStatusLine().getStatusCode());
            }

        } finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (httpPost != null) {
                httpPost.releaseConnection();
            }
        }

    }

    /**
     * @see Closeable#close()
     */
    @Override
    public void close() throws IOException {
        httpClient.close();
    }

    /**
     * on http status 200
     *
     * @param request ap request {@link APSRequest}
     * @param response ap response {@link APSResponse}
     * @param rspBody http response body
     * @param callback {@link HttpSendCallback}
     * @param <R> subclasses of {@link APSRequest}
     * @param <P> subclasses of {@link APSResponse}
     * @throws if alipay plus exception happens
     */
    protected <R extends APSRequest, P extends APSResponse> void onHttpStatus200(R request,
                                                                                 P response,
                                                                                 String rspBody,
                                                                                 HttpSendCallback<R, P> callback) throws AlipayPlusApiException,
                                                                                                                  CloneNotSupportedException,
                                                                                                                  Exception {

        boolean isVerify = false;
        //1. Signature verification.
        try {

            isVerify = SignatureTool.verify(request.getHttpMethod(), request.getRequestURI(),
                this.getApsSettings().getClientId(), response.getResponseHeader().getResponseTime(),
                rspBody, response.getResponseHeader().getSignature().getSignature(),
                this.getApsSettings().getAlipayPlusPublicKey());

        } catch (Exception e) {
            logger.error(
                "[ALIPAYPLUS_LOG][VERIFY_SIGNATURE_ERROR] exception happened when verify signature , signature={}",
                response.getResponseHeader().getSignature().getSignature());
            throw new AlipayPlusApiException(
                String.format("SignatureVerifyException[%s]: [requestURI=[%s]], responseBody=[%s]",
                    request.getClass().getName(), request.getRequestURI(), rspBody),
                e);

        }

        if (!isVerify) {
            logger.warn("[ALIPAYPLUS_LOG][INVALID_SIGNATURE] signature is invalid , signature={}",
                response.getResponseHeader().getSignature().getSignature());
            throw new AlipayPlusApiException(ResultCode.INVALID_SIGNATURE.name(),
                String.format("SignatureVerifyFailed[%s]: [requestURI=[%s]], responseBody=[%s]",
                    request.getClass().getName(), request.getRequestURI(), rspBody));
        }

        //2. Judge the result
        ResultStatusType resultStatus = response.getResult().getResultStatus();
        if (resultStatus == null) {
            logger.error(
                "[ALIPAYPLUS_LOG][INVALID_RESULT_STATUS] result.resultStatus is unknown , resultStatus={}",
                resultStatus);
            throw new AlipayPlusApiException(String.format(
                "the result.resultStatus of AlipayPlus response is unknown [%s]: [requestURI=[%s]], responseBody=[%s]",
                request.getClass().getName(), request.getRequestURI(), rspBody));
        }
        switch (resultStatus) {
            case F:
                callback.onFstatus(this, request, response);

                break;
            case U:
                callback.onUstatus(this, request, response);

                break;
            case S:
                callback.onSstatus(this, request, response);

                break;
            default:
                logger.error(
                    "[ALIPAYPLUS_LOG][INVALID_RESULT_STATUS] result.resultStatus is invalid , resultStatus={}",
                    resultStatus);
                throw new AlipayPlusApiException(String.format(
                    "the result.resultStatus of AlipayPlus response is invalid [%s]: [requestURI=[%s]], responseBody=[%s]",
                    request.getClass().getName(), request.getRequestURI(), rspBody));
        }

    }

    /**
     * compose http request header
     *  
     * @param request   subclasses of {@link APSRequest}
     * @param httpPost apache http post {@link HttpPost}
     * @throws Exception if common exception happens
     */
    private <R extends APSRequest> void composeHttpRequestHeader(R request,
                                                                 HttpPost httpPost) throws Exception {

        //Step1: Compose the HTTP request header
        //Step1.1: Compose the common request header
        request.getRequestHeader().setClientId(this.getApsSettings().getClientId());
        //Step1.2: Compose the request signature
        String signature;
        try {
            signature = SignatureTool.sign(request.getHttpMethod(), request.getRequestURI(),
                this.getApsSettings().getClientId(), request.getRequestHeader().getRequestTime(),
                request.toJSONContent(), this.getApsSettings().getPrivateKey());
        } catch (Exception e) {
            logger.error(
                "[ALIPAYPLUS_LOG][SIGN_EXCEPTION] exception happened when sign the request", e);
            throw new AlipayPlusApiException(String.format(
                "SignException: [httpMethod=[%s]], requestURI=[%s], clientId=[%s], requestTime=[%s], requestJson=[%s], privateKey=[%s]",
                request.getHttpMethod(), request.getRequestURI(),
                this.getApsSettings().getClientId(), request.getRequestHeader().getRequestTime(),
                request.toJSONContent(), this.getApsSettings().getPrivateKey()), e);
        }
        request.getRequestHeader().getSignature().setSignature(signature);
        //Step1.3: Set header info into httpPost
        for (Map.Entry<String, String> e : request.getRequestHeader().toMap().entrySet()) {
            httpPost.setHeader(e.getKey(), e.getValue());
        }

    }

    /**
     * compose http request body
     *
     * @param request   subclasses of {@link APSRequest}
     * @param httpPost apache http post {@link HttpPost}
     * @throws Exception if common exception happens
     */
    private <R extends APSRequest> void composeHttpRequestBody(R request,
                                                               HttpPost httpPost) throws Exception {

        httpPost.setEntity(
            new ByteArrayEntity(request.toJSONContent().getBytes(CommonConstants.DEFAULT_CHARSET)));

    }

}