/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.receive;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.callbacks.HttpReceiveCallback;
import com.alipay.ap.cfg.APSSettings;
import com.alipay.ap.constants.CommonConstants;
import com.alipay.ap.enums.HttpMethod;
import com.alipay.ap.enums.ResultCode;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.HttpReceive;
import com.alipay.ap.model.RequestHeader;
import com.alipay.ap.model.ResponseHeader;
import com.alipay.ap.response.HttpReceiveResponse;
import com.alipay.ap.utils.GsonUtils;
import com.alipay.ap.utils.SignatureTool;

/**
 * Default APSHttp Receive Listener
 * @author mulin.xl
 * @version $Id: DefaultAPSHttpReceiveListener.java, v 0.1 2022‐07‐13 11:58 mulin.xl Exp $$
 */
public class DefaultAPSHttpReceiveListener extends APSHttpReceiveListener {
    /** logger */
    private static final Logger logger = LoggerFactory
        .getLogger(DefaultAPSHttpReceiveListener.class);

    public DefaultAPSHttpReceiveListener(APSSettings apsSettings) {
        super(apsSettings);
    }

    @Override
    public HttpReceiveResponse onReceive(HttpReceive httpReceive,
                                         HttpReceiveCallback httpReceiveCallback) throws AlipayPlusApiException,
                                                                                  Exception {
        // 1. Deserialize receiveRequestHeader
        RequestHeader requestHeader = new RequestHeader();
        requestHeader.fromMap(httpReceive.getReceiveRequestHeader());

        // 2.Deserialize receiveRequestBody
        String receiveReqBody = new String(httpReceive.getReceiveReqBody(),
            CommonConstants.DEFAULT_CHARSET);

        // 3. Signature verification.
        this.verifySignature(requestHeader, httpReceive.getRequestURI(), receiveReqBody);

        HttpReceiveResponse httpReceiveResponse = httpReceiveCallback.doProcess(httpReceive);

        this.composeResponseHeader(httpReceive, httpReceiveResponse);

        return httpReceiveResponse;

    }

    /**
     * verify sign
     *
     * @param requestHeader
     * @param requestURI
     * @param notifyReqBody
     * @throws AlipayPlusApiException
     */
    private void verifySignature(RequestHeader requestHeader, String requestURI,
                                 String notifyReqBody) throws AlipayPlusApiException {

        if (this.getApsSettings().isDevMode()) {
            return;
        }

        boolean isVerify = false;

        try {

            isVerify = SignatureTool.verify(HttpMethod.POST.name(), requestURI,
                requestHeader.getClientId(), requestHeader.getRequestTime(), notifyReqBody,
                requestHeader.getSignature().getSignature(),
                this.getApsSettings().getAlipayPlusPublicKey());

        } catch (Exception e) {
            logger.error(
                "[ALIPAYPLUS_LOG][VERIFY_SIGNATURE_ERROR] exception happened when verify signature , e={}",
                e);
            throw new AlipayPlusApiException(
                String.format("SignatureVerifyException: [requestURI=[%s]], receiveReqBody=[%s]",
                    requestURI, notifyReqBody),
                e);

        }

        if (!isVerify) {
            logger.warn("[ALIPAYPLUS_LOG][INVALID_SIGNATURE] signature is invalid");
            throw new AlipayPlusApiException(ResultCode.INVALID_SIGNATURE.name(),
                String.format("SignatureVerifyFailed: [requestURI=[%s]], receiveReqBody=[%s]",
                    requestURI, notifyReqBody));
        }
    }

    /**
     *  compose responseHeader for httpReceiveResponse
     *
     * @param httpReceive
     * @param httpReceiveResponse
     * @throws Exception
     */
    public void composeResponseHeader(HttpReceive httpReceive,
                                      HttpReceiveResponse httpReceiveResponse) throws Exception {
        ResponseHeader responseHeader = httpReceiveResponse.getResponseHeader();
        responseHeader.setClientId(
            httpReceive.getReceiveRequestHeader().get(CommonConstants.CLIENT_ID_HEADER));
        String signature;
        try {
            signature = SignatureTool.sign(HttpMethod.POST.name(), httpReceive.getRequestURI(),
                responseHeader.getClientId(), responseHeader.getResponseTime(),
                GsonUtils.toJSONString(httpReceiveResponse), this.getApsSettings().getPrivateKey());
        } catch (Exception e) {
            logger.error(
                "[ALIPAYPLUS_LOG][SIGN_EXCEPTION] exception happened when sign the response to A+",
                e);
            throw new AlipayPlusApiException(String.format(
                "SignException: [httpMethod=[%s]], requestURI=[%s], clientId=[%s], requestTime=[%s], httpReceiveResponse=[%s], privateKey=[%s]",
                HttpMethod.POST.name(), httpReceive.getRequestURI(), responseHeader.getClientId(),
                responseHeader.getResponseTime(), GsonUtils.toJSONString(httpReceiveResponse),
                this.getApsSettings().getPrivateKey()), e);
        }
        responseHeader.getSignature().setSignature(signature);
    }
}