/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.utils;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.TimeZone;
import java.util.concurrent.TimeUnit;

import javax.xml.bind.DatatypeConverter;

import org.apache.commons.lang3.time.DateFormatUtils;

/**
 * util class that deals with date
 * 
 * @author mulin.xl
 * @version $Id: DateUtil.java, v 0.1 2019‐10-16 16:25:49 mulin.xl Exp $
 */
public class DateUtil {

    /** minimize length of date time string with iso format (has time zone like +08:00) */
    private final static int    ISO_DATE_LENGTH       = 25;

    /** minimize length of date time string with iso format (has time zone Z) */
    private final static int    UTC_0_ISO_DATE_LENGTH = 20;

    /** char that present timezone 0 */
    private final static String UTC_0_TIMEZONE        = "Z";

    /** ISO format of date **/
    public final static String  ISO8601               = "yyyy-MM-dd'T'HH:mm:ssZZZ";

    /** ISO format of date **/
    public final static String  SIMPLE                = "yyyyMMddHHmmss";

    /**
     * get now time with iso8601 format
     * 
     * @return current time 
     */
    public static String getCurISO8601Time() {
        return DateFormatUtils.format(new Date().getTime(), ISO8601);
    }

    /**
     * get now time with yyyyMMddHHmmss
     *
     * @return current time 
     */
    public static String getSimpleFormatTime() {
        return DateFormatUtils.format(new Date().getTime(), SIMPLE);
    }

    /**
     * get date time string with iso format
     * 
     * @param date date time object
     * @return date time string
     */
    public static String getISODateTimeStr(Date date) {
        return getISODateTimeStr(date, null);
    }

    /**
     * get date time string in some timezone with iso8601 format 
     * 
     * @param date date time object
     * @param timeZone timezone 
     * @return date time string
     */
    public static String getISODateTimeStr(Date date, TimeZone timeZone) {
        if (date == null) {
            return null;
        }

        Calendar cal = new GregorianCalendar();
        if (timeZone != null) {
            cal.setTimeZone(timeZone);
        }
        cal.setTime(date);
        cal.set(Calendar.MILLISECOND, 0);
        return DatatypeConverter.printDateTime(cal);
    }

    /**
     * get date time object  
     *
     * @param isoDateStr  date time string
     * @return date time object
     */
    public static Date parseISODateTimeStr(String isoDateStr) {
        if (isoDateStr == null || "".equals(isoDateStr)) {
            return null;
        }

        int minDateStrLength;
        // UTC 0时区字符串，时区标识是字符Z，不是+00:00
        if (isoDateStr.endsWith(UTC_0_TIMEZONE)) {
            minDateStrLength = UTC_0_ISO_DATE_LENGTH;

        } else {
            minDateStrLength = ISO_DATE_LENGTH;
        }

        if (isoDateStr.length() < minDateStrLength) {
            return null;
        }

        Calendar cal = DatatypeConverter.parseDate(isoDateStr);
        return cal.getTime();
    }

    /**
     * Returns the trigger time of a delayed action.
     */
    public static long triggerTime(long delay, TimeUnit unit) {
        return triggerTime(unit.toNanos((delay < 0) ? 0 : delay));
    }

    /**
     * Returns the trigger time of a delayed action.
     */
    public static long triggerTime(long delay) {
        return now() + delay;
    }

    /**
     * Returns current nanosecond time.
     */
    public static final long now() {
        return System.nanoTime();
    }
}
