/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example;

import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultApplyTokenCallback;
import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.enums.GrantType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.ApplyTokenRequest;
import com.alipay.ap.response.ApplyTokenResponse;

/**
 * After retrieving the auth code, you need to request an access token from Alipay+ with the auth code.
 * After receiving the access token from your server side, your client side then binds the access token with the user account
 * and updates the payment method page or add payment method page to display the newly bound wallet to the user.
 * The access token is also used by your client side to initiate payments later.
 * (refer to doc: [Acquiring Partner] https://docs.alipayplus.com/alipayplus/)
 * @author wqy01239909
 * @version $Id: ApplyTokenDemoCode.java, v 0.1 2022‐07-11 16:19 wqy01239909 Exp $
 */
public class ApplyTokenDemoCode {

    private static final Logger logger = LoggerFactory.getLogger(ApplyTokenDemoCode.class);

    public static void main(String[] args) {

        ApplyTokenDemoCode demoCode = new ApplyTokenDemoCode();

        demoCode.applyToken(DefaultAPS.getInstance().getApsClient(),
            demoCode.buildApplyTokenRequest(), demoCode.buildApplyTokenCallBack());
    }

    private void applyToken(APSClient apsClient, ApplyTokenRequest applyTokenRequest,
                            HttpSendCallback applyTokenCallback) {
        try {
            //TODO: ACQ handling the apply token request before calling A+ api

            //Calling ApsClient.execute to initiate a apply token
            apsClient.execute(applyTokenRequest, applyTokenCallback);

        } catch (IOException e) {

            // Below scenes raise the IOException when calling ApsClient.execute
            // #1 Calling A+ api when occurs the http connection issue
            // #2 Calling A+ api when occurs client protocol issue

            logger.error("[IOException occurred] applyTokenRequest={} , stackError={}",
                applyTokenRequest, e.getMessage());

            // TODO: Handling IOException in ACQ

        } catch (AlipayPlusApiException e) {

            // Below scenes raise the AlipayPlusApiException when calling ApsClient.execute
            // #1 Calling A+ api when respond non 200 http status code. Such as 400/404/500...
            // #2 Calling A+ api when occurs the signature verification failure

            logger.error(
                "[AlipayPlusApiException occurred] applyTokenRequest={},httpCode={},errorCode={},errorMsg={},stackError={}",
                applyTokenRequest, e.getCode(), e.getErrCode(), e.getErrMsg(), e.getMessage());

            // TODO: Handling AlipayPlusApiException in ACQ

        } catch (Exception e) {

            // unexpected/unknown exception occurs during the whole apply token process
            logger.error(
                "[Unknown or Unexpected Exception occurred] applyTokenRequest={},stackError={}",
                applyTokenRequest, e.getMessage());

            // TODO: Handling Unknown/Unexpected Exception in ACQ

        }
    }

    private ApplyTokenRequest buildApplyTokenRequest() {
        ApplyTokenRequest applyTokenRequest = new ApplyTokenRequest();
        // The unique ID that is assigned by the ACQP to identify the auth client, which is usually the merchant.
        applyTokenRequest.setAuthClientId("2188123412341233");
        applyTokenRequest.setGrantType(GrantType.AUTHORIZATION_CODE);
        // TODO: After the auth client (which is usually the merchant) receives the auth code, the ACQP requests the access token from Alipay+.
        // In this case, the applyToken API generally needs to be used with the prepare API.
        applyTokenRequest.setAuthCode("281010133AB2F588D14B432312345233"); // Required if the value of the grantType parameter is AUTHORIZATION_CODE.
        return applyTokenRequest;
    }

    private HttpSendCallback buildApplyTokenCallBack() {
        return new DefaultApplyTokenCallback() {
            @Override
            public void onApplyTokenSuccess(APSClient apsClient, ApplyTokenRequest request,
                                            ApplyTokenResponse response) throws Exception {
                logger.debug("[onApplyTokenSuccess] applyTokenRequest={},applyTokenResponse={}",
                    request, response);
                // The access token has been generated by Alipay+ successfully.
                // TODO: The Acquiring Partner sends the access token to the merchant, who then proceeds to the next action.

            }

            @Override
            public void onApplyTokenFail(APSClient apsClient, ApplyTokenRequest request,
                                         ApplyTokenResponse response) throws Exception {
                logger.debug("[onApplyTokenFail] applyTokenRequest={},applyTokenResponse={}",
                    request, response);
                // The access token application failed.
                // TODO: The Acquiring Partner syncs the result with the merchant, and the merchant needs to guide the user to restart the account binding process.

            }

            @Override
            public void onApplyTokenUnknown(APSClient apsClient, ApplyTokenRequest request,
                                            ApplyTokenResponse response) throws Exception {
                logger.debug("[onApplyTokenUnknown] applyTokenRequest={},applyTokenResponse={}",
                    request, response);
                // The result of the access token application is unknown.
                // TODO:The Acquiring Partner syncs the result with the merchant and waits for the authorization notification from Alipay+.
                // TODO:The merchant can either wait for the authorization notification from Alipay+ via the Acquiring Partner, or guide the user to restart the account binding process.

            }
        };
    }
}