/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultAuthNotifyCallback;
import com.alipay.ap.callbacks.HttpReceiveCallback;
import com.alipay.ap.cfg.APSSettings;
import com.alipay.ap.constants.CommonConstants;
import com.alipay.ap.enums.HttpReceiveType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.HttpReceive;
import com.alipay.ap.model.Result;
import com.alipay.ap.receive.APSHttpReceiveListener;
import com.alipay.ap.request.AuthNotifyRequest;
import com.alipay.ap.response.HttpReceiveResponse;
import com.alipay.ap.utils.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * After processing the applyToken API request successfully, Alipay+ asynchronously calls the authNotify API
 * to send a notification to the URL that you specified on the authNotifyUrl parameter of the prepare API request.
 * The notification also contains the access token. After receiving the notification, your server side needs to notify your client side of the access token.
 * (refer to doc: [Acquiring Partner] https://docs.alipayplus.com/alipayplus/)
 * @author wqy01239909
 * @version $Id: AuthNotifyDemoCode.java, v 0.1 2022‐06-27 14:39 wqy01239909 Exp $
 */
public class AuthNotifyDemoCode {

    private static final Logger logger = LoggerFactory.getLogger(AuthNotifyDemoCode.class);

    public static void main(String[] args) throws IOException {

        AuthNotifyDemoCode demoCode = new AuthNotifyDemoCode();

        // get DefaultAPSHttpReceiveListener（If you have customization logic, you can self-implement APSHttpReceiveListener）
        APSHttpReceiveListener listener = DefaultAPS.getInstance().getApsHttpReceiveListener();

        demoCode.receiveAuthNotify(listener, demoCode.buildHttpReceive(listener.getApsSettings()),
            demoCode.buildDefaultAuthNotifyCallback());

    }

    public void receiveAuthNotify(APSHttpReceiveListener listener, HttpReceive httpReceive,
                                  HttpReceiveCallback httpReceiveCallback) {
        try {

            // call onReceive method
            HttpReceiveResponse httpReceiveResponse = listener.onReceive(httpReceive,
                httpReceiveCallback);
            logger.debug("[receiveAuthNotify] response={}", httpReceiveResponse);
            //TODO write responseHeaders and responseContent back to the Http response to AlipayPlus.

        } catch (IOException e) {

            logger.error("[IOException occurred] stackError={}", e.getMessage());

            // TODO: Handling IOException in ACQ

        } catch (AlipayPlusApiException e) {

            logger.error("[AlipayPlusApiException occurred] stackError={}", e.getMessage());

            // TODO: Handle AlipayPlusApiException according to error messages
            // TODO: compose response according to errorCode to AlipayPlus

        } catch (Exception e) {
            // unexpected/unknown exception occurs during the whole prepare process
            logger.error("[Unknown or Unexpected Exception occurred] stackError={}",
                e.getMessage());
            // TODO Handle UnknownException
            // TODO compose unknown response to AlipayPlus

        }

    }

    /**
     * 1、ACQ provides a Web service to receive HTTP requests from AlipayPlus
     * 2、ACQ convert HTTP requests to HttpReceive
     * 
     * @return
     * @throws UnsupportedEncodingException
     * @param apsSettings
     */
    private HttpReceive buildHttpReceive(APSSettings apsSettings) throws UnsupportedEncodingException {
        //TODO: Request header from AlipayPlus notify
        Map<String, String> hashMap = new HashMap<>();
        hashMap.put(CommonConstants.REQ_SIGN_HEADER, "test_sign");
        hashMap.put(CommonConstants.CONTENT_TYPE_HEADER, "application/json; charset=UTF-8");
        hashMap.put(CommonConstants.REQ_TIME_HEADER, DateUtil.getCurISO8601Time());
        hashMap.put(CommonConstants.CLIENT_ID_HEADER, apsSettings.getClientId());
        HttpReceive httpReceive = new HttpReceive();
        httpReceive.setReceiveRequestHeader(hashMap);
        String reqBody = "{\n" + "  \"authorizationNotifyType\": \"AUTHCODE_CREATED\",\n"
                         + "  \"authClientId\": \"218823863726123456789\",\n"
                         + "  \"referenceMerchantId\": \"218823863726123456780\",\n"
                         + "  \"authCode\": \"281010133AB2F588D14B432312345678\",\n"
                         + "  \"authState\": \"663A8FA9-D836-48EE-8AA1-1FF682989DC7\",\n"
                         + "  \"referenceAgreementId\": \"aNDJWQNNabdad1234\",\n"
                         + "  \"acquirerId\": \"1021234567891230001\",\n"
                         + "  \"pspId\": \"1021234567891230002\"\n" + "}";
        httpReceive.setReceiveReqBody(reqBody.getBytes(CommonConstants.DEFAULT_CHARSET));
        httpReceive.setRequestURI("/auth/notify");
        httpReceive.setReceiveType(HttpReceiveType.AUTH_NOTIFY);
        return httpReceive;
    }

    /**
     * ACQ build Callback which deals with auth notify
     * And assemble the response returned to AlipayPlus
     *
     * Note: If you received the notification successfully, set the value of result.resultStatus to S and return an HTTP status code of 200
     * to indicate that your server received the call. Otherwise, Alipay+ takes the notification delivery as unsuccessful and retries the notification
     * sending up to 7 times. The intervals between two consecutive times are as follows: 2min, 10min, 10min, 1h, 2h, 6h, and 15h.
     * @return
     */
    private DefaultAuthNotifyCallback buildDefaultAuthNotifyCallback() {
        return new DefaultAuthNotifyCallback() {
            @Override
            public HttpReceiveResponse onTokenCreated(AuthNotifyRequest request) {
                logger.debug("[onTokenCreated] request={}", request);
                // indicates that the access token is created by the MPP.
                // TODO: Use accessToken to debit account
                return new HttpReceiveResponse(Result.S);
            }

            @Override
            public HttpReceiveResponse onTokenCanceled(AuthNotifyRequest request) {
                logger.debug("[onTokenCanceled] request={}", request);
                // indicate the access token in the request has been deactivated by the Mobile Payment Partner.
                // TODO: The merchant cannot use the access token for any further interactions.
                return new HttpReceiveResponse(Result.S);
            }

            @Override
            public HttpReceiveResponse onAuthCodeCreated(AuthNotifyRequest request) {
                logger.debug("[onAuthCodeCreated] request={}", request);
                //  indicates that the authorization code is created
                // TODO:  Use the auth code to obtain the access token
                return new HttpReceiveResponse(Result.S);
            }
        };

    }

}