/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example;

import java.io.IOException;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultCancelTokenCallback;
import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.CancelTokenRequest;
import com.alipay.ap.response.CancelTokenResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * After receiving the account unbinding request from the merchant,
 * the ACQP needs to call the cancelToken interface from Alipay+ to apply to revoke the specified access token.
 * (refer to doc: [Acquiring Partner] https://docs.alipayplus.com/alipayplus/)
 * @author wqy01239909
 * @version $Id: CancelTokenDemoCode.java, v 0.1 2022‐07-11 16:19 wqy01239909 Exp $
 */
public class CancelTokenDemoCode {
    private static final Logger logger = LoggerFactory.getLogger(CancelTokenDemoCode.class);

    public static void main(String[] args) {

        CancelTokenDemoCode demoCode = new CancelTokenDemoCode();

        demoCode.cancelToken(DefaultAPS.getInstance().getApsClient(),
            demoCode.buildCancelTokenRequest(), demoCode.buildCancelTokenCallBack());
    }

    private void cancelToken(APSClient apsClient, CancelTokenRequest cancelTokenRequest,
                             HttpSendCallback cancelTokenCallback) {
        try {
            //TODO: ACQ handling the cancel token request before calling A+ api

            //Calling ApsClient.execute to initiate a cancel token
            apsClient.execute(cancelTokenRequest, cancelTokenCallback);

            //TODO: ACQ handling the cancel token request after calling A+ api
        } catch (IOException e) {

            // Below scenes raise the IOException when calling ApsClient.execute
            // #1 Calling A+ api when occurs the http connection issue
            // #2 Calling A+ api when occurs client protocol issue

            logger.error("[IOException occurred] cancelTokenRequest={},stackError={}",
                cancelTokenRequest, e.getMessage());

            // TODO: Handling IOException in ACQ

        } catch (AlipayPlusApiException e) {

            // Below scenes raise the AlipayPlusApiException when calling ApsClient.execute
            // #1 Calling A+ api when respond non 200 http status code. Such as 400/404/500...
            // #2 Calling A+ api when occurs the signature verification failure

            logger.error(
                "[AlipayPlusApiException occurred] cancelTokenRequest={},httpCode={},errorCode={},errorMsg={},stackError={}",
                cancelTokenRequest, e.getCode(), e.getErrCode(), e.getErrMsg(), e.getMessage());

            // TODO: Handling AlipayPlusApiException in ACQ

        } catch (Exception e) {

            // unexpected/unknown exception occurs during the whole apply token process
            logger.error(
                "[Unknown or Unexpected Exception occurred] cancelTokenRequest={},stackError={}",
                cancelTokenRequest, e.getMessage());

            // TODO: Handling Unknown/Unexpected Exception in ACQ

        }
    }

    private CancelTokenRequest buildCancelTokenRequest() {
        CancelTokenRequest cancelTokenRequest = new CancelTokenRequest();
        // specifies the access token that is to be revoked.
        // The Acquiring Partner needs to retrieve the specified access token from the account unbinding request sent by the merchant.
        cancelTokenRequest.setAccessToken("281010033AB2F588D14B43238637264FCA5A1234");
        // specifies the unique identifier for the auth client. Normally it refers to the merchant.
        cancelTokenRequest.setAuthClientId("2188123412341233");
        return cancelTokenRequest;
    }

    private HttpSendCallback buildCancelTokenCallBack() {
        return new DefaultCancelTokenCallback() {
            @Override
            public void onCancelTokenSuccess(APSClient apsClient, CancelTokenRequest request,
                                             CancelTokenResponse response) {
                logger.debug("[onCancelTokenSuccess] cancelTokenRequest={},cancelTokenResponse={}",
                    request, response);

                // The authorization is successfully canceled. The access token and the related refresh token can no longer be used.

            }

            @Override
            public void onCancelTokenFail(APSClient apsClient, CancelTokenRequest request,
                                          CancelTokenResponse response) {
                logger.debug("[onCancelTokenFail] cancelTokenRequest={},cancelTokenResponse={}",
                    request, response);
                // TODO Take actions for cancel token fail.
                // TODO Retry the API call.

            }

            @Override
            public void onCancelTokenUnknown(APSClient apsClient, CancelTokenRequest request,
                                             CancelTokenResponse response) throws Exception {
                logger.debug("#[onCancelTokenUnknown] cancelTokenRequest={},cancelTokenResponse={}",
                    request, response);

                // TODO Take actions for cancel token unknown.
                // TODO Retry the API call.
            }
        };
    }
}