/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultConsultUnbindingCallback;
import com.alipay.ap.callbacks.HttpReceiveCallback;
import com.alipay.ap.cfg.APSSettings;
import com.alipay.ap.constants.CommonConstants;
import com.alipay.ap.enums.HttpReceiveType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.HttpReceive;
import com.alipay.ap.model.Result;
import com.alipay.ap.receive.APSHttpReceiveListener;
import com.alipay.ap.request.ConsultUnbindingRequest;
import com.alipay.ap.response.ConsultUnbindingResponse;
import com.alipay.ap.response.HttpReceiveResponse;
import com.alipay.ap.utils.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * The ACQP needs to accept the request of the consultUnbinding interface from Alipay+, collect the required information from the merchant,
 * and then return whether the merchant allows the user account to be unbound from the MPP side.
 * (refer to doc: [Acquiring Partner] https://docs.alipayplus.com/alipayplus/)
 * @author wqy01239909
 * @version $Id: ConsultUnbindingDemoCode.java, v 0.1 2022‐06-27 14:39 wqy01239909 Exp $
 */
public class ConsultUnbindingDemoCode {

    private static final Logger logger = LoggerFactory.getLogger(ConsultUnbindingDemoCode.class);

    public static void main(String[] args) throws IOException {

        ConsultUnbindingDemoCode demoCode = new ConsultUnbindingDemoCode();
        // get DefaultAPSHttpReceiveListener（If you have customization logic, you can self-implement APSHttpReceiveListener）
        APSHttpReceiveListener listener = DefaultAPS.getInstance().getApsHttpReceiveListener();

        demoCode.receiveConsultUnbinding(listener,
            demoCode.buildHttpReceive(listener.getApsSettings()),
            demoCode.buildDefaultConsultUnbindingCallback());
    }

    public void receiveConsultUnbinding(APSHttpReceiveListener listener, HttpReceive httpReceive,
                                        HttpReceiveCallback httpReceiveCallback) {
        try {

            // call onReceive method
            HttpReceiveResponse httpReceiveResponse = listener.onReceive(httpReceive,
                httpReceiveCallback);
            //TODO write responseHeaders and responseContent back to the Http response to AlipayPlus.

        } catch (IOException e) {

            logger.error("[IOException occurred] stackError={}", e.getMessage());

            // TODO: Handling IOException in ACQ

        } catch (AlipayPlusApiException e) {

            logger.error("[AlipayPlusApiException occurred] stackError={}", e.getMessage());

            // TODO: Handle AlipayPlusApiException according to error messages
            // TODO: compose response according to errorCode to AlipayPlus

        } catch (Exception e) {
            // unexpected/unknown exception occurs during the whole prepare process
            logger.error("[Unknown or Unexpected Exception occurred] stackError={}",
                e.getMessage());
            // TODO Handle UnknownException
            // TODO compose unknown response to AlipayPlus

        }

    }

    /**
     * 1、ACQ provides a Web service to receive HTTP requests from AlipayPlus
     * 2、ACQ convert HTTP requests to HttpReceive
     *
     * Note:The MPP needs to return the result (specified on the result parameter) according to the business processing result.
     * If you received the notification successfully, set the value of result.resultStatus to S and return an HTTP status code of 200 to
     * indicate that your server received the call. Otherwise, Alipay+ takes the notification delivery as unsuccessful and retries the
     * notification sending up to 7 times. The intervals between two consecutive times are as follows: 2min, 10min, 10min, 1h, 2h, 6h,
     * and 15h.
     * @return
     * @throws UnsupportedEncodingException
     * @param apsSettings
     */
    private HttpReceive buildHttpReceive(APSSettings apsSettings) throws UnsupportedEncodingException {
        //TODO: Request header from AlipayPlus notify
        Map<String, String> hashMap = new HashMap<>();
        hashMap.put(CommonConstants.CLIENT_ID_HEADER, apsSettings.getClientId());
        hashMap.put(CommonConstants.CONTENT_TYPE_HEADER, "application/json; charset=UTF-8");
        hashMap.put(CommonConstants.REQ_SIGN_HEADER, "test_sign");
        hashMap.put(CommonConstants.REQ_TIME_HEADER, DateUtil.getCurISO8601Time());
        HttpReceive httpReceive = new HttpReceive();
        httpReceive.setReceiveRequestHeader(hashMap);
        String reqBody = "{\n" + "  \"authClientId\": \"21888211212100\",\n"
                         + "  \"referenceMerchantId\": \"21888211212101\",\n"
                         + "  \"accessToken\": \"2810120412122ojsalksa000\",\n"
                         + "  \"acquirerId\": \"102218800000000000\",\n"
                         + "  \"pspId\": \"102208800000000000\"\n" + "}";
        httpReceive.setReceiveReqBody(reqBody.getBytes(CommonConstants.DEFAULT_CHARSET));
        httpReceive.setRequestURI("/consult/unbinding");
        httpReceive.setReceiveType(HttpReceiveType.CONSULT_UNBINDING);
        return httpReceive;
    }

    /**
     * ACQ build Callback which deals with consult unbinding
     * And assemble the response returned to AlipayPlus
     *
     * @return
     */
    private DefaultConsultUnbindingCallback buildDefaultConsultUnbindingCallback() {

        return new DefaultConsultUnbindingCallback() {
            @Override
            public ConsultUnbindingResponse handleConsultUnbinding(ConsultUnbindingRequest request) {

                // TODO: Take action to compose the consult unbinding response to A+
                ConsultUnbindingResponse consultUnbindingResponse = new ConsultUnbindingResponse();
                consultUnbindingResponse.setResult(Result.S);
                consultUnbindingResponse.setAllowUnbinding(false);
                consultUnbindingResponse.setRefuseReason("merchant not allow unbinding");

                logger.debug("consult unbinding request ->[{}], response ->[{}]", request,
                    consultUnbindingResponse);
                return consultUnbindingResponse;
            }
        };
    }

}