/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example;

import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultInquiryPaymentCallback;
import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.enums.ResultCode;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.InquiryPaymentRequest;
import com.alipay.ap.response.InquiryPaymentResponse;
import com.alipay.ap.utils.GsonUtils;
import com.alipay.ap.utils.StringUtil;

/**
 * The Acquiring Partner (ACQP) can call the inquiryPayment API to inquire about the payment result at the Mobile Payment Partner side.
 * When  you (the ACQP) cannot receive the payment result notification due to network or system problems,
 * the inquiryPayment API ensures you get the final payment result.
 * (refer to doc: [Acquiring Partner] https://docs.alipayplus.com/alipayplus/)
 * @author wqy01239909
 * @version $Id: InquiryPaymentDemoCode.java, v 0.1 2022‐06-30 10:41 wqy01239909 Exp $
 */
public class InquiryPaymentDemoCode {
    private static final Logger logger = LoggerFactory.getLogger(InquiryPaymentDemoCode.class);

    public static void main(String[] args) {
        InquiryPaymentDemoCode demoCode = new InquiryPaymentDemoCode();

        demoCode.inquiryPayment(DefaultAPS.getInstance().getApsClient(),
            demoCode.buildInquiryRequest(), demoCode.buildInquiryPaymentCallback());
    }

    private InquiryPaymentRequest buildInquiryRequest() {
        InquiryPaymentRequest inquiryPaymentRequest = new InquiryPaymentRequest();
        inquiryPaymentRequest.setPaymentRequestId(StringUtil.genRequestID("payment"));
        return inquiryPaymentRequest;
    }

    public void inquiryPayment(APSClient apsClient, InquiryPaymentRequest inquiryPaymentRequest,
                               HttpSendCallback acqInquiryPaymentCallbackSample) {
        try {
            //TODO: ACQ handling the inquiryPayment request before calling A+ api

            //Calling ApsClient.execute to initiate a inquiryPayment
            apsClient.execute(inquiryPaymentRequest, acqInquiryPaymentCallbackSample);

        } catch (IOException e) {

            // Below scenes raise the IOException when calling ApsClient.execute
            // #1 Calling A+ api when occurs the http connection issue
            // #2 Calling A+ api when occurs client protocol issue

            logger.error("[IOException occurred] inquiryPaymentRequest={},stackError={}",
                inquiryPaymentRequest, e.getMessage());

            // TODO: Handling IOException in ACQ

        } catch (AlipayPlusApiException e) {

            // Below scenes raise the AlipayPlusApiException when calling ApsClient.execute
            // #1 Calling A+ api when respond non 200 http status code. Such as 400/404/500...
            // #2 Calling A+ api when occurs the signature verification failure

            logger.error(
                "[AlipayPlusApiException occurred] inquiryPaymentRequest={},httpCode={},errorCode={},errorMsg={},stackError={}",
                inquiryPaymentRequest, e.getCode(), e.getErrCode(), e.getErrMsg(), e.getMessage());

            // TODO: Handling AlipayPlusApiException in ACQ

        } catch (Exception e) {

            // unexpected/unknown exception occurs during the whole inquiryPayment process
            logger.error(
                "[Unknown or Unexpected Exception occurred] inquiryPaymentRequest={},stackError={}",
                inquiryPaymentRequest, e.getMessage());

            // TODO: Handling Unknown/Unexpected Exception in ACQ

        }
    }

    private HttpSendCallback buildInquiryPaymentCallback() {

        return new DefaultInquiryPaymentCallback() {
            @Override
            public void onPaymentInquiryFail(APSClient apsClient, InquiryPaymentRequest request,
                                             InquiryPaymentResponse response) throws Exception {
                logger.debug(
                    "[onPaymentInquiryFail][isJobSchedule={}] inquiryPaymentResponse={},inquiryPaymentResponse={}",
                    request.isJobSchedule(), request, response);

                if (ResultCode.ORDER_NOT_EXIST == ResultCode
                    .getByCode(response.getResult().getResultCode())) {
                    // For the cases where the order does not exist (result.resultStatus == F && resultcode == ORDER_NOT_EXIST)
                    // keep querying for 10 minutes with an incremental frequency

                    /** implement the callback if this request is executed asynchronously */
                    // TODO If keep receiving the same result, call cancel payment interface.
                    // TODO register the schedule for inquiry payment retry

                    /** implement the callback if this request is executed synchronously */
                    // TODO Take actions for inquiry payment fail.
                    // TODO  register the schedule for inquiry payment retry
                    // TODO Response the result to upstream system synchronously.

                } else {
                    // For the cases where the payment request fails (result.resultStatus == F && resultcode != ORDER_NOT_EXIST), 
                    // TODO take actions according to the error message in result.resultCode or contact connect_support@service.alipay.com.

                }

            }

            @Override
            public void onPaymentInquiryUnknown(APSClient apsClient, InquiryPaymentRequest request,
                                                InquiryPaymentResponse response) throws Exception {
                logger.debug(
                    "[onPaymentInquiryUnknown][isJobSchedule={}] inquiryPaymentResponse={} , inquiryPaymentResponse={}",
                    request.isJobSchedule(), request, response);

                /** implement the callback if this request is executed asynchronously */
                // TODO If keep receiving the same result , Take actions for inquiry payment unknown 
                // TODO register the schedule for inquiry payment retry

                /** implement the callback if this request is executed synchronously */
                // TODO Take actions for inquiry payment unknown.
                // TODO register the schedule for inquiry payment retry
                // TODO Response the result to upstream system synchronously.

            }

            @Override
            public void onPaymentStatusSuccess(APSClient apsClient, InquiryPaymentRequest request,
                                               InquiryPaymentResponse response) {
                logger.debug(
                    "[onPaymentStatusSuccess][isJobSchedule={}] inquiryPaymentResponse={} , inquiryPaymentResponse={}",
                    request.isJobSchedule(), request, response);

                /** implement the callback if this request is executed asynchronously */
                // TODO Update the status from your server side.
                // TODO Notify the result to upstream/partner system

                /** implement the callback if this request is executed synchronously */
                // TODO Update the status from your server side.
                // TODO Response the result to upstream system synchronously.

            }

            @Override
            public void onPaymentStatusFail(APSClient apsClient, InquiryPaymentRequest request,
                                            InquiryPaymentResponse response) {
                logger.info("[onPaymentStatusFail][isJobSchedule={}] inquiryPaymentResponse={}",
                    request.isJobSchedule(), GsonUtils.toJSONString(response));

                /** If the paymentResult.resultCode value is ORDER_IS_CLOSED, the order is closed or cancelled; otherwise, the payment fails.*/

                /** implement the callback if this request is executed asynchronously */
                // TODO: Take actions according to the error message in result.resultCode
                // TODO Notify the result to upstream/partner system and The merchant can reinitiate a payment request in a new API call

                /** implement the callback if this request is executed synchronously */
                // TODO: Take actions according to the error message in result.resultCode
                // TODO Response the result to upstream system synchronously and The merchant can reinitiate a payment request in a new API call
            }

            @Override
            public void onPaymentStatusProcessing(APSClient apsClient,
                                                  InquiryPaymentRequest request,
                                                  InquiryPaymentResponse response) throws Exception {

                logger.info(
                    "[onPaymentStatusProcessing] , [isJobSchedule={}] , inquiryPaymentResponse->[{}]",
                    request.isJobSchedule(), GsonUtils.toJSONString(response));

                /** implement the callback if this request is executed asynchronously */
                // TODO call the cancelPayment interface for payment status is unknown after reaching the max retry times
                // TODO register the schedule for inquiry payment retry
                /** It is recommended to call the inquiryPayment API in the form of polling: keep querying for 10 minutes with an incremental frequency, after your server side gets the response of the pay API.*/

                /** implement the callback if this request is executed synchronously */
                // TODO register the schedule for inquiry payment retry
                /** It is recommended to call the inquiryPayment API in the form of polling: keep querying for 10 minutes with an incremental frequency, after your server side gets the response of the pay API.*/
                // TODO Response the result to upstream system synchronously.
            }
        };

    }
}