/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example;

import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultInquiryRegistrationStatusCallback;
import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.MerchantRegistrationStatusInquiryRequest;
import com.alipay.ap.response.MerchantInquiryRegistrationStatusResponse;

/**
 *
 * When using the inquiryRegistrationStatus API, the ACQP needs to take the following things into consideration:
 *
 * 1. The ACQP can call the API in a fixed frequency, for example, once a day, to obtain the latest registration status until all the MPPs
 * return the registration results.
 * 2. The ACQP can query the registration result of the corresponding request by specifying the value of registrationRequestId, or query the
 * registration result of the corresponding merchant by specifying the value of registrationMerchantId.
 * 3. In the response returned by Alipay+, the ACQP can check both the whole registration status of all the MPPs to be registered to, and
 * the registration status of a certain MPP.
 * (refer to doc: [Acquiring Partner] https://docs.alipayplus.com/alipayplus/)
 * @author wqy01239909
 * @version $Id: InquiryRegistrationStatusDemoCode.java, v 0.1 2022‐06-30 14:11 wqy01239909 Exp $
 */
public class InquiryRegistrationStatusDemoCode {
    private static final Logger logger = LoggerFactory
        .getLogger(InquiryRegistrationStatusDemoCode.class);

    public static void main(String[] args) {

        APSClient apsClient = DefaultAPS.getInstance().getApsClient();

        InquiryRegistrationStatusDemoCode demoCode = new InquiryRegistrationStatusDemoCode();

        MerchantRegistrationStatusInquiryRequest registrationStatusInquiryRequest = demoCode
            .buildInquiryRegistrationStatusRequest();
        HttpSendCallback inquiryRegistrationStatusCallback = demoCode
            .buildInquiryRegistrationStatusCallback();

        demoCode.registration(apsClient, registrationStatusInquiryRequest,
            inquiryRegistrationStatusCallback);
    }

    private void registration(APSClient apsClient,
                              MerchantRegistrationStatusInquiryRequest registrationStatusInquiryRequest,
                              HttpSendCallback registrationCallback) {
        try {
            //TODO: ACQ handling the inquiry registration status request before calling A+ api

            //Calling ApsClient.execute to initiate a inquiry registration status request
            apsClient.execute(registrationStatusInquiryRequest, registrationCallback);

        } catch (IOException e) {

            // Below scenes raise the IOException when calling ApsClient.execute
            // #1 Calling A+ api when occurs the http connection issue
            // #2 Calling A+ api when occurs client protocol issue

            logger.error("[IOException occurred] registrationStatusInquiryRequest={},stackError={}",
                registrationStatusInquiryRequest, e.getMessage());

            // TODO: Handling IOException in ACQ

        } catch (AlipayPlusApiException e) {

            // Below scenes raise the AlipayPlusApiException when calling ApsClient.execute
            // #1 Calling A+ api when respond non 200 http status code. Such as 400/404/500...
            // #2 Calling A+ api when occurs the signature verification failure

            logger.error(
                "[AlipayPlusApiException occurred] registrationStatusInquiryRequest={},httpCode={},errorCode={},errorMsg={},stackError={}",
                registrationStatusInquiryRequest, e.getCode(), e.getErrCode(), e.getErrMsg(),
                e.getMessage());

            // TODO: Handling AlipayPlusApiException in ACQ

        } catch (Exception e) {

            // unexpected/unknown exception occurs during the whole consult payment process
            logger.error(
                "[Unknown or Unexpected Exception occurred] registrationStatusInquiryRequest={},stackError={}",
                registrationStatusInquiryRequest, e.getMessage());

            // TODO: Handling Unknown/Unexpected Exception in ACQ

        }
    }

    private HttpSendCallback buildInquiryRegistrationStatusCallback() {
        return new DefaultInquiryRegistrationStatusCallback() {
            @Override
            public void onInquiryRegistrationStatusSuccess(APSClient apsClient,
                                                           MerchantRegistrationStatusInquiryRequest request,
                                                           MerchantInquiryRegistrationStatusResponse response) {
                logger.debug(
                    "[onInquiryRegistrationStatusSuccess] merchantRegistrationStatusInquiryRequest={},merchantInquiryRegistrationStatusResponse={}",
                    request, response);

                // 1. TODO According to the registrationResult、pspRegistrationResultList do processing
                // 3. TODO Response the result to upstream system synchronously.

            }

            @Override
            public void onInquiryRegistrationStatusFail(APSClient apsClient,
                                                        MerchantRegistrationStatusInquiryRequest request,
                                                        MerchantInquiryRegistrationStatusResponse response) {
                logger.debug(
                    "[onInquiryRegistrationStatusFail] merchantRegistrationStatusInquiryRequest={},merchantInquiryRegistrationStatusResponse={}",
                    request, response);

                // 1. TODO Take actions according to the error code in result.resultCode.
                // 2. TODO Response the result to upstream system synchronously.
            }

            @Override
            public void onInquiryRegistrationStatusUnknown(APSClient apsClient,
                                                           MerchantRegistrationStatusInquiryRequest request,
                                                           MerchantInquiryRegistrationStatusResponse response) throws Exception {
                logger.debug(
                    "[onInquiryRegistrationStatusUnknown] merchantRegistrationStatusInquiryRequest={},merchantInquiryRegistrationStatusResponse={}",
                    request, response);

                // 1. TODO Take actions for inquiryRegistrationStatus unknown. (Try to recall the API.)
                // 2. TODO Response the result to upstream system synchronously.
            }
        };
    }

    private MerchantRegistrationStatusInquiryRequest buildInquiryRegistrationStatusRequest() {
        // TODO: When specifying the request parameters, the ACQP needs to refer to the A+ documentation
        MerchantRegistrationStatusInquiryRequest registrationStatusInquiryRequest = new MerchantRegistrationStatusInquiryRequest();
        registrationStatusInquiryRequest.setReferenceMerchantId("21881200001912124");
        return registrationStatusInquiryRequest;
    }
}