/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultPaymentNotifyCallback;
import com.alipay.ap.callbacks.HttpReceiveCallback;
import com.alipay.ap.cfg.APSSettings;
import com.alipay.ap.constants.CommonConstants;
import com.alipay.ap.enums.HttpReceiveType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.HttpReceive;
import com.alipay.ap.model.Result;
import com.alipay.ap.receive.APSHttpReceiveListener;
import com.alipay.ap.request.PayNotifyRequest;
import com.alipay.ap.response.HttpReceiveResponse;
import com.alipay.ap.utils.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * from the payment notification that Alipay+ asynchronously sends to the Acquiring Partner by using the notifyPayment interface.
 * (refer to doc: [Acquiring Partner] https://docs.alipayplus.com/alipayplus/)
 * @author wqy01239909
 * @version $Id: PaymentNotifyDemoCode.java, v 0.1 2022‐06-27 14:39 wqy01239909 Exp $
 */
public class PaymentNotifyDemoCode {

    private static final Logger logger = LoggerFactory.getLogger(PaymentNotifyDemoCode.class);

    public static void main(String[] args) throws IOException {
        PaymentNotifyDemoCode demoCode = new PaymentNotifyDemoCode();

        // get DefaultAPSHttpReceiveListener（If you have customization logic, you can self-implement APSHttpReceiveListener）
        APSHttpReceiveListener listener = DefaultAPS.getInstance().getApsHttpReceiveListener();

        demoCode.receivePaymentNotify(listener,
            demoCode.buildHttpReceive(listener.getApsSettings()),
            demoCode.buildDefaultPaymentNotifyCallback());

    }

    public void receivePaymentNotify(APSHttpReceiveListener listener, HttpReceive httpReceive,
                                     HttpReceiveCallback httpReceiveCallback) {
        try {

            // call onReceive method
            HttpReceiveResponse httpReceiveResponse = listener.onReceive(httpReceive,
                httpReceiveCallback);
            logger.debug("response={}", httpReceiveResponse);
            //TODO write responseHeaders and responseContent back to the Http response to AlipayPlus.

        } catch (IOException e) {

            logger.error("[IOException occurred] stackError={}", e.getMessage());

            // TODO: Handling IOException in ACQ

        } catch (AlipayPlusApiException e) {

            logger.error("[AlipayPlusApiException occurred] stackError={}", e.getMessage());

            // TODO: Handle AlipayPlusApiException according to error messages
            // TODO: compose response according to errorCode to AlipayPlus

        } catch (Exception e) {
            // unexpected/unknown exception occurs during the whole prepare process
            logger.error("[Unknown or Unexpected Exception occurred] stackError={}",
                e.getMessage());
            // TODO Handle UnknownException
            // TODO compose unknown response to AlipayPlus

        }
    }

    /**
     * 1、ACQ provides a Web service to receive HTTP requests from AlipayPlus
     * 2、ACQ convert HTTP requests to HttpReceive
     *
     * @return
     * @throws UnsupportedEncodingException
     * @param apsSettings
     */
    private HttpReceive buildHttpReceive(APSSettings apsSettings) throws UnsupportedEncodingException {
        //TODO: Request header from AlipayPlus notify
        Map<String, String> hashMap = new HashMap<>();
        hashMap.put(CommonConstants.CONTENT_TYPE_HEADER, "application/json; charset=UTF-8");
        hashMap.put(CommonConstants.REQ_TIME_HEADER, DateUtil.getCurISO8601Time());
        hashMap.put(CommonConstants.CLIENT_ID_HEADER, apsSettings.getClientId());
        hashMap.put(CommonConstants.REQ_SIGN_HEADER, "test_sign");
        HttpReceive httpReceive = new HttpReceive();
        httpReceive.setReceiveRequestHeader(hashMap);
        String reqBody = "{\n" + "    \"acquirerId\": \"1111088000000000002\",\n"
                         + "    \"pspId\": \"1022172000000000001\",\n"
                         + "    \"paymentResult\": {\n" + "        \"resultCode\": \"SUCCESS\",\n"
                         + "        \"resultStatus\": \"U\",\n"
                         + "        \"resultMessage\": \"success\"\n" + "    },\n"
                         + "    \"paymentRequestId\": \"pay_1089760038715669_102775745075669\",\n"
                         + "    \"paymentId\": \"20200101234567890134567\",\n"
                         + "    \"paymentTime\": \"2020-01-01T12:01:01+08:30\",\n"
                         + "    \"paymentAmount\": {\n" + "        \"value\": \"100\",\n"
                         + "        \"currency\": \"JPY\"\n" + "    },\n"
                         + "    \"customerId\": \"1235678\",\n"
                         + "    \"walletBrandName\": \"KAKAOPAY\"\n" + "}";
        httpReceive.setReceiveReqBody(reqBody.getBytes(CommonConstants.DEFAULT_CHARSET));
        httpReceive.setRequestURI("/payment/notify");
        httpReceive.setReceiveType(HttpReceiveType.PAYMENT_NOTIFY);
        return httpReceive;
    }

    /**
     * ACQ build Callback which deals with payment notify
     * And assemble the response returned to AlipayPlus
     *
     * Note: If you received the notification successfully, set the value of result.resultStatus to S and return an HTTP status code of 200
     * to indicate that your server received the call. Otherwise, Alipay+ takes the notification delivery as unsuccessful and retries the notification
     * sending up to 7 times. The intervals between two consecutive times are as follows: 2min, 10min, 10min, 1h, 2h, 6h, and 15h.
     * @return
     */
    private DefaultPaymentNotifyCallback buildDefaultPaymentNotifyCallback() {

        return new DefaultPaymentNotifyCallback() {

            @Override
            public HttpReceiveResponse onPaymentSuccess(PayNotifyRequest request) {
                logger.debug("[onPaymentSuccess] request={}", request);
                //TODO: Take actions for PaymentSuccess.
                return new HttpReceiveResponse(Result.S);

            }

            @Override
            public HttpReceiveResponse onPaymentFailure(PayNotifyRequest request) {
                logger.debug("[onPaymentFailure] request={}", request);
                //TODO: Take actions for PaymentFailure.
                //TODO: ADD more comments on this step mulin.xl
                return new HttpReceiveResponse(Result.S);
            }
        };
    }

}