/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultPrepareCallback;
import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.enums.OsType;
import com.alipay.ap.enums.Scope;
import com.alipay.ap.enums.TerminalType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.Amount;
import com.alipay.ap.model.SettlementStrategy;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.PrepareRequest;
import com.alipay.ap.response.PrepareResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Your server side needs to integrate the prepare API provided by Alipay+ to obtain the URL of the Alipay+ wallet page and return it to the client side.
 * (refer to doc: [Acquiring Partner] https://docs.alipayplus.com/alipayplus/)
 * @author wqy01239909
 * @version $Id: PrepareDemoCode.java, v 0.1 2022‐07-08 17:41 wqy01239909 Exp $
 */
public class PrepareDemoCode {
    private static final Logger logger = LoggerFactory.getLogger(PrepareDemoCode.class);

    public static void main(String[] args) {

        PrepareDemoCode prepareDemoCode = new PrepareDemoCode();

        prepareDemoCode.prepare(DefaultAPS.getInstance().getApsClient(),
            prepareDemoCode.buildPrepareRequest(), prepareDemoCode.buildPrepareCallback());
    }

    public void prepare(APSClient apsClient, PrepareRequest prepareRequest,
                        HttpSendCallback prepareCallback) {
        try {

            //TODO: ACQ handling the prepare request before calling A+ api

            //Calling ApsClient.execute to initiate a prepare
            apsClient.execute(prepareRequest, prepareCallback);

        } catch (IOException e) {

            // Below scenes raise the IOException when calling ApsClient.execute
            // #1 Calling A+ api when occurs the http connection issue
            // #2 Calling A+ api when occurs client protocol issue

            logger.error("[IOException occurred] prepareRequest={},stackError={}", prepareRequest,
                e.getMessage());

            // TODO: Handling IOException in ACQ

        } catch (AlipayPlusApiException e) {

            // Below scenes raise the AlipayPlusApiException when calling ApsClient.execute
            // #1 Calling A+ api when respond non 200 http status code. Such as 400/404/500...
            // #2 Calling A+ api when occurs the signature verification failure

            logger.error(
                "[AlipayPlusApiException occurred] prepareRequest={},httpCode={},errorCode={},errorMsg={},stackError={}",
                prepareRequest, e.getCode(), e.getErrCode(), e.getErrMsg(), e.getMessage());

            // TODO: Handling AlipayPlusApiException in ACQ

        } catch (Exception e) {

            // unexpected/unknown exception occurs during the whole prepare process
            logger.error(
                "[Unknown or Unexpected Exception occurred] prepareRequest={},stackError={}",
                prepareRequest, e.getMessage());

            // TODO: Handling Unknown/Unexpected Exception in ACQ

        }

    }

    private PrepareRequest buildPrepareRequest() {
        PrepareRequest prepareRequest = new PrepareRequest();
        SettlementStrategy settlementStrategy = new SettlementStrategy();
        settlementStrategy.setSettlementCurrency("JPY");
        prepareRequest.setSettlementStrategy(settlementStrategy);
        Amount amount = new Amount();
        amount.setCurrency("CNY");
        amount.setValue(123);
        prepareRequest.setPaymentAmount(amount);
        // The unique ID that is assigned by the ACQP to identify the auth client, which is the merchant.
        // The value of this parameter must be the same as that of the referenceMerchantId parameter.
        prepareRequest.setAuthClientId("2188123412341230");
        // The registered legal name of the auth client. The name usually appears with the agreement on the authorization page.
        prepareRequest.setAuthClientName("Merchant");
        // The unique ID that is assigned by the ACQP to identify a merchant.
        prepareRequest.setReferenceMerchantId("2188123412341230");
        // The first part of the redirection URL that the user is redirected to after the user agrees to authorize.
        // The complete URL consists of the values of the authRedirectUrl, the authCode, and the authState parameter.
        prepareRequest.setAuthRedirectUrl(
            "https://www.merchant.com/authenticationResult?param1=123&param2=234");
        // The authorization scopes, which means what resources or capabilities are authorized to the auth client.
        List<Scope> scopes = new ArrayList<>();
        // AGREEMENT_PAY: indicates that the auth client can use an access token to deduct funds from the user's MPP app for Auto Debit payments.
        scopes.add(Scope.AGREEMENT_PAY);
        // USER_LOGIN_ID: indicates that the auth client can obtain the desensitized user login ID for the MPP app by using the applyToken API.
        scopes.add(Scope.USER_LOGIN_ID);
        prepareRequest.setScopes(scopes);
        // A string that is generated by the auth client to represent the prepare API request.
        // The consistency of this field and that in the redirection URL needs to be guaranteed when the authorization code is returned.
        prepareRequest.setAuthState("663A8FA9-D836-48EE-8AA1-1FF682989DC7");
        prepareRequest.setTerminalType(TerminalType.APP);
        // The unique ID that is assigned by the auth client to identify an authorization.
        // The ACQP needs to map the referenceAgreementId with the related authorization information so as to obtain the related authorization information from the authNotify API request.
        prepareRequest.setReferenceAgreementId("aNDJWQNNabdad****");
        prepareRequest.setOsType(OsType.ANDROID);
        prepareRequest.setOsVersion("6.0.2");
        // The URL that ACQP uses to receive the authorization information from Alipay+ via the authNotify API.
        // Notes：
        //Required if the ACQP wants to receive the authorization notification from Alipay+.
        //For security reasons, The URL must use HTTPS instead of HTTP.
        prepareRequest.setAuthNotifyUrl("https://www.merchant.com/authenticationNotification");
        return prepareRequest;
    }

    private HttpSendCallback buildPrepareCallback() {
        return new DefaultPrepareCallback() {
            @Override
            public void onPrepareSuccess(APSClient apsClient, PrepareRequest request,
                                         PrepareResponse response) throws Exception {
                logger.debug("[onPrepareSuccess] prepareRequest={},prepareResponse={}", request,
                    response);
                // An authorization URL has been returned from A+,
                // TODO: the merchant needs to redirect the user to the authorization URL so that the user can proceed with the account binding process.

            }

            @Override
            public void onPrepareFail(APSClient apsClient, PrepareRequest request,
                                      PrepareResponse response) throws Exception {
                logger.debug("[onPrepareFail] prepareRequest={},prepareResponse={}", request,
                    response);
                // The authorization preparation failed at Alipay+ side.
                // TODO: Take actions according to the error message in result.resultCode.

            }

            @Override
            public void onPrepareUnknown(APSClient apsClient, PrepareRequest request,
                                         PrepareResponse response) throws Exception {
                logger.debug("[onPrepareUnknown] prepareRequest={},prepareResponse={}", request,
                    response);
                // TODO: The Acquiring Partner needs to retry the prepare request with the same request parameter values until a certain S or F result is received.

            }
        };
    }
}