/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example;

import java.io.IOException;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultRefundPaymentCallback;
import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.Amount;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.RefundRequest;
import com.alipay.ap.response.RefundResponse;
import com.alipay.ap.utils.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A payment can be refunded any time after the transaction is successfully paid and before the refund period expires.
 * The refund expiration period is usually one calendar year, starting from when the initial payment is made.
 * (refer to doc: [Acquiring Partner] https://docs.alipayplus.com/alipayplus/)
 * @author wangpeng
 * @version $Id: RefundPaymentDemoCode.java, v 0.1 2022‐06-27 16:24 wangpeng Exp $
 */
public class RefundPaymentDemoCode {

    private static final Logger logger = LoggerFactory.getLogger(RefundPaymentDemoCode.class);

    public static void main(String[] args) throws Exception {

        HttpSendCallback callback = buildRefundPaymentCallBack();

        RefundPaymentDemoCode demo = new RefundPaymentDemoCode();

        demo.refund(DefaultAPS.getInstance().getApsClient(), demo.buildRefundRequest(), callback);
    }

    public void refund(APSClient apsClient, RefundRequest refundRequest,
                       HttpSendCallback callback) throws AlipayPlusApiException {

        try {
            apsClient.execute(refundRequest, callback);
        } catch (IOException e) {
            // Below scenes raise the IOException when calling ApsClient.execute
            // #1 Calling A+ api when occurs the http connection issue
            // #2 Calling A+ api when occurs client protocol issue

            logger.error("[IOException occurred] refundRequest]={},stackError={}", refundRequest,
                e.getMessage());

            // TODO: Handling IOException in ACQ
        } catch (AlipayPlusApiException e) {
            // Below scenes raise the AlipayPlusApiException when calling ApsClient.execute
            // #1 Calling A+ api when respond non 200 http status code. Such as 400/404/500...
            // #2 Calling A+ api when occurs the signature verification failure

            logger.error(
                "[AlipayPlusApiException occurred] refundRequest={},httpCode={},errorCode={},errorMsg={},stackError={}",
                refundRequest, e.getCode(), e.getErrCode(), e.getErrMsg(), e.getMessage());

            // TODO: Handling AlipayPlusApiException in ACQ

        } catch (Exception e) {
            // unexpected/unknown exception occurs during the whole payment process
            logger.error(
                "[Unknown or Unexpected Exception occurred] refundRequest={},stackError={}",
                refundRequest, e.getMessage());

            // TODO: Handling Unknown/Unexpected Exception in ACQ

        }
    }

    private RefundRequest buildRefundRequest() {

        RefundRequest refundRequest = new RefundRequest();
        Amount refundAmount = new Amount();
        refundAmount.setCurrency("PHP");
        refundAmount.setValue(30000);
        refundRequest.setRefundAmount(refundAmount);
        refundRequest.setRefundReason("refund reason");
        refundRequest.setRefundRequestId(StringUtil.genRequestID("refund"));
        refundRequest.setPaymentRequestId(StringUtil.genRequestID("payment"));

        return refundRequest;
    }

    private static HttpSendCallback buildRefundPaymentCallBack() {

        return new DefaultRefundPaymentCallback() {

            @Override
            public void onRefundPaymentSuccess(APSClient apsClient, RefundRequest request,
                                               RefundResponse response) {
                logger.debug(
                    "[onRefundPaymentSuccess][isJobSchedule={}] refundPaymentRequest={},refundPaymentPayResponse={}",
                    request.isJobSchedule(), request, response);

                /** implement the callback if this request is executed asynchronously   */
                // 1. TODO Update the order status in your side to the value such as refunded.
                // 2. TODO Notify the result to upstream system/partner system.

                /** implement the callback if this request is executed synchronously   */
                // 1. TODO Update the order status in your side to the value such as refunded.
                // 2. TODO Response the result to upstream system synchronously.
            }

            @Override
            public void onRefundPaymentFail(APSClient apsClient, RefundRequest request,
                                            RefundResponse response) {
                logger.debug(
                    "[onPaymentRefundFail][isJobSchedule={}] refundPaymentRequest={},refundPaymentPayResponse={}",
                    request.isJobSchedule(), request, response);

                /** implement the callback if this request is executed asynchronously   */
                // 1. TODO Take actions according to the error message in the result.resultMessage parameter.
                // 2. TODO Notify the result to upstream system/partner system.

                /** implement the callback if this request is executed asynchronously   */
                // 1. TODO Take actions according to the error message in the result.resultMessage parameter.
                // 2. TODO Response the result to upstream system synchronously.
            }

            @Override
            public void onRefundPaymentUnknown(APSClient apsClient, RefundRequest request,
                                               RefundResponse response) throws Exception {
                logger.debug(
                    "#[onRefundPaymentUnknown][isJobSchedule={}] refundPaymentRequest={},refundPaymentPayResponse={}",
                    request.isJobSchedule(), request, response);

                /** implement the callback if this request is executed asynchronously */
                // TODO Retry the same request until getting the response. Ensure that the parameters must be the same with the previous one */
                // TODO Take actions for refund payment unknown.
                // TODO if retry max , contact connect_support@service.alipay.com.

                /** implement the callback if this request is executed synchronously */
                // TODO Retry the same request until getting the response. Ensure that the parameters must be the same with the previous one */
                // TODO Take actions for refund payment unknown.
                // TODO Response the result to upstream system synchronously.

            }
        };
    }

}