/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultRegistrationCallback;
import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.enums.ProductCodeType;
import com.alipay.ap.enums.RegistrationType;
import com.alipay.ap.enums.WebsiteType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.Address;
import com.alipay.ap.model.MerchantRegistrationInfo;
import com.alipay.ap.model.RegistrationDetail;
import com.alipay.ap.model.WebSite;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.MerchantRegistrationRequest;
import com.alipay.ap.response.MerchantRegistrationResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * By calling the registration API, the ACQP can either register a new merchant or update an existing registration,
 * such as modify merchant information and add payment products. Note that only one merchant or store can be registered or updated at a time.
 * (refer to doc: [Acquiring Partner] https://docs.alipayplus.com/alipayplus/)
 * @author wqy01239909
 * @version $Id: RegistrationDemoCode.java, v 0.1 2022‐06-30 14:11 wqy01239909 Exp $
 */
public class RegistrationDemoCode {
    private static final Logger logger = LoggerFactory.getLogger(RegistrationDemoCode.class);

    public static void main(String[] args) {

        APSClient apsClient = DefaultAPS.getInstance().getApsClient();

        RegistrationDemoCode demoCode = new RegistrationDemoCode();

        MerchantRegistrationRequest registrationRequest = demoCode.buildRegistrationRequest();
        HttpSendCallback registrationCallback = demoCode.buildRegistrationCallback();

        demoCode.registration(apsClient, registrationRequest, registrationCallback);
    }

    private void registration(APSClient apsClient,
                              MerchantRegistrationRequest merchantRegistrationRequest,
                              HttpSendCallback registrationCallback) {
        try {
            //TODO: ACQ handling the registration request before calling A+ api

            //Calling ApsClient.execute to initiate a registration
            apsClient.execute(merchantRegistrationRequest, registrationCallback);

            //TODO: ACQ handling the registration request after calling A+ api
        } catch (IOException e) {

            // Below scenes raise the IOException when calling ApsClient.execute
            // #1 Calling A+ api when occurs the http connection issue
            // #2 Calling A+ api when occurs client protocol issue

            logger.error("[IOException occurred] merchantRegistrationRequest={},stackError={}",
                merchantRegistrationRequest, e.getMessage());

            // TODO: Handling IOException in ACQ

        } catch (AlipayPlusApiException e) {

            // Below scenes raise the AlipayPlusApiException when calling ApsClient.execute
            // #1 Calling A+ api when respond non 200 http status code. Such as 400/404/500...
            // #2 Calling A+ api when occurs the signature verification failure

            logger.error(
                "[AlipayPlusApiException occurred] merchantRegistrationRequest={},httpCode={},errorCode={},errorMsg={},stackError={}",
                merchantRegistrationRequest, e.getCode(), e.getErrCode(), e.getErrMsg(),
                e.getMessage());

            // TODO: Handling AlipayPlusApiException in ACQ

        } catch (Exception e) {

            // unexpected/unknown exception occurs during the whole consult payment process
            logger.error(
                "[Unknown or Unexpected Exception occurred] merchantRegistrationRequest={},stackError={}",
                merchantRegistrationRequest, e.getMessage());

            // TODO: Handling Unknown/Unexpected Exception in ACQ

        }
    }

    private HttpSendCallback buildRegistrationCallback() {
        return new DefaultRegistrationCallback() {
            @Override
            public void onRegistrationSuccess(APSClient apsClient,
                                              MerchantRegistrationRequest request,
                                              MerchantRegistrationResponse response) throws Exception {
                logger.debug(
                    "[onRegistrationSuccess] merchantRegistrationRequest={},merchantRegistrationResponse={}",
                    request, response);

                /** if result.resultStatus is S, it only means the registration request is successfully accepted, but does not mean the registration succeeds. */
                //TODO: The ACQ inquires the merchant registration status or waits for notifications from Alipay+
            }

            @Override
            public void onRegistrationFail(APSClient apsClient, MerchantRegistrationRequest request,
                                           MerchantRegistrationResponse response) throws Exception {
                logger.debug(
                    "[onRegistrationFail] merchantRegistrationRequest={},merchantRegistrationResponse={}",
                    request, response);
                // TODO The ACQ modifies the merchant information or updates the value of productCode then sends the merchant registration request to Alipay+  
            }

            @Override
            public void onRegistrationUnknown(APSClient apsClient,
                                              MerchantRegistrationRequest request,
                                              MerchantRegistrationResponse response) throws Exception {
                logger.debug(
                    "[onRegistrationUnknown] merchantRegistrationRequest={},merchantRegistrationResponse={}",
                    request, response);
                // TODO The ACQ needs to send the request again until the API call failed or succeeds.

            }
        };
    }

    private MerchantRegistrationRequest buildRegistrationRequest() {
        final MerchantRegistrationRequest request = new MerchantRegistrationRequest();

        //TODO build your merchant info request
        request.setRegistrationRequestId("register_202009181105860200000600142224");

        request.setProductCodes(Collections.singletonList(ProductCodeType.CASHIER_PAYMENT));
        request.setPassThroughInfo("{\"extraInfo\":\"extra\"}");
        request.setRegistrationNotifyURL("https://merchant/example");

        MerchantRegistrationInfo merchant = new MerchantRegistrationInfo();

        merchant.setMerchantDisplayName("Example Merchant Name");
        merchant.setMerchantMCC("5812");

        RegistrationDetail detail = new RegistrationDetail();
        detail.setLegalName("Example Legal Name");
        detail.setRegistrationNo("registration*****");
        detail.setRegistrationType(RegistrationType.ENTERPRISE_REGISTRATION_NO);

        Address registrationAddress = new Address();
        registrationAddress.setRegion("HK");
        registrationAddress.setAddress1("香港...");
        detail.setRegistrationAddress(registrationAddress);
        detail.setBusinessType("ENTERPRISE");
        merchant.setRegistrationDetail(detail);

        List<WebSite> webSites = new ArrayList<>();
        WebSite webSite = new WebSite();
        webSite.setUrl("http://www.webSite.com");
        webSite.setDesc("this is webSite desc");
        webSite.setName("webName");
        webSite.setWebsiteType(WebsiteType.APP);
        webSites.add(webSite);
        merchant.setWebsites(webSites);

        merchant.setReferenceMerchantId("21881200001912124");
        request.setMerchantInfo(merchant);

        return request;
    }
}