/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultRegistrationStatusNotifyCallback;
import com.alipay.ap.callbacks.HttpReceiveCallback;
import com.alipay.ap.cfg.APSSettings;
import com.alipay.ap.constants.CommonConstants;
import com.alipay.ap.enums.HttpReceiveType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.HttpReceive;
import com.alipay.ap.model.Result;
import com.alipay.ap.receive.APSHttpReceiveListener;
import com.alipay.ap.request.MerchantRegistrationStatusNotifyRequest;
import com.alipay.ap.response.HttpReceiveResponse;
import com.alipay.ap.utils.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * If the ACQP wants to receive notifications from Alipay+ about registration results,
 * the ACQP needs to integrate the notifyRegistrationStatus API. Each time Alipay+ receives a notification from the MPP,
 * Alipay+ will notify the ACQP of the registration result by calling the API.
 * (refer to doc: [Acquiring Partner] https://docs.alipayplus.com/alipayplus/)
 * @author wqy01239909
 * @version $Id: RegistrationStatusNotifyDemoCode.java, v 0.1 2022‐06-27 14:39 wqy01239909 Exp $
 */
public class RegistrationStatusNotifyDemoCode {

    private static final Logger logger = LoggerFactory
        .getLogger(RegistrationStatusNotifyDemoCode.class);

    public static void main(String[] args) throws IOException {
        RegistrationStatusNotifyDemoCode demoCode = new RegistrationStatusNotifyDemoCode();
        // get DefaultAPSHttpReceiveListener（If you have customization logic, you can self-implement APSHttpReceiveListener）
        APSHttpReceiveListener listener = DefaultAPS.getInstance().getApsHttpReceiveListener();

        demoCode.receiveRegistrationStatusNotify(listener,
            demoCode.buildHttpReceive(listener.getApsSettings()),
            demoCode.buildDefaultRegistrationStatusNotifyCallback());

    }

    public void receiveRegistrationStatusNotify(APSHttpReceiveListener listener,
                                                HttpReceive httpReceive,
                                                HttpReceiveCallback httpReceiveCallback) {
        try {

            // call onReceive method
            HttpReceiveResponse httpReceiveResponse = listener.onReceive(httpReceive,
                httpReceiveCallback);
            logger.debug("response={}", httpReceiveResponse);

            //TODO write responseHeaders and responseContent back to the Http response to AlipayPlus.

        } catch (IOException e) {

            logger.error("[IOException occurred] stackError={}", e.getMessage());

            // TODO: Handling IOException in ACQ

        } catch (AlipayPlusApiException e) {

            logger.error("[AlipayPlusApiException occurred] stackError={}", e.getMessage());

            // TODO: Handle AlipayPlusApiException according to error messages
            // TODO: compose response according to errorCode to AlipayPlus

        } catch (Exception e) {
            // unexpected/unknown exception occurs during the whole prepare process
            logger.error("[Unknown or Unexpected Exception occurred] stackError={}",
                e.getMessage());
            // TODO Handle UnknownException
            // TODO compose unknown response to AlipayPlus

        }

    }

    /**
     * ACQ build Callback which deals with registration status notify
     * And assemble the response returned to AlipayPlus
     *
     * Note: If you received the notification successfully, set the value of result.resultStatus to S and return an HTTP status code of 200
     * to indicate that your server received the call. Otherwise, Alipay+ takes the notification delivery as unsuccessful and retries the notification
     * sending up to 7 times. The intervals between two consecutive times are as follows: 2min, 10min, 10min, 1h, 2h, 6h, and 15h.
     * @return
     */
    private DefaultRegistrationStatusNotifyCallback buildDefaultRegistrationStatusNotifyCallback() {

        return new DefaultRegistrationStatusNotifyCallback() {
            @Override
            public HttpReceiveResponse onCompleted(MerchantRegistrationStatusNotifyRequest request) {
                logger.debug("[onCompleted] request={}", request);
                // indicates that all the MPPs have returned the registration result.
                return new HttpReceiveResponse(Result.S);
            }

            @Override
            public HttpReceiveResponse onPending(MerchantRegistrationStatusNotifyRequest request) {
                logger.debug("[onPending] request={}", request);
                // indicates that at least one MPP has not returned the registration result.
                // TODO: The ACQP can check the value of pspResgitrationResultList for the registration status of each MPP.
                return new HttpReceiveResponse(Result.S);
            }
        };
    }

    /**
     * 1、ACQ provides a Web service to receive HTTP requests from AlipayPlus
     * 2、ACQ convert HTTP requests to HttpReceive
     *
     * @return http notify object
     * @throws IOException throws if io exception happens
     * @param apsSettings
     */
    private HttpReceive buildHttpReceive(APSSettings apsSettings) throws IOException {
        //TODO: Request header from AlipayPlus notify
        Map<String, String> hashMap = new HashMap<>();
        hashMap.put(CommonConstants.CONTENT_TYPE_HEADER, "application/json; charset=UTF-8");
        hashMap.put(CommonConstants.REQ_SIGN_HEADER, "test_sign");
        hashMap.put(CommonConstants.REQ_TIME_HEADER, DateUtil.getCurISO8601Time());
        hashMap.put(CommonConstants.CLIENT_ID_HEADER, apsSettings.getClientId());
        HttpReceive httpReceive = new HttpReceive();
        httpReceive.setReceiveRequestHeader(hashMap);
        String reqBody = "{\n"
                         + "  \"registrationRequestId\": \"2020091811058602000006001420000\",\n"
                         + "  \"referenceMerchantId\": \"2188120000190000\",\n"
                         + "  \"referenceStoreId\": \"340000\",\n" + "  \"registrationResult\": {\n"
                         + "    \"registrationStatus\": \"COMPLETED\"\n" + "  },\n"
                         + "  \"pspRegistrationResultList\": [\n" + "    {\n"
                         + "      \"productCodes\": [\n" + "        \"CASHIER_PAYMENT\",\n"
                         + "        \"IN_STORE_PAYMENT\"\n" + "      ],\n"
                         + "      \"pspName\": \"ALIPAY_CN\",\n"
                         + "      \"registrationResult\": {\n"
                         + "        \"registrationStatus\": \"APPROVED\"\n" + "      }\n"
                         + "    }\n" + "  ]\n" + "}";
        httpReceive.setReceiveReqBody(reqBody.getBytes(CommonConstants.DEFAULT_CHARSET));
        httpReceive.setRequestURI("/registration/notify");
        httpReceive.setReceiveType(HttpReceiveType.REGISTRATION_STATUS_NOTIFY);
        return httpReceive;
    }
}