/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.InetSocketAddress;
import java.util.HashMap;
import java.util.Map;

import com.alipay.ap.constants.CommonConstants;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultRegistrationStatusNotifyCallback;
import com.alipay.ap.enums.HttpReceiveType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.model.HttpReceive;
import com.alipay.ap.model.Result;
import com.alipay.ap.receive.APSHttpReceiveListener;
import com.alipay.ap.request.MerchantRegistrationStatusNotifyRequest;
import com.alipay.ap.response.HttpReceiveResponse;
import com.sun.net.httpserver.Headers;
import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import com.sun.net.httpserver.HttpServer;
import com.sun.net.httpserver.spi.HttpServerProvider;

/**
 * If the ACQP wants to receive notifications from Alipay+ about registration results,
 * the ACQP needs to integrate the notifyRegistrationStatus API. Each time Alipay+ receives a notification from the MPP,
 * Alipay+ will notify the ACQP of the registration result by calling the API.
 * (refer to doc: [Acquiring Partner] https://docs.alipayplus.com/alipayplus/)
 * @author wqy01239909
 * @version $Id: RegistrationStatusNotifyLocalTestDemoCode.java, v 0.1 2022‐06-27 14:39 wqy01239909 Exp $
 */
public class RegistrationStatusNotifyLocalTestDemoCode implements HttpHandler {

    private static final Logger logger = LoggerFactory
        .getLogger(RegistrationStatusNotifyLocalTestDemoCode.class);

    public static void main(String[] args) throws IOException {

        HttpServerProvider provider = HttpServerProvider.provider();
        HttpServer httpServer;
        // Listen on localhost:9500, accept 100 requests at most
        httpServer = provider.createHttpServer(new InetSocketAddress("localhost", 9500), 100);
        httpServer.createContext("/", new RegistrationStatusNotifyLocalTestDemoCode());
        httpServer.setExecutor(null);
        httpServer.start();
        logger.info("[http server started] {}", httpServer.getAddress());

    }

    @Override
    public void handle(HttpExchange httpExchange) {
        logger.debug("[http receive for registrationStatus notification]");

        try {

            RegistrationStatusNotifyLocalTestDemoCode demoCode = new RegistrationStatusNotifyLocalTestDemoCode();

            APSHttpReceiveListener listener = DefaultAPS.getInstance().getApsHttpReceiveListener();

            HttpReceiveResponse httpReceiveResponse = listener.onReceive(
                buildHttpReceive(httpExchange, HttpReceiveType.REGISTRATION_STATUS_NOTIFY),
                demoCode.buildDefaultRegistrationStatusNotifyCallback());
            logger.debug("response={}", httpReceiveResponse);
            //TODO write responseHeaders and responseContent back to the Http response to AlipayPlus.
            writeResponseToAlipayPlus(httpExchange, httpReceiveResponse);

        } catch (IOException e) {

            logger.error("[IOException occurred] stackError={}", e.getMessage());

            // TODO: Handling IOException in ACQ

        } catch (AlipayPlusApiException e) {

            logger.error("[AlipayPlusApiException occurred] stackError={}", e.getMessage());

            // TODO: Handle AlipayPlusApiException according to error messages
            // TODO: compose response according to errorCode to AlipayPlus

        } catch (Exception e) {
            // unexpected/unknown exception occurs during the whole prepare process
            logger.error("[Unknown or Unexpected Exception occurred] stackError={}",
                e.getMessage());
            // TODO Handle UnknownException
            // TODO compose unknown response to AlipayPlus

        } finally {
            if (httpExchange != null) {
                httpExchange.close();
            }
        }

    }

    private DefaultRegistrationStatusNotifyCallback buildDefaultRegistrationStatusNotifyCallback() {

        return new DefaultRegistrationStatusNotifyCallback() {
            @Override
            public HttpReceiveResponse onCompleted(MerchantRegistrationStatusNotifyRequest request) {
                logger.debug("[onCompleted] request={}", request);
                // indicates that all the MPPs have returned the registration result.
                return new HttpReceiveResponse(Result.S);
            }

            @Override
            public HttpReceiveResponse onPending(MerchantRegistrationStatusNotifyRequest request) {
                logger.debug("[onPending] request={}", request);
                // indicates that at least one MPP has not returned the registration result.
                // TODO: The ACQP can check the value of pspResgitrationResultList for the registration status of each MPP.
                return new HttpReceiveResponse(Result.S);
            }
        };
    }

    /**
     * build http receive object
     *
     * @param httpExchange http info {@link HttpExchange}
     * @param httpReceiveType notify type
     * @return http notify object
     * @throws IOException throws if io exception happens
     */
    public static HttpReceive buildHttpReceive(HttpExchange httpExchange,
                                               HttpReceiveType httpReceiveType) throws IOException {
        Map<String, String> hashMap = new HashMap<>(); //Request header from Alipay notify
        Headers requestHeaders = httpExchange.getRequestHeaders();
        hashMap.put(CommonConstants.CONTENT_TYPE_HEADER,
            requestHeaders.getFirst(CommonConstants.CONTENT_TYPE_HEADER));
        hashMap.put(CommonConstants.CLIENT_ID_HEADER,
            requestHeaders.getFirst(CommonConstants.CLIENT_ID_HEADER));
        hashMap.put(CommonConstants.REQ_TIME_HEADER, requestHeaders.getFirst(CommonConstants.REQ_TIME_HEADER));
        hashMap.put(CommonConstants.REQ_SIGN_HEADER, requestHeaders.getFirst(CommonConstants.REQ_SIGN_HEADER));

        HttpReceive httpReceive = new HttpReceive();
        httpReceive.setReceiveRequestHeader(hashMap);
        httpReceive.setRequestURI(httpExchange.getRequestURI().toString());
        httpReceive.setReceiveReqBody(IOUtils.toByteArray(httpExchange.getRequestBody()));
        httpReceive.setReceiveType(httpReceiveType);

        return httpReceive;
    }

    /**
     * write response back to a+
     *
     * @param httpExchange http info {@link HttpExchange}
     * @param httpReceiveResponse notify response {@link HttpReceiveResponse}
     * @throws IOException
     */
    public static void writeResponseToAlipayPlus(HttpExchange httpExchange,
                                                 HttpReceiveResponse httpReceiveResponse) throws IOException {
        Map<String, String> responseHeaders = httpReceiveResponse.getResponseHeaders();
        byte[] responseContent = httpReceiveResponse.getBodyContent();

        // 1、httpResponse.setHeaders(responseHeaders)
        for (Map.Entry<String, String> entry : responseHeaders.entrySet()) {
            httpExchange.getResponseHeaders().set(entry.getKey(), entry.getValue());
        }

        String response = new String(responseContent, CommonConstants.DEFAULT_CHARSET);
        httpExchange.sendResponseHeaders(HttpURLConnection.HTTP_OK,
            response.getBytes(CommonConstants.DEFAULT_CHARSET).length);

        // 2、httpResponse.write(responseContent);
        try (OutputStream responseBody = httpExchange.getResponseBody();
                OutputStreamWriter writer = new OutputStreamWriter(responseBody,
                    CommonConstants.DEFAULT_CHARSET)) {
            writer.write(response);
        }
    }

}