/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example.schedule;

import java.io.IOException;
import java.util.concurrent.TimeUnit;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultCancelPaymentCallback;
import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.constants.CommonConstants;
import com.alipay.ap.enums.BizType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.job.APSScheduleJobExecutor;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.CancelPaymentRequest;
import com.alipay.ap.request.ScheduleJobRequest;
import com.alipay.ap.response.CancelPaymentPayResponse;
import com.alipay.ap.utils.DateUtil;
import com.alipay.ap.utils.GsonUtils;
import com.alipay.ap.utils.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author wangpeng
 * @version $Id: CancelPaymentWithScheduleDemoCode.java, v 0.1 2022‐06-30 10:47 wangpeng Exp $
 */
public class CancelPaymentWithScheduleDemoCode {

    private static final Logger logger             = LoggerFactory
        .getLogger(CancelPaymentWithScheduleDemoCode.class);

    private static final String PAYMENT_REQUEST_ID = StringUtil.genRequestID("payment");

    public static void main(String[] args) throws AlipayPlusApiException {
        //inject implementation of ScheduleJobSupport
        APSScheduleJobExecutor.getInstance()
            .setScheduleJobSupport(new ScheduleJobSupportInMemDemo());

        //inject implementation of Callback
        APSScheduleJobExecutor.getInstance()
            .setScheduledCancelCallback(new CancelPaymentCallbackWithSchedule());

        //inject implementation of APSClient
        APSScheduleJobExecutor.getInstance().setApsClient(DefaultAPS.getInstance().getApsClient());

        //start JobExecutor
        APSScheduleJobExecutor.getInstance().startJobExecutor();

        CancelPaymentWithScheduleDemoCode demo = new CancelPaymentWithScheduleDemoCode();

        demo.cancelPayment(DefaultAPS.getInstance().getApsClient(),
            demo.buildCancelPaymentRequest(), buildCancelPaymentCallBack());
    }

    private void cancelPayment(APSClient apsClient, CancelPaymentRequest cancelPaymentRequest,
                               HttpSendCallback callback) {

        try {
            //Calling ApsClient.execute to cancel a payment
            apsClient.execute(cancelPaymentRequest, callback);
        } catch (IOException e) {
            // Below scenes raise the IOException when calling ApsClient.execute
            // #1 Calling A+ api when occurs the http connection issue
            // #2 Calling A+ api when occurs client protocol issue

            logger.error("[IOException occurred] cancelPaymentRequest={},stackError={}",
                cancelPaymentRequest, e.getMessage());

            // TODO: Handling IOException in ACQ
        } catch (AlipayPlusApiException e) {
            // Below scenes raise the AlipayPlusApiException when calling ApsClient.execute
            // #1 Calling A+ api when respond non 200 http status code. Such as 400/404/500...
            // #2 Calling A+ api when occurs the signature verification failure

            logger.error(
                "[AlipayPlusApiException occurred] cancelPaymentRequest={},httpCode={},errorCode={},errorMsg={},stackError={}",
                cancelPaymentRequest, e.getCode(), e.getErrCode(), e.getErrMsg(), e.getMessage());

            // TODO: Handling AlipayPlusApiException in ACQ

        } catch (Exception e) {
            // unexpected/unknown exception occurs during the whole payment process
            logger.error(
                "[Unknown or Unexpected Exception occurred] cancelPaymentRequest={},stackError={}",
                cancelPaymentRequest, e.getMessage());

            // TODO: Handling Unknown/Unexpected Exception in ACQ

        }
    }

    private static HttpSendCallback buildCancelPaymentCallBack() {

        return new DefaultCancelPaymentCallback() {

            @Override
            public void onCancelPaymentSuccess(APSClient apsClient, CancelPaymentRequest request,
                                               CancelPaymentPayResponse response) {

                logger.debug(
                    "[onCancelPaymentSuccess][isJobSchedule={}] cancelPaymentRequest={},cancelPaymentPayResponse={}",
                    request.isJobSchedule(), request, response);

                /**
                 * ACQ can decide whether need to register a retry asynchronously for execution when UNKNOWN status occur.See onCancelPaymentUnknown method for referrence.
                 */
                /** implement the callback if this request is executed synchronously   */
                // 1. TODO Take actions for payment cancel success in ACQ on synchronous execution scene.
                // 2. TODO Response the result to upstream system synchronously.
                /** The merchant can reinitiate a new payment request */

            }

            @Override
            public void onCancelPaymentFail(APSClient apsClient, CancelPaymentRequest request,
                                            CancelPaymentPayResponse response) {

                logger.debug(
                    "[onCancelPaymentFail][isJobSchedule={}] cancelPaymentRequest={},cancelPaymentPayResponse={}",
                    request.isJobSchedule(), request, response);

                /**
                 * ACQ can decide whether need to register a retry asynchronously for execution when UNKNOWN status occur.See onCancelPaymentUnknown method for referrence.
                 */
                /** implement the callback if this request is executed synchronously   */
                // 1. TODO Take actions according to the error code in result.resultCode.
                // 2. TODO If the result.resultCode value returns as CANCEL_WINDOW_EXCEED，please call the refund interface.
                // 3. TODO Response the result to upstream system synchronously.

            }

            @Override
            public void onCancelPaymentUnknown(APSClient apsClient, CancelPaymentRequest request,
                                               CancelPaymentPayResponse response) throws Exception {

                logger.debug(
                    "[onCancelPaymentUnknown][isJobSchedule={}] cancelPaymentRequest={},cancelPaymentPayResponse={}",
                    request.isJobSchedule(), request, response);

                /** implement the callback if this request is executed synchronously */
                // TODO Take actions for payment cancel unknown.
                /** register the schedule for cancel retry asynchronously */
                APSScheduleJobExecutor.instance.put(new ScheduleJobRequest(
                    request.getPaymentRequestId(), BizType.CANCEL.name(),
                    GsonUtils.toJSONString(request),
                    DateUtil.triggerTime(APSScheduleJobExecutor.instance.getApsClient()
                        .getApsSettings().getCancelInterval()[CommonConstants.NUMBER_ZERO],
                        TimeUnit.SECONDS)));
                // TODO Response the result to upstream system synchronously.
            }
        };

    }

    private CancelPaymentRequest buildCancelPaymentRequest() {
        CancelPaymentRequest cancelPaymentRequest = new CancelPaymentRequest();
        cancelPaymentRequest.setPaymentRequestId(PAYMENT_REQUEST_ID);
        return cancelPaymentRequest;
    }

}