/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example.schedule;

import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.concurrent.TimeUnit;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultCashierPaymentCallback;
import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.constants.CommonConstants;
import com.alipay.ap.enums.BizType;
import com.alipay.ap.enums.OsType;
import com.alipay.ap.enums.PaymentMethodType;
import com.alipay.ap.enums.PresentmentMode;
import com.alipay.ap.enums.TerminalType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.job.APSScheduleJobExecutor;
import com.alipay.ap.model.Address;
import com.alipay.ap.model.Amount;
import com.alipay.ap.model.Env;
import com.alipay.ap.model.Merchant;
import com.alipay.ap.model.Order;
import com.alipay.ap.model.PaymentFactor;
import com.alipay.ap.model.PaymentMethod;
import com.alipay.ap.model.SettlementStrategy;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.request.CashierPaymentPayRequest;
import com.alipay.ap.request.InquiryPaymentRequest;
import com.alipay.ap.request.ScheduleJobRequest;
import com.alipay.ap.response.CashierPaymentResponse;
import com.alipay.ap.utils.DateUtil;
import com.alipay.ap.utils.GsonUtils;
import com.alipay.ap.utils.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * The demo code mainly shows how to use the API correctly and the specific implementation needs to be performed by merchants.
 *
 * @author wqy01239909
 * @version $Id: CashierPayWithScheduleDemoCode.java, v 0.1 2022‐06-24 15:30 wqy01239909 Exp $
 */
public class CashierPayWithScheduleDemoCode {

    private static final int    RETRY_COUNT = 3;

    private static final Logger logger      = LoggerFactory
        .getLogger(CashierPayWithScheduleDemoCode.class);

    public static void main(String[] args) {

        //inject implementation of ScheduleJobSupport
        APSScheduleJobExecutor.getInstance()
            .setScheduleJobSupport(new ScheduleJobSupportInMemDemo());

        //inject implementation of Callback
        APSScheduleJobExecutor.getInstance()
            .setScheduledInquiryCallback(new InquiryPaymentCallbackWithSchedule());

        //inject implementation of APSClient
        APSScheduleJobExecutor.getInstance().setApsClient(DefaultAPS.getInstance().getApsClient());

        //start JobExecutor
        APSScheduleJobExecutor.getInstance().startJobExecutor();

        CashierPayWithScheduleDemoCode demo = new CashierPayWithScheduleDemoCode();

        demo.pay(DefaultAPS.getInstance().getApsClient(), demo.buildCashierPaymentPayRequest(),
            demo.buildCashierPaymentCallback());

    }

    private void pay(APSClient apsClient, CashierPaymentPayRequest CashierPaymentPayRequest,
                     HttpSendCallback cashierPaymentCallback) {

        try {

            //TODO: ACQ handling the cashier pay request before calling A+ api

            //Calling ApsClient.execute to initiate a cashier payment
            apsClient.execute(CashierPaymentPayRequest, cashierPaymentCallback);

        } catch (IOException e) {

            // Below scenes raise the IOException when calling ApsClient.execute
            // #1 Calling A+ api when occurs the http connection issue
            // #2 Calling A+ api when occurs client protocol issue

            logger.error("[IOException occurred] CashierPaymentPayRequest]={},stackError={}",
                CashierPaymentPayRequest, e.getMessage());

            // TODO: Handling IOException in ACQ

        } catch (AlipayPlusApiException e) {

            // Below scenes raise the AlipayPlusApiException when calling ApsClient.execute
            // #1 Calling A+ api when respond non 200 http status code. Such as 400/404/500...
            // #2 Calling A+ api when occurs the signature verification failure

            logger.error(
                "[AlipayPlusApiException occurred] CashierPaymentPayRequest={},httpCode={},errorCode={},errorMsg={},stackError={}",
                CashierPaymentPayRequest, e.getCode(), e.getErrCode(), e.getErrMsg(),
                e.getMessage());

            // TODO: Handling AlipayPlusApiException in ACQ

        } catch (Exception e) {

            // unexpected/unknown exception occurs during the whole payment process
            logger.error(
                "[Unknown or Unexpected Exception occurred] CashierPaymentPayRequest={},stackError={}",
                CashierPaymentPayRequest, e.getMessage());

            // TODO: Handling Unknown/Unexpected Exception in ACQ

        }
    }

    private HttpSendCallback buildCashierPaymentCallback() {

        return new DefaultCashierPaymentCallback() {
            @Override
            public void onPaymentFail(APSClient apsClient, CashierPaymentPayRequest request,
                                      CashierPaymentResponse response) throws Exception {

                logger.debug(
                    "[onPaymentFail] CashierPaymentPayRequest={},cashierPaymentResponse={}",
                    request, response);

                //TODO: Take actions for PAYMENT_FAILURE in ACQ

                //TODO Take actions according to the error message in result.resultCode.

            }

            @Override
            public void onPaymentInProcess(APSClient apsClient, CashierPaymentPayRequest request,
                                           CashierPaymentResponse response) throws Exception {

                logger.debug(
                    "[onPaymentInProcess] CashierPaymentPayRequest={},cashierPaymentResponse={}",
                    request, response);

                // TODO: Take action for PAYMENT_IN_PROCESS in ACQ
                // TODO: On Server Side
                // TODO: #1 Listen the A+ PaymentNotify --> PaymentNotifyDemo
                // TODO: #2 Calling the A+ inquiryPayment asynchronously
                // Sample for calling inquiryPayment asynchronously
                InquiryPaymentRequest inquiryPaymentRequest = buildInquiryPaymentRequest(
                    request.getPaymentRequestId());
                APSScheduleJobExecutor.instance
                    .put(new ScheduleJobRequest(inquiryPaymentRequest.getPaymentRequestId(),
                        BizType.INQUIRY.toString(), GsonUtils.toJSONString(inquiryPaymentRequest),
                        DateUtil.triggerTime(
                            APSScheduleJobExecutor.instance.getApsClient().getApsSettings()
                                .getInquiryIntervalForCashierpay()[CommonConstants.NUMBER_ZERO],
                            TimeUnit.SECONDS)));
                // TODO: #3 Handle response to upstream system

                // TODO: On Client Side
                // TODO: #1 Open the A+ checkout page
                // TODO: #2 Handle the redirection from the MPP

            }

            @Override
            public void onPaymentUnknown(APSClient apsClient, CashierPaymentPayRequest request,
                                         CashierPaymentResponse response) throws Exception {

                logger.debug(
                    "[onPaymentUnknown] CashierPaymentPayRequest={},cashierPaymentResponse={}",
                    request, response);

                //TODO: Take actions for result.resultStatus = U && result.resultCode != PAYMENT_IN_PROCESS
            }

            @Override
            public void onPaymentSuccess(APSClient apsClient, CashierPaymentPayRequest request,
                                         CashierPaymentResponse response) throws Exception {

                logger.debug(
                    "[onPaymentSuccess] CashierPaymentPayRequest={},cashierPaymentResponse={}",
                    request, response);

                // This is for the case below.
                // User completed the payment successfully in cashier payment and ACQ invoke the pay API again with same params, the resultStatus is returned as S.
                //TODO: Take actions for PAYMENT_SUCCESS in ACQ
            }
        };

    }

    private CashierPaymentPayRequest buildCashierPaymentPayRequest() {
        CashierPaymentPayRequest alipayPayRequest = new CashierPaymentPayRequest();
        // The unique ID that is assigned by the ACQP to identify a payment order.
        alipayPayRequest.setPaymentRequestId(StringUtil.genRequestID("payment"));

        Amount paymentAmount = new Amount();
        paymentAmount.setCurrency("PHP");
        // The value of the amount as a natural number. By default, the value of this parameter is in the smallest currency unit.
        // For example, if the currency is USD and the amount is $1.00, set the value of this parameter to 100; or if the currency is JPY and the amount is ￥1, set the value of this parameter to 1.
        paymentAmount.setValue(30000);
        alipayPayRequest.setPaymentAmount(paymentAmount);

        PaymentFactor paymentFactor = new PaymentFactor();
        // fixed value: false
        paymentFactor.setInStorePayment(false);
        // fixed value: true
        paymentFactor.setCashierPayment(true);
        // fixed value: UNIFIED
        paymentFactor.setPresentmentMode(PresentmentMode.UNIFIED);
        alipayPayRequest.setPaymentFactor(paymentFactor);

        Order order = new Order();
        // The unique ID that is assigned by the merchant to identify an order.
        // The ID is used to display the user's purchase record and to track follow-up operations, such as customer complaints and disputes.
        order.setReferenceOrderId("102775765075669");
        order.setOrderDescription("SHOES");

        Merchant merchant = new Merchant();
        merchant.setMerchantMCC("5411");
        // The unique ID that is assigned by the ACQP to identify a merchant.
        // The value of this parameter must be the same as that of the merchantInfo.referenceMerchantId parameter that you specified in the registration API request.
        merchant.setReferenceMerchantId("M0000000001");
        merchant.setMerchantName("UGG");
        Address merchantAddress = new Address();
        merchantAddress.setRegion("PH");
        merchantAddress.setCity("xxx");
        merchant.setMerchantAddress(merchantAddress);
        order.setMerchant(merchant);

        Amount orderAmount = new Amount();
        orderAmount.setCurrency("PHP");
        // The value of the amount as a natural number. By default, the value of this parameter is in the smallest currency unit.
        orderAmount.setValue(30000);
        order.setOrderAmount(orderAmount);
        alipayPayRequest.setOrder(order);

        PaymentMethod paymentMethod = new PaymentMethod();
        // fixed value: CONNECT_WALLET
        paymentMethod.setPaymentMethodType(PaymentMethodType.CONNECT_WALLET);
        alipayPayRequest.setPaymentMethod(paymentMethod);

        Env env = new Env();
        env.setTerminalType(TerminalType.APP);
        env.setOsType(OsType.IOS);

        order.setEnv(env);

        // The URL that is used to receive the payment result notification from Alipay+.
        alipayPayRequest.setPaymentNotifyUrl(
            "https://xmock.inc.alipay.net/api/Ipay/globalSite/automtion/paymentNotify.htm");
        // The URL where the user is redirected to after the payment order is completed.
        //Note: Required if the user needs to be redirected from the MPP's payment result page to a predetermined page, for example, the
        // merchant result page.
        alipayPayRequest.setPaymentRedirectUrl("https://www.gcash.com?param1=sl");

        // Required if the settlement currency is different from the payment currency that is specified for the paymentAmount.currency parameter.
        SettlementStrategy settlementStrategy = new SettlementStrategy();
        settlementStrategy.setSettlementCurrency("USD");// following the contract
        alipayPayRequest.setSettlementStrategy(settlementStrategy);

        // In this scenario, the default value of this parameter is 10 minutes.
        // If the ACQP wants the payment transaction to be completed before this time,then should pass the parameter which is shorter than the default expiration time.
        // The value follows the ISO 8601 standard format. For example, "2019-11-27T12:01:01+08:00".
        Calendar nowTime = Calendar.getInstance();
        nowTime.add(Calendar.MINUTE, 5);
        alipayPayRequest
            .setPaymentExpiryTime(DateUtil.getISODateTimeStr(new Date(nowTime.getTimeInMillis())));

        return alipayPayRequest;
    }

    private InquiryPaymentRequest buildInquiryPaymentRequest(String paymentRequestId) {

        InquiryPaymentRequest inquiryPaymentRequest = new InquiryPaymentRequest();
        inquiryPaymentRequest.setPaymentRequestId(paymentRequestId);

        return inquiryPaymentRequest;
    }
}