/**
 *   Legal Disclaimer
 *   Source Code or Sample Code not belongs to Alipay technological assets, partners agree to take full responsibility for any activities or actions under its use.
 *   Partners are authorised to use for Approved Purposes.
 *   "Approved Purpose" means, subject to any restrictions set out in the Participation Documents, (i) in relation to a Partner, as is necessary in order to facilitate that Partner’s participation in Alipay+ Core; and (ii) in relation to Alipay+, as is necessary in order to facilitate the provision of the Services (including the exercise of rights and performance of obligations under Participation Documents, managing AML/Fraud Risk, resolving any Disputes or facilitating the provision of Partner Products by other Partners).
 *   You should check our NDA for detail Legal T&C.
 */
package com.alipay.ap.example.schedule;

import java.io.IOException;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alipay.ap.DefaultAPS;
import com.alipay.ap.callbacks.DefaultRefundPaymentCallback;
import com.alipay.ap.callbacks.HttpSendCallback;
import com.alipay.ap.cfg.APSSettings;
import com.alipay.ap.constants.CommonConstants;
import com.alipay.ap.enums.BizType;
import com.alipay.ap.exception.AlipayPlusApiException;
import com.alipay.ap.job.APSScheduleJobExecutor;
import com.alipay.ap.model.Amount;
import com.alipay.ap.net.APSClient;
import com.alipay.ap.net.DefaultApacheHttpPostAPSClient;
import com.alipay.ap.request.RefundRequest;
import com.alipay.ap.request.ScheduleJobRequest;
import com.alipay.ap.response.RefundResponse;
import com.alipay.ap.utils.DateUtil;
import com.alipay.ap.utils.GsonUtils;
import com.alipay.ap.utils.StringUtil;

/**
 *
 * @author wangpeng
 * @version $Id: RefundPaymentWithScheduleDemoCode.java, v 0.1 2022‐06-27 16:24 wangpeng Exp $
 */
public class RefundPaymentWithScheduleDemoCode {

    private static final Logger logger             = LoggerFactory
        .getLogger(RefundPaymentWithScheduleDemoCode.class);

    private static final String PAYMENT_REQUEST_ID = StringUtil.genRequestID("payment");

    private static final String REFUND_REQUEST_ID  = StringUtil.genRequestID("refund");

    public static void main(String[] args) throws Exception {

        //inject implementation of ScheduleJobSupport
        APSScheduleJobExecutor.getInstance()
            .setScheduleJobSupport(new ScheduleJobSupportInMemDemo());

        //inject implementation of Callback which deals with inquiry payment response
        APSScheduleJobExecutor.getInstance()
            .setScheduledRefundCallback(new RefundPaymentCallbackWithSchedule());

        //inject implementation of APSClient
        APSScheduleJobExecutor.getInstance()
            .setApsClient(new DefaultApacheHttpPostAPSClient(new APSSettings()));

        //start JobExecutor
        APSScheduleJobExecutor.getInstance().startJobExecutor();

        RefundPaymentWithScheduleDemoCode demo = new RefundPaymentWithScheduleDemoCode();

        demo.refund(DefaultAPS.getInstance().getApsClient(), demo.buildRefundRequest(),
            buildRefundPaymentCallBack());
    }

    public void refund(APSClient apsClient, RefundRequest refundRequest,
                       HttpSendCallback callback) throws AlipayPlusApiException {

        try {
            apsClient.execute(refundRequest, callback);
        } catch (IOException e) {
            // Below scenes raise the IOException when calling ApsClient.execute
            // #1 Calling A+ api when occurs the http connection issue
            // #2 Calling A+ api when occurs client protocol issue

            logger.error("[IOException occurred] refundRequest={},stackError={}", refundRequest,
                e.getMessage());

            // TODO: Handling IOException in ACQ
        } catch (AlipayPlusApiException e) {
            // Below scenes raise the AlipayPlusApiException when calling ApsClient.execute
            // #1 Calling A+ api when respond non 200 http status code. Such as 400/404/500...
            // #2 Calling A+ api when occurs the signature verification failure

            logger.error(
                "[AlipayPlusApiException occurred] refundRequest={},httpCode={},errorCode={},errorMsg={},stackError={}",
                refundRequest, e.getCode(), e.getErrCode(), e.getErrMsg(), e.getMessage());

            // TODO: Handling AlipayPlusApiException in ACQ

        } catch (Exception e) {
            // unexpected/unknown exception occurs during the whole payment process
            logger.error(
                "[Unknown or Unexpected Exception occurred] refundRequest={},stackError={}",
                refundRequest, e.getMessage());

            // TODO: Handling Unknown/Unexpected Exception in ACQ

        }
    }

    private RefundRequest buildRefundRequest() {

        RefundRequest refundRequest = new RefundRequest();
        Amount refundAmount = new Amount();
        refundAmount.setCurrency("PHP");
        refundAmount.setValue(30000);
        refundRequest.setRefundAmount(refundAmount);
        refundRequest.setRefundReason("refund reason");
        refundRequest.setRefundRequestId(REFUND_REQUEST_ID);
        refundRequest.setPaymentRequestId(PAYMENT_REQUEST_ID);

        return refundRequest;
    }

    private static HttpSendCallback buildRefundPaymentCallBack() {

        return new DefaultRefundPaymentCallback() {

            @Override
            public void onRefundPaymentSuccess(APSClient apsClient, RefundRequest request,
                                               RefundResponse response) {
                logger.debug(
                    "[onRefundPaymentSuccess][isJobSchedule={}] refundPaymentRequest={},refundPaymentPayResponse={}",
                    request.isJobSchedule(), request, response);

                /**
                 * ACQ can decide whether need to register a retry asynchronously for execution when UNKNOWN status occur.See onRefundPaymentUnknown method for referrence.
                 */
                /** implement the callback if this request is executed synchronously   */
                // 1. TODO Update the order status in your side to the value such as refunded.
                // 2. TODO Response the result to upstream system synchronously.
            }

            @Override
            public void onRefundPaymentFail(APSClient apsClient, RefundRequest request,
                                            RefundResponse response) {
                logger.debug(
                    "[onPaymentRefundFail][isJobSchedule={}] refundPaymentRequest={},refundPaymentPayResponse={}",
                    request.isJobSchedule(), request, response);

                /**
                 * ACQ can decide whether need to register a retry asynchronously for execution when UNKNOWN status occur.See onRefundPaymentUnknown method for referrence.
                 */
                /** implement the callback if this request is executed asynchronously   */
                // 1. TODO Take actions according to the error message in the result.resultMessage parameter.
                // 2. TODO Response the result to upstream system synchronously.
            }

            @Override
            public void onRefundPaymentUnknown(APSClient apsClient, RefundRequest request,
                                               RefundResponse response) throws Exception {
                logger.debug(
                    "[onRefundPaymentUnknown][isJobSchedule={}] refundPaymentRequest={},refundPaymentPayResponse={}",
                    request.isJobSchedule(), request, response);

                /** implement the callback if this request is executed synchronously */
                // TODO Take actions for refund payment unknown.
                /** register the schedule for refund retry asynchronously
                 *  Retry the same request until getting the response. Ensure that the parameters must be the same with the previous one */
                APSScheduleJobExecutor.instance.put(new ScheduleJobRequest(
                    request.getPaymentRequestId(), BizType.REFUND.toString(),
                    GsonUtils.toJSONString(request),
                    DateUtil.triggerTime(
                        apsClient.getApsSettings().getRefundInterval()[CommonConstants.NUMBER_ZERO],
                        TimeUnit.SECONDS)));
                // TODO Response the result to upstream system synchronously.
            }
        };
    }

}